# -*- coding: utf-8 -*-
"""
usage:
   xdxsmi vpuutil [options]

options:
    -h, --help                    print this help message
    -f, --filename=file           redirect query output to a specified file
    -i, --index=val               target gpu's vpu ultilization, start from 0
    -l, --loop=sec                set look up interval, whose unit is second
"""

# from typing import Dict
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *

# (encRev: int, encExc: int, decRev: int, decExc: int)->string
def makeOneMsg(encRev, encExc, decRev, decExc):
    outStr = str("")
    if(encRev == -1 or encExc == -1 or decRev == -1 or decExc == -1):  #只有任意一个读不出来就认为没有
        outStr = "N/A\n"
    else:
        outStr = 'encode-vcmd reserved %dfps, excuted %dfps; decode-vcmd reserved %dfps, excuted %dfps\n' % (encRev, encExc, 
            decRev, decExc)
    return outStr

# (theIdx: int)->string
def makeHeader(theIdx):
    outStr = str("")
    if(theIdx >=0):
        outStr += '=====================Vpu utilization of index %02d gpu=====================\n' % theIdx
    else:
        outStr += "==============================vpu utilization==============================\n"
    return outStr

# (xdx_smi: XdxSmi, theIdx: int)->string
def makeSingleBody(xdx_smi, theIdx):
    outStr = str("")
    if(theIdx >= 0):
        encRev, encExc, decRev, decExc = xdxml_device_get_vpu_utlz(xdx_smi.handle[theIdx])
        outStr += makeOneMsg(encRev, encExc, decRev, decExc)
    else:  #没指定gpu则会在前面加上编号，哪怕只有一个
        i = 0
        for devHd in xdx_smi.handle:
            encRev, encExc, decRev, decExc = xdxml_device_get_vpu_utlz(xdx_smi.handle[theIdx])
            outStr +='vpu(%02d): '%i + makeOneMsg(encRev, encExc, decRev, decExc)
            i += 1
    return outStr

# (xdx_smi: XdxSmi, args: Dict)->None
def handle(xdx_smi, args):
    theIdx = -1
    numDev = len(xdx_smi.handle)
    if(numDev == 0):
        print("No devices were found")
        return
  
    try:
        if(args['--index'] == None):
            theIdx = -1
        elif(args['--index'] != None and int(args['--index']) >= 0 and int(args['--index']) < numDev):
            theIdx = int(args['--index'])
        elif(args['--index'] != None and int(args['--index']) >= 0 and int(args['--index']) >= numDev):
            print('the gpu of specified index %d is not found' % (theIdx))
            return
        else:
            print("invalid index, must be a integer and starts from 0!")
            return
    except:
        print("No devices were found")
        return

    sleepSec = -1
    if(args['--loop'] != None):
        try:
            if(float(args['--loop']) > 0):
                sleepSec = float(args["--loop"])
            else:
                raise ValueError("Invalid combination of input arguments. Please run 'xdxsmi -h' for help.")
        except: 
            print("Invalid combination of input arguments. Please run 'xdxsmi -h' for help.")
            exit()

    outFile = None
    if(args['--filename'] != None):
        outFile = open(args['--filename'], "w")

    if(sleepSec == -1):
        outStr = makeHeader(theIdx) + makeSingleBody(xdx_smi, theIdx)
        if(outFile != None):
            outFile.write(outStr)
            outFile.close()
        else:
            print(outStr)
    else:
        if(outFile != None):
            outFile.write(makeHeader(theIdx))
        else:
            print(makeHeader(theIdx)), #有换行符，在同一行打印
        try:
            i = 0
            secs = 0
            while True:
                # xdx_smi =  XdxSmi()
                if(outFile != None):
                    outFile.write('loop %d in %d seconds:\n'%(i,secs) + makeSingleBody(xdx_smi, theIdx))
                else:
                    print('loop %d in %d seconds:\n'%(i,secs) + makeSingleBody(xdx_smi, theIdx)), #有换行符，在同一行打印
                i += 1
                time.sleep(sleepSec)
                secs += sleepSec
        except KeyboardInterrupt:
            if(outFile != None):
                outFile.close()
            else:
                print("")   #最后再换一行，好看点


if __name__ == "__main__":
    print(docopt(__doc__))