#!/bin/bash
set -x

: ${DEF_CONFIG:='ubuntu16.04-config-4.15.0-36-generic.cfg'}
: ${ROOTCMD:=}
: ${BUILD_BASE_DIR:='BUILD'}
: ${INSTALL_BASE_DIR:='INSTALL'}

XDXKMD_SRC=`pwd`
USE_CONFIG="local"

PREFIX="$XDXKMD_SRC/INSTALL/x86_64"
BUILD_DIR=${BUILD_BASE_DIR}/x86_64
INSTALL_DIR=${INSTALL_BASE_DIR}/x86_64

bbh_init_kernel() {
    KERNEL_NAME="linux"
    KERNEL_DEPS='fakeroot ccache bc kernel-package'

	KERNEL_ARCH=amd64
	KERNEL_CROSS_COMPILE=x86_64-unknown-linux-gnu
	KERNEL_CC=gcc
	
	cd $XDXKMD_SRC/linux
	KERNEL_TAGS=-deli-`git describe --tags --dirty --long`
	echo "$KERNEL_TAGS"
}

prep_kernel_script() {
    sh -ex <<-END_PREP_SCRIPT
	cd $XDXKMD_SRC
	rm -rf $BUILD_DIR/linux
	mkdir -p $BUILD_DIR/linux
	cp -rs $XDXKMD_SRC/linux $BUILD_DIR
	touch $BUILD_DIR/linux/REPORTING-BUGS
END_PREP_SCRIPT
}

update_kernel_files() {
    sh -ex <<-END_PREP_SCRIPT
	cd $XDXKMD_SRC
	cp -rs $XDXKMD_SRC/linux $BUILD_DIR >/dev/null 2>&1
END_PREP_SCRIPT
}

bbh_build_kernel() {
    local path config mod cik

    if [ -z $USE_CONFIG ]; then
        config="$XDXKMD_SRC/utils/build_script/kernel_config/$DEF_CONFIG"
    elif [ "$USE_CONFIG" = "local" ]; then
        config=/boot/config-`uname -r`
    else
        config="$XDXKMD_SRC/utils/build_script/kernel_config/$USE_CONFIG"
    fi

    sh -ex <<-END_BUILD_SCRIPT
	cd $XDXKMD_SRC
	cd $BUILD_DIR/linux
	cp $config .config
	yes '' | make oldconfig
	#./scripts/kconfig/merge_config.sh .config $XDXKMD_SRC/utils/build_script/kernel_config/config.pro
	make-kpkg -j 8 --rootcmd fakeroot --initrd --append-to-version=$KERNEL_TAGS kernel_image kernel_headers
END_BUILD_SCRIPT
}

build_kernel_image() {
     sh -ex <<-END_BUILD_SCRIPT
        cd $XDXKMD_SRC
        cd $BUILD_DIR/linux
        make-kpkg -j 8 --rootcmd fakeroot --initrd --append-to-version=$KERNEL_TAGS kernel_image kernel_headers
END_BUILD_SCRIPT
}

case $1 in
	"1")
		bbh_init_kernel
		prep_kernel_script
		bbh_build_kernel
	;;
	"2")
		bbh_init_kernel
        update_kernel_files
		build_kernel_image
	;;
esac
