#!/bin/bash

export RGX_BNC=35.V.1632.23
export RGX_BVNC=35.4.1632.23
printUsage() {
    echo
    echo "Usage: $(basename "${BASH_SOURCE}") [options ...] [make options]"
    echo
    echo "Options:"
    echo "  -c,  --clean              Clean output and delete all intermediate work"
    echo "  -p,  --package            Make a release build instead of a debug build"
    echo "  -h,  --help               Prints this help"
    echo
    return 0
}

KERNEL_VERSION=`uname -r`
source ./script/setup_ddk_config.sh
#KERNELBUILD='/home/yajunl/workspace/linux'
KERNELBUILD="/lib/modules/$KERNEL_VERSION/build"
INSTALL_PATH="/lib/modules/$KERNEL_VERSION/extra"

XDXKMD_SRC=`pwd`

export XDXGPU_VERSION_BUILD="$(cd $XDXKMD_SRC/ && git describe --abbrev=5 --always)"
export XDXGPU_VERSION_BUILD=$((16#$XDXGPU_VERSION_BUILD))

echo "XDXGPU_VERSION_BUILD: $XDXGPU_VERSION_BUILD"

#parse the arguments
VALID_STR=`getopt -o hcraso: --long help,clean,package: -- "$@"`
eval set -- "$VALID_STR"

TARGET="build"

while true ;
do
    #echo "parocessing $1"
    case "$1" in
        (-h | --help)
                printUsage ; exit 0;;
        (-c | --clean)
                TARGET="clean" ; ((CLEAN_OR_OUT|=1)) ; shift ;;
        (-r | --release)
                BUILD_TYPE="Release" ; shift ;;
        --)     shift; break;; # end delimiter
        (*)
                echo " This should never come but just incase : UNEXPECTED ERROR Parm : [$1] ">&2 ; exit 20;;
    esac

done

do_build_kmd()
{
    make -C $KERNELBUILD M=$XDXKMD_SRC modules -j16
    ##sudo cp $XDXKMD_SRC/xdxgpu/xdxgpu.ko $INSTALL_PATH
}

do_clean_kmd()
{
    find $XDXKMD_SRC -name "*.o" -exec rm {} \;
    find $XDXKMD_SRC -name "*.o.*" -exec rm {} \;
    find $XDXKMD_SRC -name "Module.symvers" -exec rm {} \;
    find $XDXKMD_SRC -name "modules.order" -exec rm {} \;
    find $XDXKMD_SRC -name "*.ko" -exec rm {} \;
    find $XDXKMD_SRC -name "*.cmd" -exec rm {} \;
    find $XDXKMD_SRC -name "*.mod.c" -exec rm {} \;
    find $XDXKMD_SRC -name "*.mod" -exec rm {} \;
}

case $TARGET in
    (clean) 
        do_clean_kmd ;;
    (build)
        do_build_kmd ;;
    (*) die "Invalid target $TARGET" ;;
esac

echo "Operation complete"

