#!/bin/bash
set -e

VERSION=1.10.100
BUILD_TYPE="release"

ARCH=`uname -m`
OS=`cat /etc/os-release  |sed -n '/^ID=/p' |sed -e 's/"//g' -e 's/ID=//g'`
printUsage() {
    echo
    echo "Usage: $(basename "${BASH_ROOT}") [options ...]"
    echo
    echo "Options:"
    echo "  -t,  --build-type [debug|release]     Set the build type [Default: release]"
    echo "  -v,  --version       		  Set the deb version"
    echo "  -h,  --help                           Prints this help"
    echo
    return 0
}

while [ "$1" != "" ]; do
    case $1 in
        (-t | --build-type)
            BUILD_TYPE=$2; shift;;
        (-v | --version)
            VERSION=$2; shift;;
        (-h | --help)
            printUsage; exit 0;;
        (*)
            echo " This should never come but just incase : UNEXPECTED ERROR Parm : [$1] ">&2 ; exit 20;;
    esac
    shift 1
done

BUILD_MACHINE=`uname -m`
SCRIPT_PATH="$(cd $(dirname $0)&&pwd)"
WK_ROOT="$(cd $SCRIPT_PATH/../&&pwd)"
OUT=$WK_ROOT/../out-$VERSION/$OS/$BUILD_TYPE

export PVR_BUILD_TYPE=$BUILD_TYPE
SOURCE_DIR="$WK_ROOT"

export XDXGPU_VERSION_BUILD="$(cd $SOURCE_DIR/ && git describe --abbrev=5 --always)"
export XDXGPU_VERSION_BUILD=$((16#$XDXGPU_VERSION_BUILD))

echo "XDXGPU_VERSION_BUILD: $XDXGPU_VERSION_BUILD"

source $WK_ROOT/script/setup_ddk_config.sh
do_target_build() {
    TARGET_OUT=$OUT/$KERNEL_VERSION-$BUILD_MACHINE
    BUILD_DIR="$TARGET_OUT/xdxkmd"
    BUILD_OUT_DIR="$OUT/$VERSION-$ARCH/$KERNEL_VERSION/extra"
    KERNELBUILD="/lib/modules/$KERNEL_VERSION/build"

    if [ ! -d $BUILD_DIR ]; then
        mkdir -p $BUILD_DIR
    fi

    if [ ! -d $BUILD_OUT_DIR ]; then
        mkdir -p $BUILD_OUT_DIR
    fi

    cp -pr $SOURCE_DIR/* $BUILD_DIR/

    make -C $KERNELBUILD M=$BUILD_DIR modules -j8

    cp $BUILD_DIR/xdxgpu/xdxgpu.ko   $BUILD_OUT_DIR/
}

MODS=`ls /lib/modules/`
for KERNEL_VERSION in $MODS
do
        do_target_build
done
