/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __DM_H__
#define __DM_H__

#include "pangu.h"
#include "dm_dc.h"
#include "xdx_shared.h"
#include "devicemem_typedefs.h"
#include "img_types.h"

struct xdx_out_funcs{
	int (*hdmi_init)(struct xdx_dm *dm, u32 id);
	int (*dp_init)(struct xdx_dm *dm, u32 id);
	int (*hdmi_deinit)(struct xdx_dm *dm, u32 id);
	int (*dp_deinit)(struct xdx_dm *dm, u32 id);
	int (*hdmi_hw_init)(struct xdx_dm *dm, u32 id);
	int (*dp_hw_init)(struct xdx_dm *dm, u32 id);
	int (*suspend)(struct xdx_dm *dm);
	int (*resume)(struct xdx_dm *dm);
};

struct xdx_output {
	struct drm_connector *connector;
	struct drm_encoder *encoder;
	struct hdmi_dev *hdmi[MAX_OUTPUT_NUM];
	struct dp_dev *dp[MAX_OUTPUT_NUM];
	DEVMEM_MEMDESC	*psDoorbellShareMemDesc[4];
	void *psDoorbellVirAddr[4];
	IMG_DEV_PHYADDR psDoorbellPhyAddr[4];
};

struct dm_rpm_data {
	struct xdx_device *xdev;
	struct dc8200_plane *plane;
	struct drm_gem_object *normal_gobj;
	struct drm_gem_object *rpm_gobj;
	uint64_t *pages_backup;
	uint32_t num_pages_backup;
};

/* display manager */
struct xdx_dm {
	struct xdx_ip_block base;
	struct dm_dc *dc[MAX_OUTPUT_NUM];
	struct xdx_output output;
	struct dm_rpm_data *rpm_data[XDX_MAX_CRTCS];
	uint8_t dc_num;
};

static inline struct xdx_dm *output_to_dm(struct xdx_output *output)
{
	return container_of(output, struct xdx_dm, output);
}

static inline struct xdx_output *dm_to_output(struct xdx_dm *dm)
{
	return &dm->output;
}

int dc8200_early_init(struct xdx_dm *dm, u32 id);

int xdx_dm_init(void *handle);
int xdx_dm_fini(void *handle);
int xdx_dm_rpm_suspend(struct xdx_dm *dm);
int xdx_dm_rpm_resume(struct xdx_dm *dm);


extern const struct xdx_ip_block_version dm_ip_block_version;

#endif /* __DM_H__ */
