/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __DM_CONNECTOR_H__
#define __DM_CONNECTOR_H__

#include <drm/drm_atomic.h>
#include <drm/drm_connector.h>
#include <drm/drm_crtc.h>
#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 19, 0)))
#include <drm/dp/drm_dp_mst_helper.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_mst_helper.h>
#else
#include <drm/drm_dp_mst_helper.h>
#endif
#include <drm/drm_plane.h>

#include "dm_encoder.h"

struct dm_connector {
	struct drm_connector base;
	int connector_type;
	uint32_t id;

	/* we need to mind the EDID between detect
	   and get modes due to analog/digital/tvencoder */
	struct edid *edid;

	/* number of modes generated from EDID */
	int num_modes;

	struct drm_dp_aux aux;

	struct i2c_adapter i2c;
};

static inline struct drm_connector *
to_drm_connector(struct dm_connector *dm_connector)
{
	return &dm_connector->base;
}

static inline struct dm_connector *
to_dm_connector(struct drm_connector *connector)
{
	return container_of(connector, struct dm_connector, base);
}

struct dm_connector_state {
	struct drm_connector_state base;
	struct {
		int mode; /* DRM_MODE_SCALE_* */
		struct {
			enum {
				UNDERSCAN_OFF,
				UNDERSCAN_ON,
				UNDERSCAN_AUTO,
			} mode;
			u32 hborder;
			u32 vborder;
		} underscan;
		bool full;
	} scaler;
};

static inline struct drm_connector_state *
to_drm_connector_state(struct dm_connector_state *dm_connector_state)
{
	return &dm_connector_state->base;
}

static inline struct dm_connector_state *
to_dm_connector_state(struct drm_connector_state *state)
{
	return container_of(state, struct dm_connector_state, base);
}

int dm_connector_init(struct xdx_dm *dm,
				    struct dm_connector *dm_connector,
				    uint32_t id, int connector_type,
				    struct dm_encoder *dm_encoder);

#endif /* __DM_CONNECTOR_H__ */