/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include <linux/moduleparam.h>
#include <linux/version.h>
#include <linux/component.h>
#include <linux/module.h>
#include "xdx_drv.h"
#include "pangu.h"
#include "xdx_device.h"
#include "syscommon.h"
#include "rgxfwutils.h"
#include "pvrsrv_error.h"
#include "pvr_drv.h"
#include "doorbell.h"

int DPDoorbellShareMemAllocate(struct xdx_output *output,u32 index)
{
	struct xdx_dm *dm = output_to_dm(output);
	struct xdx_device *xdev = dm_to_xdev(dm);

	PVRSRV_DEVICE_NODE *psDeviceNode = xdev->gfx.pvrdev;
	PVRSRV_RGXDEV_INFO *psDevInfo = psDeviceNode->pvDevice;
	PVRSRV_ERROR		eError = PVRSRV_OK;
	IMG_BOOL bValid;

	dev_info(xdev->dev,"doorbell allocate:device node:%llx devinfo:%llx\n",(uint64_t)psDeviceNode,(uint64_t)psDevInfo);
	eError = DevmemFwAllocate(psDevInfo,
							sizeof(struct dp_trans_para),
							RGX_FWSHAREDMEM_SHARE_RAM_ALLOCFLAGS,
							"FwDpTraStructure",
							&output->psDoorbellShareMemDesc[index]);
	if (eError != PVRSRV_OK)
	{
		dev_err(xdev->dev, "Display Doorbell allocate share ram fail\n");
		return -ENOMEM;
	}	
	eError =  DevmemAcquireCpuVirtAddr(output->psDoorbellShareMemDesc[index],(void **) &output->psDoorbellVirAddr[index]);
	if (eError != PVRSRV_OK)
	{
		dev_err(xdev->dev, "Display Doorbell get viraddr fail\n");
		return -ENOMEM;
	}
	eError = PMR_DevPhysAddr(output->psDoorbellShareMemDesc[index]->psImport->hPMR,
			OSGetPageShift(), /* FW will be using the same page size as the OS */
			1,
			output->psDoorbellShareMemDesc[index]->uiOffset,
			&output->psDoorbellPhyAddr[index],
			&bValid);
	if (eError != PVRSRV_OK)
	{
		dev_err(xdev->dev, "Display Doorbell get phyaddr fail\n");
		return -ENOMEM;
	}
	return 0;
}

void DPDoorbellShareMemFree(struct xdx_output *output,u32 index)
{

	if(output->psDoorbellShareMemDesc[index])
	{
		if(output->psDoorbellVirAddr[index] != NULL)
		{
			DevmemReleaseCpuVirtAddr(output->psDoorbellShareMemDesc[index]);
			output->psDoorbellVirAddr[index] = NULL;
		}
		DevmemFree(output->psDoorbellShareMemDesc[index]);
        output->psDoorbellShareMemDesc[index] = NULL;
	}
}