/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_DP_DRV_H__
#define __XDX_DP_DRV_H__

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_helper.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0))
#include <drm/dp/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif
#include <drm/drm_crtc.h>
#include <drm/drm_crtc_helper.h>
#include <drm/drm_of.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)
#include <drm/drm_print.h>
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0)) || defined(REDHAT_83) || (defined(CENTOS_81))
#include <drm/drm_probe_helper.h>
#else
#include <drm/drm_crtc_helper.h>
#endif
#include <drm/drm_simple_kms_helper.h>
#include <drm/drm_edid.h>

#include <linux/bitfield.h>
#include <linux/component.h>
#include <linux/iopoll.h>
#include <linux/irq.h>
#include <linux/of_device.h>
#include <linux/of_graph.h>

#include <sound/hdmi-codec.h>
#include <uapi/linux/videodev2.h>
#include <drm/drm_connector.h>
#include <drm/xdxgpu_drm.h>

#include "doorbell.h"
#include "dm.h"

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0))
#define DRM_COLOR_FORMAT_YCRCB444 DRM_COLOR_FORMAT_YCBCR444
#define DRM_COLOR_FORMAT_YCRCB422 DRM_COLOR_FORMAT_YCBCR422
#define DRM_COLOR_FORMAT_YCRCB420 DRM_COLOR_FORMAT_YCBCR420
#endif /* (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0)) */
/* pixel encoding/colorimetry format */
#ifndef _DP_MSA_MISC_COLOR
#define _DP_MSA_MISC_COLOR(misc1_7, misc0_21, misc0_3, misc0_4) \
	((misc1_7) << 15 | (misc0_4) << 4 | (misc0_3) << 3 | ((misc0_21) << 1))
#endif
#ifndef DP_MSA_MISC_COLOR_RGB
#define DP_MSA_MISC_COLOR_RGB			_DP_MSA_MISC_COLOR(0, 0, 0, 0)
#endif
#ifndef DP_MSA_MISC_COLOR_YCBCR_444_BT709
#define DP_MSA_MISC_COLOR_YCBCR_444_BT709	_DP_MSA_MISC_COLOR(0, 2, 1, 1)
#endif
#ifndef DP_MSA_MISC_COLOR_YCBCR_422_BT709
#define DP_MSA_MISC_COLOR_YCBCR_422_BT709	_DP_MSA_MISC_COLOR(0, 1, 1, 1)
#endif
#ifndef DP_MSA_MISC_6_BPC
#define DP_MSA_MISC_6_BPC			(0 << 5)
#endif
#ifndef DP_MSA_MISC_8_BPC
#define DP_MSA_MISC_8_BPC			(1 << 5)
#endif
#ifndef DP_MSA_MISC_10_BPC
#define DP_MSA_MISC_10_BPC			(2 << 5)
#endif
#ifndef DP_MSA_MISC_12_BPC
#define DP_MSA_MISC_12_BPC			(3 << 5)
#endif
#ifndef DP_MSA_MISC_16_BPC
#define DP_MSA_MISC_16_BPC			(4 << 5)
#endif
#ifndef DP_EXTENDED_RECEIVER_CAP_FIELD_PRESENT
# define DP_EXTENDED_RECEIVER_CAP_FIELD_PRESENT	(1 << 7) /* DP 1.3 */
#endif

#define DPTX_PHYIF_CTRL_RATE_RBR	0x0
#define DPTX_PHYIF_CTRL_RATE_HBR	0x1
#define DPTX_PHYIF_CTRL_RATE_HBR2	0x2
#define DPTX_PHYIF_CTRL_RATE_HBR3	0x3

#define DPTX_PHYIF_CTRL_TPS_NONE    0
#define DPTX_PHYIF_CTRL_TPS_1       1
#define DPTX_PHYIF_CTRL_TPS_2       2
#define DPTX_PHYIF_CTRL_TPS_3       3
#define DPTX_PHYIF_CTRL_TPS_4       4
#define DPTX_PHYIF_CTRL_TPS_SYM_ERM 5
#define DPTX_PHYIF_CTRL_TPS_PRBS7   6
#define DPTX_PHYIF_CTRL_TPS_CUSTOM80    7
#define DPTX_PHYIF_CTRL_TPS_CP2520_1    8
#define DPTX_PHYIF_CTRL_TPS_CP2520_2    9

#define DPTX_MAX_LINK_LANES 4
#define DPTX_PHYIF_CTRL_LANE_PWRDOWN_SHIFT 17
#define DPTX_PHYIF_CTRL_LANE_PWRDOWN_MASK GENMASK(20, 17)
#define DPTX_PHYIF_CTRL_TPS_SEL_SHIFT   0
#define DPTX_PHYIF_CTRL_TPS_SEL_MASK    GENMASK(3, 0)
#define DPTX_PHYIF_CTRL_RATE_SHIFT  4
#define DPTX_PHYIF_CTRL_RATE_MASK   GENMASK(5, 4)
#define DPTX_PHYIF_CTRL_LANES_SHIFT 6
#define DPTX_PHYIF_CTRL_LANES_MASK  GENMASK(7, 6)
#define DPTX_PHYIF_CTRL_SSC_DIS     BIT(16)



struct dptx_link {
    bool cr_state;
    bool ch_eq_state;
    bool trained;
    u32 rate;
    u8 preemp_level[4];
    u8 vswing_level[4];
    u8 cr_preemp_level[4];
    u8 cr_vswing_level[4];
    u8 train_value[4];
    u8 lanes;
    u8 link_state;
    u8 reserve0;
    u8 reserve1;
    u8 rx_caps[DP_RECEIVER_CAP_SIZE];
    u8 status[DP_LINK_STATUS_SIZE];
};

struct video_params {
    u8 pixel_mode;
    u8 pix_enc;
    u8 bpc_mapping;
    u8 bpc;
    u8 bpp;
    u8 colorimetry;
    u8 dynamic_range;
    u8 aver_bytes_per_tu;
    u8 aver_bytes_per_tu_frac;
    u8 init_threshold;
    u8 video_format;
    u8 downstream_max_bpc;
};

enum {
    DPTX_PHY_PATTERN_NONE,
    DPTX_PHY_PATTERN_TPS_1,
    DPTX_PHY_PATTERN_TPS_2,
    DPTX_PHY_PATTERN_TPS_3,
    DPTX_PHY_PATTERN_TPS_4,
    DPTX_PHY_PATTERN_SERM,
    DPTX_PHY_PATTERN_PBRS7,
    DPTX_PHY_PATTERN_CUSTOM_80BIT,
    DPTX_PHY_PATTERN_CP2520_1,
    DPTX_PHY_PATTERN_CP2520_2,
};

enum {
    DPTX_RGB_6BIT,
    DPTX_RGB_8BIT,
    DPTX_RGB_10BIT,
    DPTX_RGB_12BIT,
    DPTX_RGB_16BIT,
    DPTX_YCBCR444_8BIT,
    DPTX_YCBCR444_10BIT,
    DPTX_YCBCR444_12BIT,
    DPTX_YCBCR444_16BIT,
    DPTX_YCBCR422_8BIT,
    DPTX_YCBCR422_10BIT,
    DPTX_YCBCR422_12BIT,
    DPTX_YCBCR422_16BIT,
    DPTX_YCBCR420_8BIT,
    DPTX_YCBCR420_10BIT,
    DPTX_YCBCR420_12BIT,
    DPTX_YCBCR420_16BIT,
};

enum {
    LANE_RATE_RBR,
    LANE_RATE_HBR1,
    LANE_RATE_HBR2,
    LANE_RATE_HBR3,
};

enum {
    TRAIN_INIT,
    TRAIN_START,
    CLOCK_RECOVERY,
    CHANNEL_EQUALIZATION,
    TRAIN_COMPLETED,
    TRAIN_FAILED,
};

struct dp_output_format {
    u8 pix_enc;
    u8 bpc_mapping;
    u8 bpc;
    u8 bpp;
};

static const struct dp_output_format possible_output[] = {
    { DRM_COLOR_FORMAT_RGB444, DPTX_RGB_10BIT, 10, 30 },
    { DRM_COLOR_FORMAT_RGB444, DPTX_RGB_8BIT, 8, 24 },
    { DRM_COLOR_FORMAT_RGB444, DPTX_RGB_6BIT, 6, 18 },
    { DRM_COLOR_FORMAT_YCRCB444, DPTX_YCBCR444_10BIT, 10, 30 },
    { DRM_COLOR_FORMAT_YCRCB444, DPTX_YCBCR444_8BIT, 8, 24},
    { DRM_COLOR_FORMAT_YCRCB422, DPTX_YCBCR422_10BIT, 10, 20 },
    { DRM_COLOR_FORMAT_YCRCB422, DPTX_YCBCR422_8BIT, 8, 16 },
    { DRM_COLOR_FORMAT_YCRCB420, DPTX_YCBCR420_10BIT, 10, 15 },
    { DRM_COLOR_FORMAT_YCRCB420, DPTX_YCBCR420_8BIT, 8, 12 },
};


enum {
    RGB6BITS_DSCRESERVED,
    RGB8BITS,
    RGB10BITS,
    RGB12BITS,
    RGB16BITS_DSCRESERVED,
    YCC444_8BITS,
    YCC444_10BITS,
    YCC444_12BITS,
    YVV444_16BITS_DSCRESERVED,
    YCC422_8BITS_DSCSIMPLEYCC4228BITS,
    YCC422_10BITS_DSCSIMPLEYCC42210BITS,
    YCC422_12BITS_DSCSIMPLEYCC42212BITS,
    YCC422_16BITS_DSCRESERVED,
    YCC420_8BITS,
    YCC420_10BITS,
    YCC420_12BITS,
    YCC420_16BITS_DSCRESERVED,
};

struct dp_dev {
    struct device *dev;
    struct drm_connector connector;
    struct drm_encoder encoder;
	struct workqueue_struct *workqueue;
    struct work_struct hpd_work;

    /* Video parameters */
    struct drm_display_info display_info;
    struct edid *edid;
    struct drm_display_mode dp_mode;
    struct drm_dp_aux aux;

    struct mutex mutex; /* generic mutex for dptx */
    struct video_params vparams;
    struct dptx_link link;
    struct xdx_output *output;
    bool encoder_is_on;
    bool link_trained;
    bool long_hpd;
#ifdef CONFIG_DEBUG_FS
    struct dentry **dirs;
    u32 dirs_num;
    u32 debugfs_dpcd_address;
    u32 debugfs_dpcd_size;
#endif
    u32 id;
    struct xdx_irq_src *irq_src;
    u8 downstream_ports[0x10];
    bool ssc_en;
    int cts;
};

struct dp_trans_para{
	int clock;
	int hdisplay;
	int hsync_start;
	int hsync_end;
	int htotal;
	int hskew;
	int vdisplay;
	int vsync_start;
	int vsync_end;
	int vtotal;
	int vscan;
	int vrefresh;
	u32 flag;

	u32 colordeep;
	u32 colorformat;
	u32 vic_code;
	u32 pixclock;

	u32 mode;
	u32 bpc;
	u32 multipixel;
	u32 mode_flag_pvsync;
	u32 lanes;
	u32 rate;
	u32 dc_idx;
	enum hdmi_picture_aspect picture_aspect_ratio;
};

static inline struct dp_dev *connector_to_dp(struct drm_connector *connector) {
    return container_of(connector, struct dp_dev, connector);
}

static inline struct dp_dev *encoder_to_dp(struct drm_encoder *encoder) {
    return container_of(encoder, struct dp_dev, encoder);
}

static inline bool
xdx_drm_dp_fast_training_cap(const u8 dpcd[DP_RECEIVER_CAP_SIZE])
{
    return dpcd[DP_DPCD_REV] >= 0x11 &&
        (dpcd[DP_MAX_DOWNSPREAD] & DP_NO_AUX_HANDSHAKE_LINK_TRAINING);
}

void dptx_interrupt_disable(struct dp_dev *dptx);
void dptx_interrupt_enable(struct dp_dev *dptx);
int xdx_dp_deinit(struct xdx_dm *dm, u32 id);
int xdx_dp_hw_init(struct xdx_dm *dm, u32 id);
int xdx_dp_init(struct xdx_dm *dm, u32 id);
void xdx_dp_suspend(struct dp_dev *dptx);
void xdx_dp_resume(struct xdx_dm *dm, u32 id);

void dptx_enable_ssc(struct dp_dev *dptx);
void dptx_phy_set_lanes(struct dp_dev *dptx, unsigned int lanes);




#endif /* __XDX_DP_DRV_H__ */
