/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef __XDX_DP_REG_H__
#define __XDX_DP_REG_H__

/* Constants */
#define DPTX_MP_SINGLE_PIXEL                0x0
#define DPTX_MP_DUAL_PIXEL                  0x1
#define DPTX_MP_QUAD_PIXEL                  0x2
#define DPTX_VERSION_NUMBER                 0x0000
#define DPTX_VERSION_TYPE                   0x0004
#define DPTX_ID                             0x0008

#define DPTX_CONFIG_REG1                    0x0100
#define DPTX_CONFIG_REG2                    0x0104
#define DPTX_CONFIG_REG3                    0x0108

#define DPTX_CCTL                           0x0200
#define FORCE_HPD                           BIT(4)
#define DEFAULT_FAST_LINK_TRAIN_EN          BIT(2)
#define ENHANCE_FRAMING_EN                  BIT(1)
#define SCRAMBLE_DIS                        BIT(0)
#define DPTX_SOFT_RESET_CTRL                0x0204
#define VIDEO_RESET                         BIT(5)
#define AUX_RESET                           BIT(4)
#define AUDIO_SAMPLER_RESET                 BIT(3)
#define PHY_SOFT_RESET                      BIT(1)
#define CONTROLLER_RESET                    BIT(0)

#define DPTX_VSAMPLE_CTRL                   0x0300
#define PIXEL_MODE_SELECT                   GENMASK(22, 21)
#define VIDEO_MAPPING                       GENMASK(20, 16)
#define VIDEO_STREAM_ENABLE                 BIT(5)
#define DPTX_VSAMPLE_STUFF_CTRL1            0x0304
#define DPTX_VSAMPLE_STUFF_CTRL2            0x0308
#define DPTX_VINPUT_POLARITY_CTRL           0x030c
#define DE_IN_POLARITY                      BIT(2)
#define HSYNC_IN_POLARITY                   BIT(1)
#define VSYNC_IN_POLARITY                   BIT(0)
#define DPTX_VIDEO_CONFIG1                  0x0310
#define HACTIVE                             GENMASK(31, 16)
#define HBLANK                              GENMASK(15, 2)
#define I_P                                 BIT(1)
#define R_V_BLANK_IN_OSC                    BIT(0)
#define DPTX_VIDEO_CONFIG2                  0x0314
#define VBLANK                              GENMASK(31, 16)
#define VACTIVE                             GENMASK(15, 0)
#define DPTX_VIDEO_CONFIG3                  0x0318
#define H_SYNC_WIDTH                        GENMASK(31, 16)
#define H_FRONT_PORCH                       GENMASK(15, 0)
#define DPTX_VIDEO_CONFIG4                  0x031c
#define V_SYNC_WIDTH                        GENMASK(31, 16)
#define V_FRONT_PORCH                       GENMASK(15, 0)
#define DPTX_VIDEO_CONFIG5                  0x0320
#define INIT_THRESHOLD_HI                   GENMASK(22, 21)
#define AVERAGE_BYTES_PER_TU_FRAC           GENMASK(19, 16)
#define INIT_THRESHOLD                      GENMASK(13, 7)
#define AVERAGE_BYTES_PER_TU                GENMASK(6, 0)
#define DPTX_VIDEO_MSA1                     0x0324
#define VSTART                              GENMASK(31, 16)
#define HSTART                              GENMASK(15, 0)
#define DPTX_VIDEO_MSA2                     0x0328
#define MISC0                               GENMASK(31, 24)
#define DPTX_VIDEO_MSA3                     0x032c
#define MISC1                               GENMASK(31, 24)
#define DPTX_VIDEO_HBLANK_INTERVAL          0x0330
#define HBLANK_INTERVAL_EN                  BIT(16)
#define HBLANK_INTERVAL                     GENMASK(15, 0)
#define DPTX_PHYIF_CTRL                     0x0a00
#define PHY_WIDTH                           BIT(25)
#define PHY_POWERDOWN                       GENMASK(20, 17)
#define PHY_BUSY                            GENMASK(15, 12)
#define SSC_DIS                             BIT(16)
#define XMIT_ENABLE                         GENMASK(11, 8)
#define PHY_LANES                           GENMASK(7, 6)
#define PHY_RATE                            GENMASK(5, 4)
#define TPS_SEL                             GENMASK(3, 0)
#define DPTX_PHY_TX_EQ                      0x0a04
#define DPTX_PHY_TX_EQ_PREEMP_SHIFT(lane)   (6 * lane)
#define DPTX_PHY_TX_EQ_PREEMP_MASK(lane)    GENMASK(6 * lane + 1, 6 * lane)
#define DPTX_PHY_TX_EQ_VSWING_SHIFT(lane)   (6 * lane + 2)
#define DPTX_PHY_TX_EQ_VSWING_MASK(lane)    GENMASK(6 * lane + 3, \
                                            6 * lane + 2)
#define DPTX_CUSTOMPAT0                     0x0a08
#define DPTX_CUSTOMPAT1                     0x0a0c
#define DPTX_CUSTOMPAT2                     0x0a10
#define DPTX_HBR2_COMPLIANCE_SCRAMBLER_RESET    0x0a14
#define DPTX_PHYIF_PWRDOWN_CTRL             0x0a18

#define DPTX_AUX_CMD                        0x0b00
#define AUX_CMD_TYPE                        GENMASK(31, 28)
#define AUX_ADDR                            GENMASK(27, 8)
#define I2C_ADDR_ONLY                       BIT(4)
#define AUX_LEN_REQ                         GENMASK(3, 0)
#define DPTX_AUX_STATUS                     0x0b04
#define AUX_RECEIVED                        BIT(16)
#define AUX_TIMEOUT                         BIT(17)
#define AUX_REPLY_ERR                       BIT(18)
#define AUX_SINK_DWA                        BIT(24)
#define AUX_BYTES_READ                      GENMASK(23, 19)
#define AUX_STATUS                          GENMASK(7, 4)
#define AUX_STATUS_ACK                      0x0
#define AUX_STATUS_NACK                     0x1
#define AUX_STATUS_DEFER                    0x2
#define AUX_STATUS_I2C_NACK                 0x4
#define AUX_STATUS_I2C_DEFER                0x8
#define DPTX_AUX_DATA0                      0x0b08
#define DPTX_AUX_DATA1                      0x0b0c
#define DPTX_AUX_DATA2                      0x0b10
#define DPTX_AUX_DATA3                      0x0b14

#define DPTX_GENERAL_INTERRUPT              0x0d00
#define VIDEO_FIFO_OVERFLOW_STREAM0         BIT(6)
#define AUDIO_FIFO_OVERFLOW_STREAM0         BIT(5)
#define SDP_EVENT_STREAM0                   BIT(4)
#define AUX_CMD_INVALID                     BIT(3)
#define AUX_REPLY_EVENT                     BIT(1)
#define HPD_EVENT                           BIT(0)
#define GENERAL_INTERRUPT_ENABLE            0x0d04
#define AUX_REPLY_EVENT_EN                  BIT(1)
#define HPD_EVENT_EN                        BIT(0)
#define DPTX_HPD_STATUS                     0x0d08
#define HPD_STATE                           GENMASK(11, 9)
#define HPD_STATUS                          BIT(8)
#define HPD_HOT_UNPLUG                      BIT(2)
#define HPD_HOT_PLUG                        BIT(1)
#define HPD_IRQ                             BIT(0)
#define HPD_INTERRUPT_ENABLE                0x0d0c
#define HPD_UNPLUG_ERR_EN                   BIT(3)
#define HPD_UNPLUG_EN                       BIT(2)
#define HPD_PLUG_EN                         BIT(1)
#define HPD_IRQ_EN                          BIT(0)

#define PHYREF_PLL_EN                       0x44
#define PLL0_PHYREF_EN                      BIT(0)
#define PHYREF_PLL_LOCK                     0x5c
#define PLL0_PHYREF_CORE_LOCK               BIT(0)
#define DP_PHY_CTRL                         0x04
#define SRAM_BYPASS                         BIT(1)
#define DP_CTRL                             0x18
#define TILE_VIDEO_MAPPING                  GENMASK(6, 2)
#define DP_PCS_CTRL                         0x0c
#define DP_PCS_WIDTH                        GENMASK(5, 4)
#define DP_LANE_RATE                        GENMASK(9, 6)
#define DP_LANE_MPLLB_SSC_EN                BIT(3)
// CTS
#define DP_TEST_PHY_PATTERN_SEL_MASK        GENMASK(2, 0)
#define DP_TEST_PHY_PATTERN_NONE            0x0
#define DP_TEST_PHY_PATTERN_D10             0x1
#define DP_TEST_PHY_PATTERN_SEMC            0x2
#define DP_TEST_PHY_PATTERN_PRBS7           0x3
#define DP_TEST_PHY_PATTERN_CUSTOM          0x4
#define DP_TEST_PHY_PATTERN_CP2520_1        0x5
#define DP_TEST_PHY_PATTERN_CP2520_2        0x6
#define DP_TEST_PHY_PATTERN_CP2520_3_TPS4   0x7

#define DP_TEST_80BIT_CUSTOM_PATTERN_0      0x250
#define DP_TEST_80BIT_CUSTOM_PATTERN_1      0x251
#define DP_TEST_80BIT_CUSTOM_PATTERN_2      0x252
#define DP_TEST_80BIT_CUSTOM_PATTERN_3      0x253
#define DP_TEST_80BIT_CUSTOM_PATTERN_4      0x254
#define DP_TEST_80BIT_CUSTOM_PATTERN_5      0x255
#define DP_TEST_80BIT_CUSTOM_PATTERN_6      0x256
#define DP_TEST_80BIT_CUSTOM_PATTERN_7      0x257
#define DP_TEST_80BIT_CUSTOM_PATTERN_8      0x258
#define DP_TEST_80BIT_CUSTOM_PATTERN_9      0x259

#define DPTX_SRST_CTRL_CONTROLLER   BIT(0)
#define DPTX_SRST_CTRL_PHY      BIT(1)
#define DPTX_SRST_CTRL_HDCP     BIT(2)
#define DPTX_SRST_CTRL_AUDIO_SAMPLER    BIT(3)
#define DPTX_SRST_CTRL_AUX      BIT(4)
#define DPTX_SRST_VIDEO_RESET_N(n)  BIT(5 + n)
#define DPTX_SRST_CTRL_ALL (DPTX_SRST_CTRL_CONTROLLER |     \
                DPTX_SRST_CTRL_PHY |      \
                DPTX_SRST_CTRL_HDCP |       \
                DPTX_SRST_CTRL_AUDIO_SAMPLER |  \
                DPTX_SRST_CTRL_AUX)

#endif
