/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef UTILS_H_
#define UTILS_H_

/***************************** Include Files ********************************/
#include <linux/module.h>
#include "debug_print.h"
#include "dma_debug.h"
/************************** Constant Definitions ****************************/

/**************************** Type Definitions ******************************/
typedef enum RET_CODE {
	RET_SUCCESS = 0,
	RET_FAILURE = 255,
	RET_ERR_DMACH_FIFOFULL
} RET_CODE_t;

#define TEST_ALIGNED(X,A)       (((uint32_t)(X)&(A-1))==0)          ///< Test A size aligned (A shall be power of 2)

#define BIT_U64(pos)                ((uint64_t)1U << (pos))                       ///< Convert bit position to value
#define GET_BITS_U64(X, H, L)       (((X)>>(L))&(BIT_U64(H-L+1)-1))

//#define BIT(pos)                (1U << (pos))                       ///< Convert bit position to value
#define GET_BITS(X, H, L)       (((X)>>(L))&(BIT(H-L+1)-1))

#ifdef ENV_STARCU
	#define DBG_PRINTF(format, ...) DBG_Printf_Exectb(format, ##__VA_ARGS__)
#elif defined(ENV_EMU)
	#define DBG_PRINTF(format, ...) DBG_Printf_Scratch(format, ##__VA_ARGS__)
#else
	#if defined DEBUG_PANGU
	#define DBG_PRINTF(format, ...) printk(format, ##__VA_ARGS__)
	#else
	#define DBG_PRINTF(format, ...)
	#endif
#endif


#define DBG_PRINT_VAR(var) XDX_DMA_INFO_VAR(var)
#define DBG_PRINT_VAR64(var) XDX_DMA_INFO_VAR64(var)
#ifdef DEBUG
#define DBG_ASSERT(cond) if (!(cond)) { \
							DBG_PRINTF("Assert on %s:%d\n", __FILE__, __LINE__);	\
							DBG_WritePostcode(0xffff0001);	\
							while(1);		\
							}
#else
#define DBG_ASSERT(cond)
#endif

#define STATIC_ASSERT(cond, msg) typedef char static_assertion_##msg[(cond)?1:-1]

#define RESULT_FAIL (-1)
#define RESULT_PASS	(0)

#define CASE_BEGIN() DBG_PRINTF("DMA_VA(%s): Begin!\n", __func__)
#define CASE_PASS(case_name) DBG_PRINTF("DMA_VA(%s): Pass!\n\n", case_name)
#define CASE_FAIL(case_name) DBG_PRINTF("DMA_VA(%s): Fail!\n\n", case_name)
#define CASE_STATUS(pass_flag)		\
	do {							\
		if (pass_flag) {			\
			CASE_PASS(__func__);	\
		} else {					\
			CASE_FAIL(__func__);	\
		}							\
	} while(0)



#endif /* UTILS_H_ */
