/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/moduleparam.h>
#include <linux/version.h>

#include "sysinfo.h"
//#include "pangu_drv.h"
//#include "pangu_regs.h"

//#include "pvrmodule.h"

#include "kernel_compatibility.h"
#include "vdma_hal.h"

/*
 * Common Register
 */
// RST
void VDMA_HAL_COM_WrRstReg(uint64_t VdmaBaseAddr, uint32_t Val)
{
	DBG_ASSERT((0 == Val) || (1 == Val));
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->SOFT_RST = Val;
}

uint32_t VDMA_HAL_COM_RdRstReg(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->SOFT_RST;
}

// DMA IDLE
uint32_t VDMA_HAL_COM_RdDmaIdleReg(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->DMA_IDLE;
}

// HOST AXLEN
void VDMA_HAL_COM_WrHostArLen(uint64_t VdmaBaseAddr, uint32_t Val)
{
	DBG_ASSERT(Val <= VDMA_PORT_AXLEN_MAX);
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->HOST_AXI_RD_CTRL = Val;
}

uint32_t VDMA_HAL_COM_RdHostArLen(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->HOST_AXI_RD_CTRL;
}

void VDMA_HAL_COM_WrHostAwLen(uint64_t VdmaBaseAddr, uint32_t Val)
{
	DBG_ASSERT(Val <= VDMA_PORT_AXLEN_MAX);
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->HOST_AXI_WR_CTRL = Val;
}

uint32_t VDMA_HAL_COM_RdHostAwLen(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->HOST_AXI_WR_CTRL;
}

// NDF AXLEN
void VDMA_HAL_COM_WrNdfArLen(uint64_t VdmaBaseAddr, uint32_t Val)
{
	DBG_ASSERT(Val <= VDMA_PORT_AXLEN_MAX);
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->NDF_AXI_RD_CTRL = Val;
}

uint32_t VDMA_HAL_COM_RdNdfArLen(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->NDF_AXI_RD_CTRL;
}

void VDMA_HAL_COM_WrNdfAwLen(uint64_t VdmaBaseAddr, uint32_t Val)
{
	DBG_ASSERT(Val <= VDMA_PORT_AXLEN_MAX);
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->NDF_AXI_WR_CTRL = Val;
}

uint32_t VDMA_HAL_COM_RdNdfAwLen(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->NDF_AXI_WR_CTRL;
}

// Channel Priority
void VDMA_HAL_COM_WrChannelPriority(uint64_t VdmaBaseAddr, uint32_t ChannelPriority)
{
	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(ChannelPriority <= 0xff);	// BIT[7:0] available

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->CHANNEL_PRIORITY = ChannelPriority;
}

uint32_t VDMA_HAL_COM_RdChannelPriority(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->CHANNEL_PRIORITY;
}

// Channel Req Timeout
void VDMA_COM_WrChannelReqTimeout(uint64_t VdmaBaseAddr, uint32_t ReqTimeout)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->CHANEL_REQ_TIMEOUT_SIZE = ReqTimeout;
}

uint32_t VDMA_COM_RdChannelReqTimeout(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->CHANEL_REQ_TIMEOUT_SIZE;
}

// HOST Port outstanding size
void VDMA_HAL_COM_WrHostPortOutstandingSize(uint64_t VdmaBaseAddr, uint32_t OutstadningSize)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->HOST_PORT_OUTSTANDING_SIZE = OutstadningSize;
}

uint32_t VDMA_HAL_COM_RdHostPortOutstandingSize(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->HOST_PORT_OUTSTANDING_SIZE;
}

// NDF Port outstanding size
void VDMA_HAL_COM_WrNdfPortOutstandingSize(uint64_t VdmaBaseAddr, uint32_t OutstadningSize)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->NDF_PORT_OUTSTANDING_SIZE = OutstadningSize;
}

uint32_t VDMA_HAL_COM_RdNdfPortOutstandingSize(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->NDF_PORT_OUTSTANDING_SIZE;
}

// VD-MCU ID
uint32_t VDMA_HAL_COM_RdVdMcuId(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);
	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->VD_MCU_ID;
}

// Scratch Reg
void VDMA_HAL_COM_WrScratchReg(uint64_t VdmaBaseAddr, uint32_t WrVal)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	((VDMA_Common_Regs_t*)VdmaBaseAddr)->SCRATCH = WrVal;
}

uint32_t VDMA_HAL_COM_RdScratchReg(uint64_t VdmaBaseAddr)
{
	DBG_ASSERT(0 != VdmaBaseAddr);

	return ((VDMA_Common_Regs_t*)VdmaBaseAddr)->SCRATCH;
}

/*
 * Channel Register
 */
// Channel Ctrl
void VDMA_HAL_CH_WrCtrlReg(uint64_t ChannelBaseAddr, uint32_t Ctrl)
{
	DBG_ASSERT(0 != ChannelBaseAddr);
	DBG_ASSERT(Ctrl <= 0x1ff);		// BIT[8][1:0] available

	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CTRL = Ctrl;
}

uint32_t VDMA_HAL_CH_RdCtrlReg(uint64_t ChannelBaseAddr)
{
	DBG_ASSERT(0 != ChannelBaseAddr);
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CTRL;
}

// Channel Idle
uint32_t VDMA_HAL_CH_RdChannelIdleReg(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CHANNEL_IDLE;
}

// Host AxQos
void VDMA_HAL_CH_WrHostAxQos(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t AxQos)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(AxQos < (0x1<<20));		// BIT[19:16][3:0] available

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->HOST_AXI_CTRL = AxQos;
}

uint32_t VDMA_HAL_CH_RdHostAxQos(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->HOST_AXI_CTRL;
}

// Ndf AxQos
void VDMA_HAL_CH_WrNdfAxQos(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t AxQos)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(AxQos < (0x1<<20));		// BIT[19:16][3:0] available

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->NDF_AXI_CTRL = AxQos;
}

uint32_t VDMA_HAL_CH_RdNdfAxQos(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->NDF_AXI_CTRL;
}

// CmdCmplIntSts
uint64_t VDMA_HAL_CH_RdCmdCmplIntSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	// Get High 32Bit
	return ((uint64_t)((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_INTR_STS[1] << 32) |
					((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_INTR_STS[0];
}

// PollingTimeoutIntSts
uint64_t VDMA_HAL_CH_RdPollingTimeoutIntSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	return ((uint64_t)((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_TIMEOUT_INTR_STS[1] << 32) |
				((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_TIMEOUT_INTR_STS[0];
}

// BusErrorIntSts
uint64_t VDMA_HAL_CH_RdBusErrIntSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	return ((uint64_t)((VDMA_Channel_Regs_t*)ChannelBaseAddr)->BUS_ERR_INTR_STS[1] << 32) |
						((VDMA_Channel_Regs_t*)ChannelBaseAddr)->BUS_ERR_INTR_STS[0];
}

// CmdCmplRawSts
uint64_t VDMA_HAL_CH_RdCmdCmplRawSts(uint64_t ChannelBaseAddr)
{
	uint32_t RawStsH, RawStsL;

	//DBG_ASSERT(0 != VdmaBaseAddr);
	//DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	RawStsL = ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_RAW_STS[0];
	RawStsH = ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_RAW_STS[1];

	return ((uint64_t)RawStsH << 32) | RawStsL;
}

uint64_t VDMA_HAL_CH_RdAndClrCmdCmplRawSts(uint64_t ChannelBaseAddr)
{
	uint32_t RawStsH, RawStsL;

	//DBG_ASSERT(0 != VdmaBaseAddr);
	//DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	RawStsL = ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_RAW_STS[0];
	RawStsH = ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_RAW_STS[1];

	//clear RAW_STS
	if(RawStsL)
	{
		((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_CLR[0] = RawStsL;
	}

	if(RawStsH)
	{
		((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_CLR[1] = RawStsH;
	}
	return ((uint64_t)RawStsH << 32) | RawStsL;
}

// PollingTimeoutRawSts
uint64_t VDMA_HAL_CH_RdPollingTimeoutRawSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	return ((uint64_t)((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_TIMEOUT_RAW_STS[1] << 32) |
							((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_TIMEOUT_RAW_STS[0];
}


uint64_t VDMA_HAL_CH_RdAndClrPollingTimeoutRawSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;
	uint64_t MaskVal;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	MaskVal = ((uint64_t)((VDMA_Channel_Regs_t *)ChannelBaseAddr)->POLLING_TIMEOUT_RAW_STS[1] << 32) |
							((VDMA_Channel_Regs_t *)ChannelBaseAddr)->POLLING_TIMEOUT_RAW_STS[0];

	// Clear PollingTimeoutSts
	if (MaskVal) {
		((VDMA_Channel_Regs_t *)ChannelBaseAddr)->POLLING_TIMEOUT_CLR[0] = (uint32_t)MaskVal;
		((VDMA_Channel_Regs_t *)ChannelBaseAddr)->POLLING_TIMEOUT_CLR[1] = (uint32_t)(MaskVal >> 32);
	}

	return MaskVal;
}

// BusErrRawSts
uint64_t VDMA_HAL_CH_RdBusErrRawSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	return ((uint64_t)((VDMA_Channel_Regs_t*)ChannelBaseAddr)->BUS_ERR_RAW_STS[1] << 32) |
							((VDMA_Channel_Regs_t*)ChannelBaseAddr)->BUS_ERR_RAW_STS[0];

}

uint64_t VDMA_HAL_CH_RdAndClrBusErrRawSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;
	uint64_t MaskVal;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	MaskVal = ((uint64_t)((VDMA_Channel_Regs_t *)ChannelBaseAddr)->BUS_ERR_RAW_STS[1] << 32) |
							((VDMA_Channel_Regs_t *)ChannelBaseAddr)->BUS_ERR_RAW_STS[0];

	// Clear BusErrSts
	if (MaskVal) {
		((VDMA_Channel_Regs_t *)ChannelBaseAddr)->BUS_ERR_CLR[0] = (uint32_t)MaskVal;
		((VDMA_Channel_Regs_t *)ChannelBaseAddr)->BUS_ERR_CLR[1] = (uint32_t)(MaskVal >> 32);
	}

	return MaskVal;
}


// Clear CmdCmplSts
void VDMA_HAL_CH_ClearCmdCmplSts(uint64_t ChannelBaseAddr, uint64_t MaskVal)
{
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_CLR[0] = (uint32_t)MaskVal;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_CMPL_CLR[1] = (uint32_t)(MaskVal >> 32);
}

// Clear PollingTimeoutSts
void VDMA_HAL_CH_ClearPollingTimeoutSts(uint64_t VdmaBaseAddr, uint32_t Channel, uint64_t MaskVal)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_TIMEOUT_CLR[0] = (uint32_t)MaskVal;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_TIMEOUT_CLR[1] = (uint32_t)(MaskVal >> 32);
}

// Clear BusErrSts
void VDMA_HAL_CH_ClearBusErrSts(uint64_t VdmaBaseAddr, uint32_t Channel, uint64_t MaskVal)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->BUS_ERR_CLR[0] = (uint32_t)MaskVal;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->BUS_ERR_CLR[1] = (uint32_t)(MaskVal >> 32);
}

// Polling Interval
void VDMA_HAL_CH_WrPollingIntval(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t PollingIntval)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_INTERVAL = PollingIntval;
}

uint32_t VDMA_HAL_CH_RdPollingIntval(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_INTERVAL;
}

// Polling Max Size
void VDMA_HAL_CH_WrPollingMaxSize(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t PollingMaxSize)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_MAX_SIZE = PollingMaxSize;
}

uint32_t VDMA_HAL_CH_RdPollingMaxSize(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_MAX_SIZE;
}

// RD_SRC_DESC STREAM ID
void VDMA_HAL_CH_WrSrcDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(StreamId <= VDMA_STREAM_ID_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->RD_SRC_DESC_STREAM_ID = StreamId;
}

uint32_t VDMA_HAL_CH_RdSrcDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->RD_SRC_DESC_STREAM_ID;
}

// RD_DST_DESC STREAM ID
void VDMA_HAL_CH_WrDstDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(StreamId <= VDMA_STREAM_ID_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->RD_DST_DESC_STREAM_ID = StreamId;
}

uint32_t VDMA_HAL_CH_RdDstDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->RD_DST_DESC_STREAM_ID;
}

// RD_SRC_DATA STREAM ID
void VDMA_HAL_CH_WrSrcDataReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(StreamId <= VDMA_STREAM_ID_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->RD_SRC_DATA_STREAM_ID = StreamId;
}

uint32_t VDMA_HAL_CH_RdSrcDataReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->RD_SRC_DATA_STREAM_ID;
}

// POLLING_RD_DATA STREAM ID
void VDMA_HAL_CH_WrPollingRdDataStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(StreamId <= VDMA_STREAM_ID_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_RD_DATA_STREAM_ID = StreamId;
}

uint32_t VDMA_HAL_CH_RdPollingRdDataStreamId(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->POLLING_RD_DATA_STREAM_ID;
}

// WR_DST_DATA STREAM ID
void VDMA_HAL_CH_WrDstDataWriteStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(StreamId <= VDMA_STREAM_ID_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->WR_DST_DATA_STREAM_ID = StreamId;
}

uint32_t VDMA_HAL_CH_RdDstDataWriteStreamId(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->WR_DST_DATA_STREAM_ID;
}

// PHYSICAL_ADDR STREAM ID
void VDMA_HAL_CH_WrPhyAddrStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(StreamId <= VDMA_STREAM_ID_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->PHYSICAL_ADDR_STREAM_ID = StreamId;
}

uint32_t VDMA_HAL_CH_RdPhyAddrStreamId(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->PHYSICAL_ADDR_STREAM_ID;
}

// Remain Cmd Buf Size
uint32_t VDMA_HAL_CH_RdRemainCmdBufSize(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->REMAIN_CMD_BUF_SIZE;
}

// Cmd Fifo Overflow Sts
uint32_t VDMA_HAL_CH_RdCmdBufOverflowSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_FIFO_STS;
}

// Cmd Fifo Overflow Sts
void VDMA_HAL_CH_ClearCmdBufOverflowSts(uint64_t VdmaBaseAddr, uint32_t Channel)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD_FIFO_STS = 1;
}

// Reference Buffer
void VDMA_HAL_CH_WrRefBufSingle(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t RefBufIdx, uint32_t WrData)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(RefBufIdx <= VDMA_REFBUF_IDX_MAX);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->REF_BUF[RefBufIdx] = WrData;
}

void VDMA_HAL_CH_WrRefBufBurst(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StartRefBufIdx, uint32_t *WrDataBuf, uint32_t Wrlen)
{
	uint64_t ChannelBaseAddr;
	uint32_t* RefBuf;
	uint32_t i;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);
	DBG_ASSERT(Wrlen != 0);
	DBG_ASSERT(Wrlen % 4 == 0);
	DBG_ASSERT(WrDataBuf != NULL);
	DBG_ASSERT((StartRefBufIdx + Wrlen/sizeof(uint32_t)) <= VDMA_REFBUF_IDX_MAX + 1);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;

	RefBuf = (uint32_t*)&(((VDMA_Channel_Regs_t*)ChannelBaseAddr)->REF_BUF[StartRefBufIdx]);
	for (i = 0; i < Wrlen/sizeof(uint32_t); i++) {
		RefBuf[i] = WrDataBuf[i];
	}
}

// Random values need to be cleared
void VDMA_HAL_CH_ClrAndWrRefBufSingle(uint64_t ChannelBaseAddr, uint32_t RefBufIdx, uint32_t WrData)
{
	DBG_ASSERT(0 != ChannelBaseAddr);
	DBG_ASSERT(RefBufIdx <= VDMA_REFBUF_IDX_MAX);
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->REF_BUF[RefBufIdx] = 0;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->REF_BUF[RefBufIdx] = WrData;
}


// Command Buffer
void VDMA_HAL_CH_WrCmdBufSingle(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t WrData)
{
	uint64_t ChannelBaseAddr;

	DBG_ASSERT(0 != VdmaBaseAddr);
	DBG_ASSERT(Channel < VDMA_CHANNEL_TOTAL_NUM);

	ChannelBaseAddr = VdmaBaseAddr + VDMA_CHANNEL_REG_OFFSET + VDMA_CHANNEL_REG_SEC_SIZE*Channel;
	((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD[0] = WrData;
}

// Wrlen : the count of uint64_t
void VDMA_HAL_CH_WrCmdBufBurst(uint64_t ChannelBaseAddr, uint64_t *WrDataBuf, uint32_t Wrlen)
{
	uint64_t* CmdBuf;
	uint32_t i;

	CmdBuf = (uint64_t*)&(((VDMA_Channel_Regs_t*)ChannelBaseAddr)->CMD[0]);
	
	for (i = 0; i < Wrlen; i++)
	{
		CmdBuf[i] = WrDataBuf[i];
	}
}

void VDMA_HAL_WritePostcode(uint64_t VdmaBaseAddr,uint32_t Postcode)
{
	((VDMA_Common_Regs_t*)VdmaBaseAddr)->SCRATCH = Postcode;
	return ;
}

uint32_t VDMA_HAL_GetChCmdRemainSize(uint64_t ChannelBaseAddr)
{
    DBG_ASSERT(0 != ChannelBaseAddr);

    return ((VDMA_Channel_Regs_t*)ChannelBaseAddr)->REMAIN_CMD_BUF_SIZE;
}

