/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef VDMA_HAL_H_
#define VDMA_HAL_H_

/***************************** Include Files ********************************/
#include "platform.h"

/************************** Constant Definitions ****************************/
#define VDMA_BASE_ADDR 0x20200000
#define VDMA0_BASE_ADDR 0xB00000
#define VDMA1_BASE_ADDR 0xD00000
#define DOORBELL_ADDR 0x100000

#define VDMA_CHANNEL_REG_OFFSET 0x1000
#define VDMA_CHANNEL_REG_SEC_SIZE	0X800

#define VDMA_PORT_AXLEN_MAX	 		7
#define VDMA_CHANNEL_TOTAL_NUM 		4
#define VDMA_STREAM_ID_MAX			1023
#define VDMA_COMMAND_TAG_TOTAL_NUM 	64
#define VDMA_CLEAR_REG_WIDTH		32
#define VDMA_CLEAR_REG_WIDTH_SHIFT	5
#define VDMA_REFBUF_IDX_MAX 		191
#define VDMA_CHANNEL_PRIORITY_MAX	3
#define VDMA_CHANNEL_QOS_MAX		0xf

#ifndef MOD_2
#define MOD_2(Y, X)       ((Y) & ((X) - 1))                   ///< Mod (pow 2)
#endif

/**************************** Type Definitions ******************************/
/* Common Registers Map*/
typedef struct VDMA_Common_Regs {
	volatile uint32_t SOFT_RST;						// 0x00, RW, write 1 assert, write 0 release
	volatile uint32_t DMA_IDLE;						// 0x04, RO, if 1, DMA in IDLE state(all channel in idle state)
	volatile uint32_t RSVD1[2];
	volatile uint32_t HOST_AXI_RD_CTRL;				// 0x10, SWWE, HOST ARLEN (3'h0:1, 3'h1:2, 3'h3:4, ... 3'h7:128)
	volatile uint32_t HOST_AXI_WR_CTRL;				// 0x14, SWWE, HOST AWLEN
	volatile uint32_t NDF_AXI_RD_CTRL;				// 0x18, SWWE, HOST ARLEN (3'h0:1, 3'h1:2, 3'h3:4, ... 3'h7:128)
	volatile uint32_t NDF_AXI_WR_CTRL;				// 0x1C, SWWE, HOST AWLEN
	volatile uint32_t CHANNEL_PRIORITY;				// 0x20, SWWE, Priority, 0 Lowest, 3 Highest
													// 		[1:0] CH0_PRIORITY, [3:2] CH1, [5:4] CH2, [7:6] CH3
	volatile uint32_t CHANEL_REQ_TIMEOUT_SIZE;		// 0x24, SWWE, if timeout, low priority channel will be served
	volatile uint32_t HOST_PORT_OUTSTANDING_SIZE;	// 0x28, SWWE
	volatile uint32_t NDF_PORT_OUTSTANDING_SIZE;	// 0x2C, SWWE
	volatile uint32_t RSVD3[4];
	volatile uint32_t VD_MCU_VERSION;				// 0x40 VD-MCU Version
	volatile uint32_t VD_MCU_ID;					// 0x44, RO, 0 - VD-MCU0, 1- VD-MCU1
	volatile uint32_t RSVD4[2];
	volatile uint32_t SCRATCH;						// 0x50, RW, Debug Purpose
	volatile uint32_t PAD_MUX_CFG;					// 0x54, RW, Debug Purpose
}VDMA_Common_Regs_t;

//STATIC_ASSERT(offsetof(VDMA_Common_Regs_t, PAD_MUX_CFG) == 0x54, VDMA_Common_Regs_t);
//STATIC_ASSERT(offsetof(VDMA_Common_Regs_t, VD_MCU_ID) == 0x44, VDMA_Common_Regs_t);
//STATIC_ASSERT(offsetof(VDMA_Common_Regs_t, NDF_PORT_OUTSTANDING_SIZE) == 0x2C, VDMA_Common_Regs_t);

/* Channel Registers Map*/
typedef struct VDMA_Channel_Regs {
	volatile uint32_t CTRL;							// 0x00, DMA Channel Ctrl
													// 		[0], SWWE, Channel Enable
													// 		[1], SWWE, Interrupt Enable
	volatile uint32_t CHANNEL_IDLE;					// 0x04, DMA Channel Idle
	volatile uint32_t HOST_AXI_CTRL;				// 0x08, HOST ARQOS and AWQOS
													//		[3:0], SWWE, ARQOS
													// 		[19:16], SWWE, AWQOS
	volatile uint32_t NDF_AXI_CTRL;					// 0x0C, NDF ARQOS and AWQOS
													//		[3:0], SWWE, ARQOS
													// 		[19:16], SWWE, AWQOS
	volatile uint32_t CMD_CMPL_INTR_STS[2];			// 0x10 ~ 0x14, RO, Command Completion Int Status
													//		0x10: Tag31-0, 0x14: Tag63-32
	volatile uint32_t POLLING_TIMEOUT_INTR_STS[2];	// 0x18 ~ 0x1C, RO, Polling Timeout Int Status
	volatile uint32_t BUS_ERR_INTR_STS[2];			// 0x20 ~ 0x24, RO, Bus Error Int Status

	volatile uint32_t CMD_CMPL_RAW_STS[2];			// 0x28 ~ 0x2C, RO, Command Completion Raw Status
	volatile uint32_t POLLING_TIMEOUT_RAW_STS[2];	// 0x30 ~ 0x34, RO, Polling Timeout Raw Status
	volatile uint32_t BUS_ERR_RAW_STS[2];			// 0x38 ~ 0x3C, RO, Bus Error Raw Status

	volatile uint32_t CMD_CMPL_CLR[2];				// 0x40 ~ 0x44, W1P, Command Completion Clear
	volatile uint32_t POLLING_TIMEOUT_CLR[2];		// 0x48 ~ 0x4c, W1P, Polling Timeout Clear
	volatile uint32_t BUS_ERR_CLR[2];				// 0x50 ~ 0x54, W1P, Bus Error Clear

	volatile uint32_t POLLING_INTERVAL;				// 0x58, SWWE, Polling Interval, Cycle
	volatile uint32_t POLLING_MAX_SIZE;				// 0x5C, SWWE, Polling Maximum Size

	volatile uint32_t RD_SRC_DESC_STREAM_ID;		// 0x60, SWWE, NDF read source descriptor axi request stream id
	volatile uint32_t RD_DST_DESC_STREAM_ID;		// 0x64, SWWE, NDF read destination descriptor axi request stream id
	volatile uint32_t RD_SRC_DATA_STREAM_ID;		// 0x68, SWWE, NDF read src data axi request stream id
	volatile uint32_t POLLING_RD_DATA_STREAM_ID;	// 0x6C, SWWE, NDF polling read data axi request stream id
	volatile uint32_t WR_DST_DATA_STREAM_ID;		// 0x70, SWWE, NDF write destination data axi request stream id
	volatile uint32_t PHYSICAL_ADDR_STREAM_ID;		// 0x74, SWWE, NDF port for GPA, use this stream id

	volatile uint32_t REMAIN_CMD_BUF_SIZE;			// 0x78, SWWE, Remain FIFO Cnt

			 uint32_t RSVD2[33];					// 0x7C ~ 0xFC

	volatile uint32_t CMD_FIFO_STS;					// 0x100, W1C, Command FIFO overflow status
	volatile uint32_t DBG_HSRC_FETCH_SRC_DESC_REQ;	// 0X104
	volatile uint32_t DBG_HSRC_FETCH_DST_DESC_REQ;	// 0X108
	volatile uint32_t DBG_HSRC_FETCH_SRC_DATA_REQ;	// 0X10C
	volatile uint32_t DBG_HSRC_POLLING_CMD_REQ;		// 0X110
	volatile uint32_t DBG_NSRC_FETCH_SRC_DESC_REQ;	// 0X114
	volatile uint32_t DBG_NSRC_FETCH_DST_DESC_REQ;	// 0X118
	volatile uint32_t DBG_NSRC_FETCH_SRC_DATA_REQ;	// 0x11c
	volatile uint32_t DBG_NSRC_POLLING_CMD_REQ;		// 0x120
	volatile uint32_t DBG_CMD_PARSE0;				// 0x124
	volatile uint32_t DBG_CMD_PARSE1;				// 0x128
	volatile uint32_t DBG_CMD_PARSE2;				// 0x12C
	volatile uint32_t DBG_HSRC_RD_RESP_DEMUX;		// 0x130
	volatile uint32_t DBG_NSRC_RD_RESP_DEMUX;		// 0x134
	volatile uint32_t DBG_HSRC_WR_DST_DATA_REQ;		// 0x138
	volatile uint32_t DBG_NSRC_WR_DST_DATA_REQ;		// 0x13C
	volatile uint32_t DBG_WR_DIRECT_REQ;			// 0x140

	volatile uint32_t N_H_COPY_CMD_CNT;				// 0x144
	volatile uint32_t H_H_COPY_CMD_CNT;				// 0x148
	volatile uint32_t H_WR_DIRECT_CMD_CNT;			// 0x14C
	volatile uint32_t H_POLLING_CMD_CNT;			// 0x150
	volatile uint32_t N_N_COPY_CMD_CNT;				// 0x154
	volatile uint32_t H_N_COPY_CMD_CNT;				// 0x158
	volatile uint32_t N_WR_DIRECT_CMD_CNT;			// 0x15C
	volatile uint32_t N_POLLING_CMD_CNT;			// 0x160
	volatile uint32_t FENCE_CMD_CNT;				// 0x164
	volatile uint32_t RECEIVE_CMD_CNT;				// 0x168
	volatile uint32_t COMPLE_CMD_CNT;				// 0x16C
	volatile uint32_t H_RD_BEAT_CNT;				// 0x170
	volatile uint32_t H_WR_BEAT_CNT;				// 0x174
	volatile uint32_t N_RD_BEAT_CNT;				// 0x178
	volatile uint32_t N_WR_BEAT_CNT;				// 0x17C
	volatile uint32_t DMA_ERROR_DETAIL;				// 0x180


			 uint32_t RSVD3[31];					// 0x184 ~ 0x1FC

	volatile uint32_t CMD[128];						// 0x200 ~ 0x3FC
	volatile uint32_t REF_BUF[192];					// 0x400 ~ 0x6FC
}VDMA_Channel_Regs_t;

//STATIC_ASSERT(offsetof(VDMA_Channel_Regs_t, CMD) == 0x200, VDMA_Channel_Regs_t);
//STATIC_ASSERT(offsetof(VDMA_Channel_Regs_t, REF_BUF) == 0x400, VDMA_Channel_Regs_t);
//STATIC_ASSERT(offsetof(VDMA_Channel_Regs_t, POLLING_INTERVAL) == 0x58, VDMA_Channel_Regs_t);

// ChannelPriority
typedef struct {
	uint32_t Ch0Priority:2;
	uint32_t Ch1Priority:2;
	uint32_t Ch2Priority:2;
	uint32_t Ch3Priority:2;
	uint32_t Rsvd:24;
}VDMA_COM_ChPriority_Reg_t;

typedef union {
	uint32_t val;
	VDMA_COM_ChPriority_Reg_t f;
}VDMA_COM_ChPriority_Reg_u;

// DMA Channel Ctrl
typedef struct {
	uint32_t DmaEnable:1;
	uint32_t IntrEn:1;
	uint32_t Rsvd:30;
}VDMA_CH_Ctrl_Reg_t;

typedef union {
	uint32_t val;
	VDMA_CH_Ctrl_Reg_t f;
}VDMA_CH_Ctrl_Reg_u;

// AX CTRL
typedef struct {
	uint32_t ArQos:4;
	uint32_t Rsvd1:12;
	uint32_t AwQos:4;
	uint32_t Rsvd2:12;
}VDMA_CH_AxQos_Reg_t;

typedef union {
	uint32_t val;
	VDMA_CH_AxQos_Reg_t f;
}VDMA_CH_AxQos_Reg_u;

/* VDMA_COM_STATUS_t */
typedef enum {
	VDMA_COM_STATUS_BUSY = 0x0,
	VDMA_COM_STATUS_IDLE = 0x1
} VDMA_COM_STATUS_t;

/* VDMA_COM_AXLEN_t */
typedef enum {
	VDMA_COM_AXLEN_1 = 0x0,
	VDMA_COM_AXLEN_2 = 0x1,
	VDMA_COM_AXLEN_4 = 0x2,
	VDMA_COM_AXLEN_8 = 0x3,
	VDMA_COM_AXLEN_16 = 0x4,
	VDMA_COM_AXLEN_32 = 0x5,
	VDMA_COM_AXLEN_64 = 0x6,
	VDMA_COM_AXLEN_128 = 0x7
} VDMA_COM_AXLEN_t;


/* VDMA_COM_PRIORITY_t */
typedef enum {
	VDMA_COM_PRIORITY_0 = 0x0,		// Lowest
	VDMA_COM_PRIORITY_1 = 0x1,
	VDMA_COM_PRIORITY_2 = 0x2,
	VDMA_COM_PRIORITY_3 = 0x3		// Highest
} VDMA_COM_PRIORITY_t;

/* VDMA_CH_STATUS_t */
typedef enum {
	VDMA_CH_STATUS_BUSY = 0x0,
	VDMA_CH_STATUS_IDLE = 0x1
} VDMA_CH_STATUS_t;

/***************** Macros (Inline Functions) Definitions ********************/



/************************** Function Prototypes *****************************/
/* Common Reg Function */
void VDMA_HAL_COM_WrRstReg(uint64_t VdmaBaseAddr, uint32_t Val);
uint32_t VDMA_HAL_COM_RdRstReg(uint64_t VdmaBaseAddr);
uint32_t VDMA_HAL_COM_RdDmaIdleReg(uint64_t VdmaBaseAddr);

void VDMA_HAL_COM_WrHostArLen(uint64_t VdmaBaseAddr, uint32_t Val);
uint32_t VDMA_HAL_COM_RdHostArLen(uint64_t VdmaBaseAddr);
void VDMA_HAL_COM_WrHostAwLen(uint64_t VdmaBaseAddr, uint32_t Val);
uint32_t VDMA_HAL_COM_RdHostAwLen(uint64_t VdmaBaseAddr);

void VDMA_HAL_COM_WrNdfArLen(uint64_t VdmaBaseAddr, uint32_t Val);
uint32_t VDMA_HAL_COM_RdNdfArLen(uint64_t VdmaBaseAddr);
void VDMA_HAL_COM_WrNdfAwLen(uint64_t VdmaBaseAddr, uint32_t Val);
uint32_t VDMA_HAL_COM_RdNdfAwLen(uint64_t VdmaBaseAddr);

void VDMA_HAL_COM_WrChannelPriority(uint64_t VdmaBaseAddr, uint32_t ChannelPriority);
uint32_t VDMA_HAL_COM_RdChannelPriority(uint64_t VdmaBaseAddr);

void VDMA_COM_WrChannelReqTimeout(uint64_t VdmaBaseAddr, uint32_t ReqTimeout);
uint32_t VDMA_COM_RdChannelReqTimeout(uint64_t VdmaBaseAddr);

void VDMA_HAL_COM_WrHostPortOutstandingSize(uint64_t VdmaBaseAddr, uint32_t OutstadningSize);
uint32_t VDMA_HAL_COM_RdHostPortOutstandingSize(uint64_t VdmaBaseAddr);
void VDMA_HAL_COM_WrNdfPortOutstandingSize(uint64_t VdmaBaseAddr, uint32_t OutstadningSize);
uint32_t VDMA_HAL_COM_RdNdfPortOutstandingSize(uint64_t VdmaBaseAddr);

uint32_t VDMA_HAL_COM_RdVdMcuId(uint64_t VdmaBaseAddr);

void VDMA_HAL_COM_WrScratchReg(uint64_t VdmaBaseAddr, uint32_t WrVal);
uint32_t VDMA_HAL_COM_RdScratchReg(uint64_t VdmaBaseAddr);

/* Channel Reg Function */
void VDMA_HAL_CH_WrCtrlReg(uint64_t ChannelBaseAddr, uint32_t Ctrl);
uint32_t VDMA_HAL_CH_RdCtrlReg(uint64_t ChannelBaseAddr);

uint32_t VDMA_HAL_CH_RdChannelIdleReg(uint64_t VdmaBaseAddr, uint32_t Channel);

void VDMA_HAL_CH_WrHostAxQos(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t AxQos);
uint32_t VDMA_HAL_CH_RdHostAxQos(uint64_t VdmaBaseAddr, uint32_t Channel);

void VDMA_HAL_CH_WrNdfAxQos(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t AxQos);
uint32_t VDMA_HAL_CH_RdNdfAxQos(uint64_t VdmaBaseAddr, uint32_t Channel);

uint64_t VDMA_HAL_CH_RdCmdCmplIntSts(uint64_t VdmaBaseAddr, uint32_t Channel);
uint64_t VDMA_HAL_CH_RdPollingTimeoutIntSts(uint64_t VdmaBaseAddr, uint32_t Channel);
uint64_t VDMA_HAL_CH_RdBusErrIntSts(uint64_t VdmaBaseAddr, uint32_t Channel);

uint64_t VDMA_HAL_CH_RdCmdCmplRawSts(uint64_t ChannelBaseAddr);
uint64_t VDMA_HAL_CH_RdAndClrCmdCmplRawSts(uint64_t ChannelBaseAddr);
uint64_t VDMA_HAL_CH_RdPollingTimeoutRawSts(uint64_t VdmaBaseAddr, uint32_t Channel);
uint64_t VDMA_HAL_CH_RdBusErrRawSts(uint64_t VdmaBaseAddr, uint32_t Channel);

void VDMA_HAL_CH_ClearCmdCmplSts(uint64_t ChannelBaseAddr, uint64_t MaskVal);
void VDMA_HAL_CH_ClearPollingTimeoutSts(uint64_t VdmaBaseAddr, uint32_t Channel, uint64_t MaskVal);
void VDMA_HAL_CH_ClearBusErrSts(uint64_t VdmaBaseAddr, uint32_t Channel, uint64_t MaskVal);

void VDMA_HAL_CH_WrPollingIntval(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t PollingIntval);
uint32_t VDMA_HAL_CH_RdPollingIntval(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_WrPollingMaxSize(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t PollingMaxSize);
uint32_t VDMA_HAL_CH_RdPollingMaxSize(uint64_t VdmaBaseAddr, uint32_t Channel);

void VDMA_HAL_CH_WrSrcDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId);
uint32_t VDMA_HAL_CH_RdSrcDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_WrDstDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId);
uint32_t VDMA_HAL_CH_RdDstDescReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_WrSrcDataReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId);
uint32_t VDMA_HAL_CH_RdSrcDataReadStreamId(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_WrPollingRdDataStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId);
uint32_t VDMA_HAL_CH_RdPollingRdDataStreamId(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_WrDstDataWriteStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId);
uint32_t VDMA_HAL_CH_RdDstDataWriteStreamId(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_WrPhyAddrStreamId(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StreamId);
uint32_t VDMA_HAL_CH_RdPhyAddrStreamId(uint64_t VdmaBaseAddr, uint32_t Channel);

uint32_t VDMA_HAL_CH_RdRemainCmdBufSize(uint64_t VdmaBaseAddr, uint32_t Channel);
uint32_t VDMA_HAL_CH_RdCmdBufOverflowSts(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_ClearCmdBufOverflowSts(uint64_t VdmaBaseAddr, uint32_t Channel);

void VDMA_HAL_CH_WrRefBufSingle(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t RefBufIdx, uint32_t WrData);
void VDMA_HAL_CH_WrRefBufBurst(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t StartRefBufIdx, uint32_t *WrDataBuf, uint32_t Wrlen);
void VDMA_HAL_CH_WrCmdBufSingle(uint64_t VdmaBaseAddr, uint32_t Channel, uint32_t WrData);
void VDMA_HAL_CH_WrCmdBufBurst(uint64_t ChannelBaseAddr, uint64_t *WrDataBuf, uint32_t Wrlen);
void VDMA_HAL_WritePostcode(uint64_t VdmaBaseAddr,uint32_t Postcode);
void VDMA_HAL_DumpComRegs(uint64_t VdmaBaseAddr);
void VDMA_HAL_DumpChannelRegs(uint64_t ChannelBaseAddr);
uint64_t VDMA_HAL_CH_RdAndClrPollingTimeoutRawSts(uint64_t VdmaBaseAddr, uint32_t Channel);
uint64_t VDMA_HAL_CH_RdAndClrBusErrRawSts(uint64_t VdmaBaseAddr, uint32_t Channel);
void VDMA_HAL_CH_ClrAndWrRefBufSingle(uint64_t ChannelBaseAddr, uint32_t RefBufIdx, uint32_t WrData);
uint32_t VDMA_HAL_GetChCmdRemainSize(uint64_t ChannelBaseAddr);



#endif /* VDMA_HAL_H_ */
