/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef XDX_DMA_H
#define XDX_DMA_H
#include <linux/clk.h>
#include <linux/delay.h>
#include <linux/dmaengine.h>
#include <linux/dmapool.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/of_dma.h>
#include <linux/of_device.h>
#include <linux/platform_device.h>
#include <linux/reset.h>
#include <linux/slab.h>
#include <linux/types.h>
#include <linux/dma-mapping.h>
#include <linux/version.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5,10,0))
#include <linux/dma-map-ops.h>
#endif
#include <linux/time.h>
#include <linux/timer.h>
#include <img_types.h>
#include <linux/mempool.h>
#include "virt-dma.h"
#include <asm/cacheflush.h>
#include <linux/pci.h>
#include "./hal/vdma_hal.h"
#include "./hal/vdma_drv.h"
#include <linux/random.h>
#include <linux/smp.h>
#include <linux/delay.h>
#include "B0_HalVdma.h"
#include "xdx_irq.h"
#include "vdma_irq.h"
#include "osfunc.h"
//#include "dma_common.h"
#include "pangu.h"
#include "xdx_dma_dbglevel.h"

#define XDX_DMA_ADDR_FROM (0)
#define XDX_DMA_ADDR_GPU (0 << XDX_DMA_ADDR_FROM)
#define XDX_DMA_ADDR_HOST (1 << XDX_DMA_ADDR_FROM)
#define XDX_DMA_ADDR_SMMU (1)
#define XDX_DMA_ADDR_NOSMMU (0 << XDX_DMA_ADDR_SMMU)
#define XDX_DMA_ADDR_SUPSMMU (1 << XDX_DMA_ADDR_SMMU)

#define XDX_DMA_ADDR_MASK 0x7
#define XDX_DMA_ADDR_GET_FLAG(t) (t & XDX_DMA_ADDR_MASK)
#define XDX_DMA_ADDR_CLEAR_FLAG(t) (t & ~XDX_DMA_ADDR_MASK)

#define XDX_DMA_MEMSET_SUPPORT_VALUE_LEN 4 //Currently only 4 bytes are supported


int xdx_dma_drv_init(void);
void xdx_dma_drv_exit(void);
struct dma_async_tx_descriptor *xdx_prep_2d_memcpy(
		struct dma_chan *chan,
		void *interleav_info,
		enum dma_transfer_direction dir,
		unsigned long flags);
struct dma_async_tx_descriptor *xdx_prep_twd_2d3d_memcpy(
		struct dma_chan *chan, void *interleav_info,
		enum dma_transfer_direction dir, unsigned long flags);

static inline struct dma_async_tx_descriptor *xdx_dmaengine_prep_dma_memset_sg(
		struct dma_chan *chan, struct scatterlist *sgl,
		unsigned int nents, int value, unsigned long flags)
{
	if (!chan || !chan->device || !chan->device->device_prep_dma_memset_sg)
		return NULL;

	return chan->device->device_prep_dma_memset_sg(chan, sgl, nents, value, flags);
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 11, 0))
static inline struct dma_async_tx_descriptor *xdx_dmaengine_prep_dma_memcpy(
		struct dma_chan *chan, dma_addr_t dest, dma_addr_t src,
		size_t len, unsigned long flags)
{
	if (!chan || !chan->device || !chan->device->device_prep_dma_memcpy)
		return NULL;

	return chan->device->device_prep_dma_memcpy(chan, dest, src,
						    len, flags);
}
#endif


#endif
