/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/debugfs.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include "xdx_dma_fs.h"
#include "rgxdevice.h"

static ssize_t xdx_dma_status_show(struct device *dev,
			struct device_attribute *devattr, char *buf)
{
	struct xdx_dma_dev *sdev = dev_get_drvdata(dev);
	ssize_t len = 0;
	int i;
	int k;
	struct xdx_vchan *vchan;
	struct xdx_pchan *pchan;

	for (i = 0, vchan = sdev->vchans; i < sdev->num_vchans; i++,vchan++) {
		len += sprintf(buf + len, "Vchan:%d  -> Pchans:%d\n\r", vchan->tag , vchan->pchan_cnt);
		for(k = 0; k <  vchan->pchan_cnt; k++) {
			pchan = vchan->pchans[k];
			len += sprintf(buf + len, "\tPchan:%d  ID:%04x\n\r", k, pchan->idx);
			spin_lock_irq(&pchan->lock);
			len += sprintf(buf + len, "\t\tDmaCmdPoolMask:%016llx  free_cmd:%d submit_cmd:%d\n\r",
				pchan->DmaCmdPoolMask, pchan->free_cmd, pchan->submit_cmd);
			spin_unlock_irq(&pchan->lock);
		}
	}
	return len;
}

static DEVICE_ATTR(status, S_IRUGO, xdx_dma_status_show, NULL);
static struct attribute *xdx_dma_dev_attributes[] = {
	&dev_attr_status.attr,
	NULL,
};
static const struct attribute_group xdx_dma_dev_attr_group = {
	.attrs	= xdx_dma_dev_attributes,
};

int xdx_dma_dev_fs_init(struct xdx_dma_dev *sdev)
{
	int ret;

	ret = sysfs_create_group(&sdev->dev->kobj, &xdx_dma_dev_attr_group);
	return ret;
}

void xdx_dma_dev_fs_exit(struct xdx_dma_dev *sdev)
{
	sysfs_remove_group(&sdev->dev->kobj, &xdx_dma_dev_attr_group);
}
