/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __PARAMETER_PAGE_H__
#define __PARAMETER_PAGE_H__

#define COMPILE_ASSERT(expr, message) extern int compile_assert_failed[(expr) ? 1 : -1] __attribute__((unused))

#define FAKE_FUSE     1

#define PARAMETER_FROM_FW  0
#define PARAMETER_FROM_SPI 1 

#define GDDR_SPEED_8GBPS   0
#define GDDR_SPEED_10GBPS  1
#define GDDR_SPEED_12GBPS  2
#define GDDR_SPEED_14GBPS  3
#define GDDR_SPEED_16GBPS  4

#define GDDR_SIZE_1GB                                        UINT64_C(0x40000000)
#define GDDR_SIZE_2GB                                        UINT64_C(0x80000000)
#define GDDR_SIZE_4GB                                        UINT64_C(0x100000000)
#define GDDR_SIZE_8GB                                        UINT64_C(0x200000000)
#define GDDR_SIZE_16GB                                       UINT64_C(0x400000000)
#define GDDR_SIZE_32GB                                       UINT64_C(0x800000000)
#define DEFAULT_GDDR_SIZE                                    GDDR_SIZE_1GB

#define PCIE_LINK_WIDTH_X1  0
#define PCIE_LINK_WIDTH_X2  1
#define PCIE_LINK_WIDTH_X4  2
#define PCIE_LINK_WIDTH_X8  3
#define PCIE_LINK_WIDTH_X16 4
#define PCIE_LINK_WIDTH_X32 5

#define PCIE_LINK_SPEED_GEN1  0
#define PCIE_LINK_SPEED_GEN2  1
#define PCIE_LINK_SPEED_GEN3  2
#define PCIE_LINK_SPEED_GEN4  3
#define PCIE_LINK_SPEED_GEN5  4
#define PCIE_LINK_SPEED_GEN6  5

#define HOST_TYPE_RESIZE_BAR_64BIT             0
#define HOST_TYPE_NONE_RESIZE_BAR_64BIT        1
#define HOST_TYPE_NONE_RESIZE_BAR_32BIT        2

#define DEVICE_TYPE_PANGU_A0_4PHY                0
#define DEVICE_TYPE_PANGU_A0_2PHY                1

#define GDDR_4PHY_SWAP    0
#define GDDR_2PHY_SWAP    1


#define PCIE_SUB_CLASS_VGA_COMPATIBLE       0x0000
#define PCIE_SUB_CLASS_VGA_8514_COMPATIBLE  0x0001
#define PCIE_SUB_CLASS_XGA                  0x0100
#define PCIE_SUB_CLASS_3D                   0x0200
#define PCIE_SUB_CLASS_OTHER                0x8000

#define PCIE_RESIZE_BAR_EN                     0
#define PCIE_RESIZE_BAR_DIS                    1

#define PCIE_DEFAULT_BARSIZE_256MB             0
#define PCIE_DEFAULT_BARSIZE_512MB             1
#define PCIE_DEFAULT_BARSIZE_1GB               2
#define PCIE_DEFAULT_BARSIZE_2GB               3
#define PCIE_DEFAULT_BARSIZE_4GB               4
#define PCIE_DEFAULT_BARSIZE_8GB               5
#define PCIE_DEFAULT_BARSIZE_16GB              6
#define PCIE_DEFAULT_BARSIZE_32GB              7
#define PCIE_DEFAULT_BARSIZE_64GB              8
#define PCIE_DEFAULT_BARSIZE_128GB             9
#define PCIE_DEFAULT_BARSIZE_256GB             10


#define enc_fw_num 5
#define fw_num 7

typedef enum
{
    cNonSecure = 0,
    cCheckSum,
    cHash,
    cSignature,
    cECCAES
}SecurityOption_T;

typedef enum
{
    cFwACPU = 0,
    cFwCMCU,
    cFwVDMCU0,
    cFwVDMCU1,
    cFwEXPROM,
    cFwGDDR,
    cFwMax
}FwType_T;

typedef struct
{
    volatile uint32_t magicNumber;
    uint32_t so;
    uint32_t imageSize;
    union
    {
        uint32_t version;
        struct
        {
            uint8_t  major;
            uint8_t  minor;
            uint16_t patch;
        };
    };
    uint32_t signature_r[8];
    uint32_t signature_s[8];
    uint32_t hash[8];
}ImageHeader_T;
COMPILE_ASSERT(sizeof(ImageHeader_T)==112, "size fault");

typedef struct
{
    uint32_t imageType;
    uint32_t segmentOffset;
    uint32_t segmentCount;
    uint32_t so;
}FirmwareHeader_T;
COMPILE_ASSERT(sizeof(FirmwareHeader_T)==16, "size fault");

typedef struct
{
    uint32_t sourceOffset;
    uint32_t dstAddrH;
    uint32_t dstAddrL;
    uint32_t segmentSize;
}SegmentInfo_T;
COMPILE_ASSERT(sizeof(SegmentInfo_T)==16, "size fault");

typedef struct
{
    FirmwareHeader_T header;
    SegmentInfo_T segment[2];
}FirmwareInfo_T;
COMPILE_ASSERT(sizeof(FirmwareInfo_T)==48, "size fault");

typedef struct
{
    uint32_t signature_r[8];
    uint32_t signature_s[8];
    uint32_t hash[8];
}SecurityHeader_T;
COMPILE_ASSERT(sizeof(SecurityHeader_T)==96, "size fault");

typedef union
{
    uint32_t all; //< All bits
    struct
    {
        uint8_t gpuDis : 4;
        uint8_t gddrDis : 4;
        uint8_t pcie4lane : 1;
        uint8_t disDebugSignal : 1;
        uint8_t disStrappingOverride : 1;
        uint8_t coreSightFeatureDis : 5;
        uint8_t resv[2];
    };
} Fuse_Func_t;
COMPILE_ASSERT(sizeof(Fuse_Func_t)==4, "size fault");


typedef union
{
    uint32_t all;
    struct
    {
        uint32_t resv0       : 8;
        uint32_t pcie_config : 8;
        uint32_t hw_func_opt :10;
        uint32_t cpm_v_grade : 5;
        uint32_t resv1       : 1;
    };
}F_Config1_T;
COMPILE_ASSERT(sizeof(F_Config1_T)==4, "size fault");

typedef union
{
    uint32_t all;
    struct
    {
        uint32_t vendorID        : 16;
        uint32_t deviceID        : 16;
    };
}F_Sku_T;
COMPILE_ASSERT(sizeof(F_Sku_T)==4, "size fault");

typedef union
{
    uint32_t all[2];
    struct
    {
        uint32_t thm_ch0_ddelta       : 8;
        uint32_t thm_ch1_ddelta       : 8;
        uint32_t thm_ch2_ddelta       : 8;
        uint32_t thm_ch3_ddelta       : 8;
        uint32_t thm_ch7_ddelta       : 8;
        uint32_t thm_tref             : 4;
        uint32_t resv                 : 20;
    };
}F_Thm_T;
COMPILE_ASSERT(sizeof(F_Thm_T)==8, "size fault");

typedef union
{
    uint32_t all[78];
    struct
    {
        uint32_t lcs;
        uint32_t osc;
        uint32_t func;
        uint32_t hdmi;
        uint32_t privKeyHash[8];
        uint32_t lcsOrig;
        uint32_t disOrig;
        uint32_t disStrapOverride;
        uint32_t oscOrig;
        uint32_t lcsBK;
        uint32_t disBK;
        uint32_t disStrapOverrideBK;
        uint32_t oscBK;
        uint32_t reopenKeyHash[8];
        uint32_t publicKeyX[8];
        uint32_t publicKeyY[8];
        uint32_t aesKey[4];
        uint32_t thm0[2];
        uint32_t thm1[2];
        F_Config1_T config1;
        uint32_t time;
        uint32_t chipID[2];
        uint32_t hdcpKPF[4];
        uint32_t bootLoaderBakcupAddr;
        uint32_t chaTimeout;
        uint32_t memRepairTimeout;
        uint32_t reservedTimeout;
        uint32_t spiParam;
        uint32_t bootCodeRsvd[11];
        uint32_t clkConfig;
        F_Sku_T sku;
    };
}Fuse_T;
COMPILE_ASSERT(sizeof(Fuse_T)==312, "size fault");

typedef union
{
    struct
    {
        SecurityHeader_T sec_header[enc_fw_num][2]; //768 Byte
    };
    uint8_t pad[1024];
}SecParameter_T;
COMPILE_ASSERT(sizeof(SecParameter_T)==1024, "size fault");

typedef union
{
    struct
    {
        uint32_t fakeFuse;
        uint32_t parameterSrc;
        uint32_t hostType;
        uint32_t deviceType;
        uint32_t fpgaType;
        uint32_t emuType;
        uint32_t gpuFreq;
        uint32_t ndfFreq;
        uint32_t skuDeviceType;
        uint32_t gddrInitDone;
        uint32_t fastSim;
        uint32_t platform;
        uint32_t acpuClkInMHz;
        uint32_t acpuGTimerClk;
        uint32_t acpuGTimerClkInKHz;
        uint32_t glbTimeStampClkInMHz;
        uint32_t glbTimeStampClkInKHz;
        uint32_t cmcuClkInMHz;
        uint32_t gpuClkInMHz;
        uint32_t apbClkInMHz;
        uint32_t ahbClkInMHz;
        uint32_t sysClkInMHz;
        uint32_t officialVer[16];
        uint32_t externalVer;
        uint32_t flashID;
    };
    uint8_t pad[1024];
}FwParameter_T;
COMPILE_ASSERT(sizeof(FwParameter_T)==1024, "size fault");

typedef union
{
    struct
    {
        uint32_t func1;
        uint32_t subClass;
        uint32_t resizeBarEn;
        uint32_t defaultBarSize;
    };
    uint8_t pad[256];
}PCIeParameter_T;
COMPILE_ASSERT(sizeof(PCIeParameter_T)==256, "size fault");

typedef union
{
    struct
    {
        uint32_t singleGddrSize;
        uint32_t gddrSpeed;
        uint32_t gddrCount;
        uint32_t ui32VendorID;
        uint64_t totalGDDRSize;
        uint32_t interleave;
        uint32_t reesv1;
        uint64_t fwReservedGddrAddr;
        uint32_t gddrSwap;
        uint32_t gddrType;
        uint32_t a0ECO;
        uint32_t initMode;
        uint32_t gddrWidth;
    };
    uint8_t pad[256];
}DDRParameter_T;
COMPILE_ASSERT(sizeof(DDRParameter_T)==256, "size fault");

typedef union
{
    struct
    {
        uint32_t fwSlot;                  //F2B - F2F  0:mainFw  1:bakupFw
        uint32_t signature_r_loader[8];   //F30 - F4F
        uint32_t signature_s_loader[8];   //F50 - F6F
        uint32_t hash_loader[8];          //F70 - F8F
        uint32_t signature_r_fw[8];       //F90 - FAF
        uint32_t signature_s_fw[8];       //FB0 - FCF
        uint32_t hash_fw[8];              //FD0 - FEF
        uint32_t iv_version;              //FF0 - FF3
        uint32_t iv_rsv[2];               //FF4 - FFB
        uint32_t iv_rsv_para_checksum;    //FFC - FFF
    };
    uint8_t pad[212];
}FWUpdateParameter_T;
COMPILE_ASSERT(sizeof(FWUpdateParameter_T)==212, "size fault");

typedef struct
{
    ImageHeader_T header;                           // 112B--
    FirmwareInfo_T fwInfo[fw_num];                  // 336B  |--total 512B
    uint8_t revheader[64];                          // 64B--
    SecParameter_T secParameter;                    // 1024B
    FwParameter_T fwParameter;                      // 1024B
    PCIeParameter_T pcieParameter;                  // 256B
    DDRParameter_T ddrParameter;                    // 256B
    Fuse_T fuse;                                    // 312B
    uint8_t rev2048[500];                           // 500B
    FWUpdateParameter_T fwupdateParameter;          // 212B
}ParameterPage_T;
COMPILE_ASSERT(sizeof(ParameterPage_T)==4096, "size fault");

#endif
