/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDXCMCU_IRQ_H__
#define __XDXCMCU_IRQ_H__
#include "xdxacpu.h"

#define __V volatile

struct xdx_device;

typedef union
{
	__V u64 all;
	struct
	{
		__V u32 low32;
		__V u32 high32;
	};
	struct
	{
		__V u64 done	: 1;
		__V u64 cmd		: 3;
		__V u64 type	: 4;
		__V u64 resv	: 24;
		__V u64 data	: 32;
	};
}ReportDBCmd_T;

struct cmcu_data {
	struct work_struct work;
	struct xdx_device *xdev;
};

struct acpu_data {
	struct work_struct work;
	struct xdx_device *xdev;
};

struct vdmcu0_data {
	struct work_struct work;
	struct xdx_device *xdev;
};

struct vdmcu1_data {
	struct work_struct work;
	struct xdx_device *xdev;
};

struct xdx_cmcu {
	struct xdx_irq_src *irq_src;
	/* register mmio */
	resource_size_t cmcu_rmmio_base;
	resource_size_t cmcu_rmmio_size;
	void __iomem *cmcu_rmmio;
	struct cmcu_data cmcu;
	struct acpu_data acpu;
	struct vdmcu0_data vdmcu0;
	struct vdmcu1_data vdmcu1;
};

typedef enum
{
	Warning = 0,
	Error = 1,
	FatalError,
}ErrorType_T;

typedef enum
{
	cCMCUWarning = 0,
	cCMCUError = 1,
	cCMCUFatalError
}CMCUStatusDBCmdType_T;

typedef enum
{
	cCMCUUnexpectedInterrupt = 0,
	cCMCUAssert,
	cCMCUErrorMax,
}CMCUErrorType_T;

typedef enum
{
	cCMCUNMIException = 0,
	cCMCUHardFaultException,
	cCMCUMMFaultException,
	cCMCUBusFaultException,
	cCMCUUsageFaultException,
	cCMCUSecureFaultException,
	cCMCUSVCallException,
	cCMCUDebugMonitorException,
	cCMCUPendSVException,
	cCMCUSysTickException,
	cCMCUWatchDog,
	cCMCUSocTempOverProtect,
	cCMCUGddrTempOverProtect,
	cCMCUFatalErrorMax,
}CMCUFatalErrorType_T;

typedef enum
{
	cACPUWarning = 0,
	cACPUError,
	cACPUFatalError
}ACPUStatusDBCmdType_T;

typedef enum
{
	cACPUUnexpectedInterrupt = 0,
	cACPUAssert,
	cACPUErrorMax,
}ACPUErrorType_T;

typedef enum
{
	eGddrReadCrcCount = 0,
	eGddrWriteCrcCount,
	cACPUWarningMax,
}ACPUWarningType_T;

typedef enum
{
	cACPUErrorException = 0,
	cACPUUnexceptedException,
	eGddrReadRetryCount,
	eGddrWriteRetryCount,
	cACPUFatalErrorMax,
}ACPUFatalErrorType_T;

typedef enum
{
	cVDMCU0Warning = 0,
	cVDMCU0Error,
	cVDMCU0FatalError
}VDMCU0StatusDBCmdType_T;

typedef enum
{
	cVDMCU0UnexpectedInterrupt = 0,
	cVDMCU0ssert,
	cVDMCU0ErrorMax,
}VDMCU0ErrorType_T;

typedef enum
{
	cVDMCU0NMIException = 0,
	cVDMCU0HardFaultException,
	cVDMCU0MMFaultException,
	cVDMCU0BusFaultException,
	cVDMCU0UsageFaultException,
	cVDMCU0SecureFaultException,
	cVDMCU0SVCallException,
	cVDMCU0DebugMonitorException,
	cVDMCU0PendSVException,
	cVDMCU0SysTickException,
	cVDMCU0FatalErrorMax,
}VDMCU0FatalErrorType_T;

typedef enum
{
	cVDMCU1Warning = 0,
	cVDMCU1Error,
	cVDMCU1FatalError
}VDMCU1StatusDBCmdType_T;

typedef enum
{
	cVDMCU1UnexpectedInterrupt = 0,
	cVDMCU1ssert,
	cVDMCU1ErrorMax,
}VDMCU1ErrorType_T;

typedef enum
{
	cVDMCU1NMIException = 0,
	cVDMCU1HardFaultException,
	cVDMCU1MMFaultException,
	cVDMCU1BusFaultException,
	cVDMCU1UsageFaultException,
	cVDMCU1SecureFaultException,
	cVDMCU1SVCallException,
	cVDMCU1DebugMonitorException,
	cVDMCU1PendSVException,
	cVDMCU1SysTickException,
	cVDMCU1FatalErrorMax,
}VDMCU1FatalErrorType_T;

int xdx_cmcu_init(struct xdx_device *xdev);
int xdx_cmcu_fini(struct xdx_device *xdev);

#endif /* __XDXCMCU_IRQ_H__ */
