/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0))
#include <linux/hwmon-sysfs.h>
#else
#include "hwmon-sysfs.h"
#endif
#include <linux/hwmon.h>
#include "xdx_hwmon.h"
#include "xdx_device.h"
#include "xdxacpu.h"
#include "xdx_pm.h"
#include "fwshare/FirmwareInfo.h"

#define GET_B31_24(X)           ((uint8_t)((X)>>24))
#define GET_B23_16(X)           ((uint8_t)((X)>>16))
#define GET_B15_00(X)           ((uint16_t)(X))

/* TODO: replace the dummy implement with host to cmcu doorbell */
struct xdxgpu_hwmon_dummy_record {
	long temp_crit;
	long temp_emergency;
	long power_cap;
	long pwm;
	long pwm_enable;
	long fan_enable;
	long fan_target;
};

static struct xdxgpu_hwmon_dummy_record dummy = {
	.temp_crit = 0,
	.temp_emergency = 0,
	.power_cap = 100,
	.pwm = 100,
	.pwm_enable = 0,
	.fan_enable = 1,
	.fan_target = 100
};

#if 0
static int xdxgpu_get_temp_input(struct xdx_device *xdev, long *val)
{
	*val = 0;
	return 0;
}

static int xdxgpu_get_temp_highest(struct xdx_device *xdev, long *val)
{
	*val = 0;
	return 0;
}

static int xdxgpu_get_temp_crit(struct xdx_device *xdev, long *val)
{
	*val = dummy.temp_crit;
	return 0;
}

static int xdxgpu_get_temp_emergency(struct xdx_device *xdev, long *val)
{
	*val = dummy.temp_emergency;
	return 0;
}

static int xdxgpu_get_in_input(struct xdx_device *xdev, long *val)
{
	*val = pangu_default_voltage;
	return 0;
}

static int xdxgpu_get_curr_input(struct xdx_device *xdev, long *val)
{
	*val = 500;
	return 0;
}

static int xdxgpu_get_power_average(struct xdx_device *xdev, long *val)
{
	*val = 50000;
	return 0;
}
#endif

static int xdxgpu_get_power_min(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_power_max(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_power_cap(struct xdx_device *xdev, long *val)
{
	*val = dummy.power_cap;
	return 0;
}

#if 0
static int xdxgpu_get_pwm(struct xdx_device *xdev, long *val)
{
	*val = dummy.pwm;
	return 0;
}
#endif

#if 0
static int xdxgpu_get_fan_input(struct xdx_device *xdev, long *val)
{
	*val = 200;
	return 0;
}

static int xdxgpu_get_fan_min(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_fan_max(struct xdx_device *xdev, long *val)
{
	*val = 20000;
	return 0;
}

static int xdxgpu_get_fan_target(struct xdx_device *xdev, long *val)
{
	*val = dummy.fan_target;
	return 0;
}

static int xdxgpu_set_temp_crit(struct xdx_device *xdev, long val)
{
	dummy.temp_crit = val;
	return 0;
}

static int xdxgpu_set_temp_emergency(struct xdx_device *xdev, long val)
{
	dummy.temp_emergency = val;
	return 0;
}
#endif

static int xdxgpu_set_power_cap(struct xdx_device *xdev, long val)
{
	dummy.power_cap = val;
	return 0;
}

#if 0
static int xdxgpu_set_pwm(struct xdx_device *xdev, long val)
{
	dummy.pwm = val;
	return 0;
}
#endif

#if 0
static int xdxgpu_set_fan_target(struct xdx_device *xdev, long val)
{
	dummy.fan_target = val;
	return 0;
}
#endif

static const xdx_device_t xdx_dev_table[] = {
	{0x10A0, "XDX110"},
	{0x10F0, "XDX115"},
	{0x1140, "XDX120"},
	{0x1150, "XDX121"},
	{0x1190, "XDX125"},
	{0x11E0, "XDX130"},
	{0x1230, "XDX135"},
	{0x1280, "XDX140"},
	{0x1290, "XDX141"},
	{0x12D0, "XDX145"},
	{0x1320, "XDX150"},
	{0x1330, "XDX150 Super"},
	{0x1340, "XDX151"},
	{0x1350, "XDX151 Super"},
	{0x1370, "XDX155"},
	{0x13C0, "XDX160"},
	{0x1410, "XDX165"},
	{0x1460, "XDX170"},
	{0x14B0, "XDX175"},
	{0x1500, "XDX180"},
	{0x1510, "XDX180 Super"},
	{0x1520, "XDX180 Ti"},
	{0x1550, "XDX185"},
	{0x15A0, "XDX190"},
	{0x15F0, "XDX195"},
	{0x10A2, "XDX110M"},
	{0x1142, "XDX120M"},
	{0x1503, "XDX R1800"},
	{0x15A3, "XDX R1900"},
	{0x1084, "XDX E1080"},
	{0x1094, "XDX E1090"},
	{0x10A4, "XDX E1100"},
	{0x1324, "XDX X1500"},
	{0x1504, "XDX X1800"},
	{0x15A4, "XDX X1900"},
	{0x15A5, "XDX X1900M2"},
};

static char* match_device_model(u16 device_id) {
	int i;
	int dev_num = sizeof(xdx_dev_table)/sizeof(xdx_dev_table[0]);

	for(i=0; i<dev_num; ++i) {
		if(device_id == xdx_dev_table[i].dev_id)
			return xdx_dev_table[i].dev_model;
	}

	return "Unknown";
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 2, 0))
#define HWMON_CHANNEL_INFO(stype, ...)	\
	(&(struct hwmon_channel_info) {	\
		.type = hwmon_##stype,	\
		.config = (u32 []) {	\
			__VA_ARGS__, 0	\
		}			\
	})
#endif
static const struct hwmon_channel_info *xdxgpu_hwmon_info[] = {
	HWMON_CHANNEL_INFO(temp,
			   HWMON_T_INPUT | HWMON_T_HIGHEST | HWMON_T_CRIT | \
			   HWMON_T_EMERGENCY | HWMON_T_MAX | HWMON_T_MIN),
	HWMON_CHANNEL_INFO(in, HWMON_I_INPUT),
	HWMON_CHANNEL_INFO(curr, HWMON_C_INPUT),
	HWMON_CHANNEL_INFO(power,
			   HWMON_P_AVERAGE | HWMON_P_CAP_MIN | \
			   HWMON_P_CAP_MAX | HWMON_P_CAP),
	HWMON_CHANNEL_INFO(fan,
			   HWMON_F_INPUT | \
			   HWMON_F_MIN | HWMON_F_MAX | HWMON_F_TARGET),
	NULL
};

static umode_t xdxgpu_is_visible(const void *drvdata, enum hwmon_sensor_types type, u32 attr, int channel)
{
	switch(type) {
	case hwmon_temp:
		switch(attr) {
		case hwmon_temp_input:
		case hwmon_temp_highest:
			return S_IRUGO;
		case hwmon_temp_crit:
		case hwmon_temp_max:
		case hwmon_temp_min:
		case hwmon_temp_emergency:
			return S_IRUGO;
		default:
			return (umode_t)0;
		}
	case hwmon_power:
		switch(attr) {
		case hwmon_power_average:
		case hwmon_power_cap_min:
		case hwmon_power_cap_max:
			return S_IRUGO;
		case hwmon_power_cap:
			return S_IRUGO | S_IWUSR;
		default:
			return (umode_t)0;
		}
	case hwmon_in:
		switch(attr) {
		case hwmon_in_input:
			return S_IRUGO  | S_IWUSR;
		default:
			return (umode_t)0;
		}
	case hwmon_curr:
		switch(attr) {
		case hwmon_curr_input:
			return S_IRUGO;
		default:
			return (umode_t)0;
		}
	case hwmon_fan:
		switch(attr) {
		case hwmon_fan_input:
		case hwmon_fan_min:
		case hwmon_fan_max:
			return S_IRUGO;
		case hwmon_fan_target:
			return S_IRUGO | S_IWUSR;
		default:
			return (umode_t)0;
		}
	default:
		return (umode_t)0;
	}
}

static int xdxgpu_hwmon_read(struct device *dev, enum hwmon_sensor_types type,
		      u32 attr, int channel, long *val)
{
	struct xdx_device *xdev;
    PublicFirmwareInfo_T *fwinfo;

	xdev = dev_get_drvdata(dev);
    fwinfo =(PublicFirmwareInfo_T *)xdev->fwinfo.fwinfo;

	switch(type) {
	case hwmon_temp:
		switch(attr) {
		case hwmon_temp_input:
			*val = get_gpu_temp(xdev);
            break;
		case hwmon_temp_highest:
		    *val = get_gpu_maxHistoryTemp(xdev);
            break;
		case hwmon_temp_crit:
	        *val = xdev->thm.slowdown_temp;
            break;
		case hwmon_temp_max:
		    *val = xdev->fw_st_info.temp_max;
			break;
		case hwmon_temp_min:
		    *val = xdev->fw_st_info.temp_min;
			break;
		case hwmon_temp_emergency:
	        *val = xdev->fw_st_info.temp_prot;
            break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_power:
		switch(attr) {
		case hwmon_power_average:
		    *val = fwinfo->curPower;
            break;
		
		//There aren't these attributes in firmware (HostEvent.h)
		case hwmon_power_cap_min:
			return xdxgpu_get_power_min(xdev, val);
		case hwmon_power_cap_max:
			return xdxgpu_get_power_max(xdev, val);
		case hwmon_power_cap:
			return xdxgpu_get_power_cap(xdev, val);

		default:
			return -EINVAL;
		}
		break;
	case hwmon_in:
		switch(attr) {
		case hwmon_in_input:
		    *val = get_gpu_volt(xdev);
            break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_curr:
		switch(attr) {
		case hwmon_curr_input:
		    *val = fwinfo->curCurr;
            break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_fan:
		switch(attr) {
		case hwmon_fan_input:
		case hwmon_fan_target:
		    *val = get_fan_rpm(xdev);
            break;
		case hwmon_fan_min:
		    *val = xdev->fw_st_info.min_rpm;
			break;
		case hwmon_fan_max:
		    *val = xdev->fw_st_info.max_rpm;
            break;
		default:
			return -EINVAL;
		}
		break;
	default:
		return -EINVAL;
	}

	return 0;
}

static int xdxgpu_hwmon_write(struct device *dev, enum hwmon_sensor_types type,
		       u32 attr, int channel, long val)
{
	uint8_t cmd, subcmd;
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);

	switch(type) {
	case hwmon_power:
		/* h02c doorbell interface not ready for now, using dummy data */
		switch(attr) {
		case hwmon_power_cap:
			return xdxgpu_set_power_cap(xdev, val);
		default:
			return -EINVAL;
		}
		break;
	case hwmon_in:
		if(xdev->asic_type == CHIP_PANGU_A0) {
			dev_warn(xdev->dev, "Forbid regulate A0 voltage\n");
			return -EINVAL;
		}
			
		cmd = cH02CPowerDBCmdDVFS;
		switch(attr) {
		case hwmon_in_input:
			subcmd = cH02CPowerDVFSSubCmdSetVolt;
			break;
		default:
			return -EINVAL;
		}
		break;
	case hwmon_fan:
		if(val < 0 || val > 100) {
			dev_warn(xdev->dev, "Must set fan within [0, 100]\n");
			return -EINVAL;
		}

		cmd = cH02CThermalCmdFan;
		switch(attr) {
		case hwmon_fan_target:
			subcmd = FanSubCmdPercentSet;
			break;
		default:
			return -EINVAL;
		}
		break;
	default:
		return -EINVAL;
	}

	if (type == hwmon_fan) {
		xdx_thermal_set(xdev, cmd, subcmd, channel, val);
	}
	else {
		xdx_power_set(xdev, cmd, subcmd, channel, val);
	}

	return 0;
}

static const struct hwmon_ops xdxgpu_hwmon_ops = {
	.is_visible = xdxgpu_is_visible,
	.read = xdxgpu_hwmon_read,
	.write = xdxgpu_hwmon_write
};

static const struct hwmon_chip_info xdxgpu_hwmon_chip_info = {
	.ops = &xdxgpu_hwmon_ops,
	.info = xdxgpu_hwmon_info
};

static ssize_t xdx_gddr_phys_speed_show(struct device *dev,
					   struct device_attribute *attr,
					   char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%llu\n", xdev->fw_st_info.gddr_speed);
#else
	return sprintf(buf, "%llu\n", xdev->fw_st_info.gddr_speed);
#endif
}

static ssize_t xdx_gddr_size_show(struct device *dev,
					   struct device_attribute *attr,
					   char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.gddr_size);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.gddr_size);
#endif
}

static ssize_t xdx_gfx_freq_mhz_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct xdx_device *xdev;
	xdev = dev_get_drvdata(dev);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", get_gpu_freq(xdev));
#else
	return sprintf(buf, "%u\n", get_gpu_freq(xdev));
#endif
}

static ssize_t xdx_gfx_freq_mhz_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	u32 value;

	int err;
	err = kstrtou32(buf, 10, &value);
	if (err)
		return err;
	
	xdev = dev_get_drvdata(dev);

	xdx_dvfs_set_freq(xdev, value);

	return count;
}

static ssize_t xdx_total_power_show(struct device *dev,
				     struct device_attribute *attr,
				     char *buf)
{
	struct xdx_device *xdev;
	PublicFirmwareInfo_T *fwinfo;
	uint32_t ret;
	xdev = dev_get_drvdata(dev);

	fwinfo =(PublicFirmwareInfo_T *)xdev->fwinfo.fwinfo;
	ret = fwinfo->curTotalPower;

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", ret);
#else
	return sprintf(buf, "%u\n", ret);
#endif
}

static ssize_t xdx_supported_gfx_freq_mhz_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->pm.current_gfxclk);
#else	
	return sprintf(buf, "%u\n", xdev->pm.current_gfxclk);
#endif
}

static ssize_t fan1_ctrl_mode_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	uint8_t cmd, subcmd;
	int value;
	int err;
	
	xdev = dev_get_drvdata(dev);

	cmd = cH02CThermalCmdFan;
	subcmd = FanSubCmdFanControlModeSet;
	
	err = kstrtoint(buf, 10, &value);

	if (err)
		return err;

	switch (value) {
		case 0:  /* auto */
			xdx_thermal_set(xdev, cmd, subcmd, 1, FanControlModeAuto);
			break;
		case 1:  /* manual */
			xdx_thermal_set(xdev, cmd, subcmd, 1, FanControlModeUser);
			break;
		default: /* disable */
			break;
	}

	return count;
}

static ssize_t fan1_ctrl_mode_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	PublicFirmwareInfo_T *fwinfo;
	IMG_UINT32 ret;
	xdev = dev_get_drvdata(dev);

    fwinfo =(PublicFirmwareInfo_T *)xdev->fwinfo.fwinfo;
	ret = fwinfo->fanControlMode;

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%i\n", ret);
#else
	return sprintf(buf, "%i\n", ret);
#endif
}

static ssize_t xdx_fw_ver_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "v%x.%x.%x\n", GET_B31_24(xdev->fw_st_info.fw_ver), GET_B23_16(xdev->fw_st_info.fw_ver), GET_B15_00(xdev->fw_st_info.fw_ver));
#else
	return sprintf(buf, "v%x.%x.%x\n", GET_B31_24(xdev->fw_st_info.fw_ver), GET_B23_16(xdev->fw_st_info.fw_ver), GET_B15_00(xdev->fw_st_info.fw_ver));
#endif
}

static ssize_t xdx_fw_ver_str_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.fw_ver_str);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.fw_ver_str);
#endif
}

static ssize_t xdx_vbios_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "v%x.%x.%x\n", GET_B31_24(xdev->fw_st_info.vbios_ver), GET_B23_16(xdev->fw_st_info.vbios_ver), GET_B15_00(xdev->fw_st_info.vbios_ver));
#else
	return sprintf(buf, "v%x.%x.%x\n", GET_B31_24(xdev->fw_st_info.vbios_ver), GET_B23_16(xdev->fw_st_info.vbios_ver), GET_B15_00(xdev->fw_st_info.vbios_ver));
#endif
}

static ssize_t xdx_vbios_str_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.vbios_ver_str);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.vbios_ver_str);
#endif
}

static ssize_t xdx_device_model_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	char* dev_model;
	xdev = dev_get_drvdata(dev);

	dev_model = match_device_model(xdev->fw_st_info.device_id);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s %u\n", dev_model, xdev->asic_type);
#else
	return sprintf(buf, "%s %u\n", dev_model, xdev->asic_type);
#endif
}

static ssize_t xdx_vendor_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	char* vendors[] = {"XDX", "BITLAND", "GOODTIMES", "OTHERS"};
	int size = ARRAY_SIZE(vendors);

	xdev = dev_get_drvdata(dev);
	if (xdev->fw_st_info.vendor >= 0 && xdev->fw_st_info.vendor < size) {
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
		return sysfs_emit(buf, "%s\n", vendors[xdev->fw_st_info.vendor]);
#else
		return sprintf(buf, "%s\n", vendors[xdev->fw_st_info.vendor]);
#endif
	} else {
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
		return sysfs_emit(buf, "%s\n", vendors[size - 1]);
#else
		return sprintf(buf, "%s\n", vendors[size - 1]);
#endif
	}
}

static ssize_t xdx_chip_id_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.chip_id);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.chip_id);
#endif
}

static ssize_t xdx_serial_number_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%s\n", xdev->fw_st_info.serial_number);
#else
	return sprintf(buf, "%s\n", xdev->fw_st_info.serial_number);
#endif
}

static ssize_t xdx_gpu_primary_id_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.gpu_primary_id);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.gpu_primary_id);
#endif
}

static ssize_t xdx_gddr_count_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.gddr_count);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.gddr_count);
#endif
}

static ssize_t pwm1_min_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	IMG_UINT32 ret;
	
	xdev = dev_get_drvdata(dev);
	ret = xdev->fw_st_info.min_duty;

	return sprintf(buf, "%i\n", ret);
}


static ssize_t pwm1_max_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;
	IMG_UINT32 ret;
	
	xdev = dev_get_drvdata(dev);
	ret = xdev->fw_st_info.max_duty;

	return sprintf(buf, "%i\n", ret);
}

static ssize_t xdx_gpu_primary_core_id_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%u\n", xdev->fw_st_info.gpu_primary_id);
#else
	return sprintf(buf, "%u\n", xdev->fw_st_info.gpu_primary_id);
#endif
}

static ssize_t xdx_gddr_temp_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%d\n", get_gddr_temp(xdev));
#else
	return sprintf(buf, "%d\n", get_gddr_temp(xdev));
#endif
}

static ssize_t xdx_support_fan_info_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.support_fan_info);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.support_fan_info);
#endif
}

static ssize_t xdx_support_power_info_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%x\n", xdev->fw_st_info.support_power_info);
#else
	return sprintf(buf, "%x\n", xdev->fw_st_info.support_power_info);
#endif
}

static ssize_t xdx_slowdown_temp_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%d\n", xdev->thm.slowdown_temp);
#else
	return sprintf(buf, "%d\n", xdev->thm.slowdown_temp);
#endif
}

static ssize_t xdx_slowdown_temp_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	int value;

	int err;
	err = kstrtoint(buf, 10, &value);
	if (err)
		return err;

	xdev = dev_get_drvdata(dev);

	if (value >= xdev->thm.shutdown_temp) {
		dev_warn(xdev->dev, "Slowdown temp should be less than shutdown temp %d", xdev->thm.shutdown_temp);
		return -EINVAL;
	}

	xdev->thm.slowdown_temp = value;
	dev_info(xdev->dev, "Set Slowdown Temp as %d\n", value);

	return count;
}

static ssize_t xdx_temp_hyst_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%d\n", xdev->thm.hyst);
#else
	return sprintf(buf, "%d\n", xdev->thm.hyst);
#endif
}

static ssize_t xdx_temp_hyst_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	int value;

	int err = kstrtoint(buf, 10, &value);
	if (err)
		return err;

	xdev = dev_get_drvdata(dev);
	if(value <= 0) {
		dev_warn(xdev->dev, "Temp Hysteresis should be larger than 0\n");
		return -EINVAL;
	}

	xdev->thm.hyst = value;
	dev_info(xdev->dev, "Set Temp Hysteresis as %d\n", value);

	return count;
}

static ssize_t xdx_shutdown_temp_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%d\n", xdev->thm.shutdown_temp);
#else
	return sprintf(buf, "%d\n", xdev->thm.shutdown_temp);
#endif
}

static ssize_t xdx_shutdown_temp_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	int shutdown_lower_limit, shutdown_upper_limit;
	int value;

	int err;
	err = kstrtoint(buf, 10, &value);
	if (err)
		return err;

	xdev = dev_get_drvdata(dev);

	shutdown_lower_limit = xdev->thm.slowdown_temp;
	shutdown_upper_limit = xdev->fw_st_info.temp_prot;
	if(value<=shutdown_lower_limit || value>=shutdown_upper_limit) {
		dev_warn(xdev->dev, "Shutdown Temp Setting Range (%d, %d)", shutdown_lower_limit, shutdown_upper_limit);
		return -EINVAL;
	}

	xdev->thm.shutdown_temp = value;

	return count;
}

static ssize_t xdx_enable_thm_shutdown_show(struct device *dev,
					       struct device_attribute *attr,
					       char *buf)
{
	struct xdx_device *xdev;

	xdev = dev_get_drvdata(dev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 10, 0))
	return sysfs_emit(buf, "%d\n", xdev->thm.enable_shutdown);
#else
	return sprintf(buf, "%d\n", xdev->thm.enable_shutdown);
#endif
}

static ssize_t xdx_enable_thm_shutdown_store(struct device *dev,
				      struct device_attribute *attr,
				      const char *buf,
				      size_t count)
{
	struct xdx_device *xdev;
	bool enable;

	int err;
	err = kstrtobool(buf, &enable);
	if (err)
		return err;

	xdev = dev_get_drvdata(dev);
	xdev->thm.enable_shutdown = enable;

	return count;
}

static SENSOR_DEVICE_ATTR_RO(xdx_gddr_phys_speed, xdx_gddr_phys_speed, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gddr_size, xdx_gddr_size, 0);
static SENSOR_DEVICE_ATTR_RW(xdx_gfx_freq_mhz, xdx_gfx_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_total_power, xdx_total_power, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_supported_gfx_freq_mhz, xdx_supported_gfx_freq_mhz, 0);
static SENSOR_DEVICE_ATTR_RW(fan1_ctrl_mode, fan1_ctrl_mode, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_fw_ver, xdx_fw_ver, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_fw_ver_str, xdx_fw_ver_str, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_vbios, xdx_vbios, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_vbios_str, xdx_vbios_str, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_device_model, xdx_device_model, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_vendor, xdx_vendor, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_chip_id, xdx_chip_id, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_serial_number, xdx_serial_number, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gpu_primary_id, xdx_gpu_primary_id, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gddr_count, xdx_gddr_count, 0);
static SENSOR_DEVICE_ATTR_RO(pwm1_min, pwm1_min, 0);
static SENSOR_DEVICE_ATTR_RO(pwm1_max, pwm1_max, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gpu_primary_core_id, xdx_gpu_primary_core_id, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_gddr_temp, xdx_gddr_temp, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_support_fan_info, xdx_support_fan_info, 0);
static SENSOR_DEVICE_ATTR_RO(xdx_support_power_info, xdx_support_power_info, 0);
static SENSOR_DEVICE_ATTR_RW(xdx_slowdown_temp, xdx_slowdown_temp, 0);
static SENSOR_DEVICE_ATTR_RW(xdx_temp_hyst, xdx_temp_hyst, 0);
static SENSOR_DEVICE_ATTR_RW(xdx_shutdown_temp, xdx_shutdown_temp, 0);
static SENSOR_DEVICE_ATTR_RW(xdx_enable_thm_shutdown, xdx_enable_thm_shutdown, 0);

static struct attribute * xdx_extra_attrs[] = {
	&sensor_dev_attr_xdx_gddr_phys_speed.dev_attr.attr,
	&sensor_dev_attr_xdx_gddr_size.dev_attr.attr,
	&sensor_dev_attr_xdx_gfx_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_xdx_total_power.dev_attr.attr,
	&sensor_dev_attr_xdx_supported_gfx_freq_mhz.dev_attr.attr,
	&sensor_dev_attr_fan1_ctrl_mode.dev_attr.attr,
	&sensor_dev_attr_xdx_fw_ver.dev_attr.attr,
	&sensor_dev_attr_xdx_fw_ver_str.dev_attr.attr,
	&sensor_dev_attr_xdx_vbios.dev_attr.attr,
	&sensor_dev_attr_xdx_vbios_str.dev_attr.attr,
	&sensor_dev_attr_xdx_device_model.dev_attr.attr,
	&sensor_dev_attr_xdx_vendor.dev_attr.attr,
	&sensor_dev_attr_xdx_chip_id.dev_attr.attr,
	&sensor_dev_attr_xdx_serial_number.dev_attr.attr,
	&sensor_dev_attr_xdx_gpu_primary_id.dev_attr.attr,
	&sensor_dev_attr_xdx_gddr_count.dev_attr.attr,
	&sensor_dev_attr_pwm1_min.dev_attr.attr,
	&sensor_dev_attr_pwm1_max.dev_attr.attr,
	&sensor_dev_attr_xdx_gpu_primary_core_id.dev_attr.attr,
	&sensor_dev_attr_xdx_gddr_temp.dev_attr.attr,
	&sensor_dev_attr_xdx_support_fan_info.dev_attr.attr,
	&sensor_dev_attr_xdx_support_power_info.dev_attr.attr,
	&sensor_dev_attr_xdx_slowdown_temp.dev_attr.attr,
	&sensor_dev_attr_xdx_temp_hyst.dev_attr.attr,
	&sensor_dev_attr_xdx_shutdown_temp.dev_attr.attr,
	&sensor_dev_attr_xdx_enable_thm_shutdown.dev_attr.attr,
	NULL
};

static const struct attribute_group hwmon_attrgroup = {
	.attrs = xdx_extra_attrs,
};

static const struct attribute_group *hwmon_groups[] = {
	&hwmon_attrgroup,
	NULL
};

int xdxgpu_pm_sysfs_init(struct xdx_device *xdev)
{
	int ret;

	xdev->hwmon_dev = devm_hwmon_device_register_with_info(xdev->dev,
							 "xdxgpu",
							 xdev,
							 &xdxgpu_hwmon_chip_info,
							 hwmon_groups);
	if (IS_ERR(xdev->hwmon_dev)) {
		ret = PTR_ERR(xdev->hwmon_dev);
		dev_err(xdev->dev,
			"Unable to register hwmon device: %d\n", ret);
		return ret;
	}
	return 0;
}

void xdxgpu_pm_sysfs_fini(struct xdx_device *xdev)
{
	devm_hwmon_device_unregister(xdev->dev);
}
