/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_PM_H__
#define __XDX_PM_H__

#include "xdx_shared.h"
#if defined(SUPPORT_LINUX_DVFS)
#include <linux/devfreq.h>
#endif /* SUPPORT_LINUX_DVFS */

struct xdx_device;

enum xdx_dvfs_freq_level {
	CLK_1500M =0,
	CLK_1400M,
	CLK_1350M,
	CLK_1300M,
	CLK_1250M,
	CLK_1200M,
	CLK_1150M,
	CLK_1100M,
	CLK_1050M,
	CLK_1000M,
	CLK_900M,
	CLK_800M,
	CLK_700M,
	CLK_600M,
	CLK_500M,
	CLK_400M,
	CLK_300M,
	CLK_200M,
	CLK_100M,
	CLK_50M,
	CLK_NUM,
};

const static u32 xdx_gpu_freq_list[CLK_NUM] = {
	1500000000,
	1400000000,
	1350000000,
	1300000000,
	1250000000,
	1200000000,
	1150000000,
	1100000000,
	1050000000,
	1000000000,
	900000000,
	800000000,
	700000000,
	600000000,
	500000000,
	400000000,
	300000000,
	200000000,
	100000000,
	50000000
};

enum xdx_therm_fan_mode {
	XDX_FAN_CTRL_AUTO = 0,
	XDX_FAN_CTRL_USER_SET = 1,
};
enum xdx_temp_ctrl_mode {
	XDX_TEMP_THRESHOLD_AUTO = 0,
	XDX_TEMP_THRESHOLD_USER_SET = 1,
};

enum xdx_power_state xdx_power_state_get(struct xdx_device *xdev);
int xdx_switch_power_state(struct xdx_device *xdev, enum xdx_power_state target);

long xdx_power_get(struct xdx_device *xdev, uint8_t subcmd,
		   uint8_t type, uint8_t channel);
void xdx_power_set(struct xdx_device *xdev, uint8_t subcmd,
		   uint8_t type, uint8_t channel, long val);

void xdx_thermal_set(struct xdx_device *xdev, uint8_t cmd,
		   uint8_t subcmd, uint8_t channel, long val);

void xdx_dvfs_set_freq(struct xdx_device *xdev, u32 gfx_clk);

#if defined(SUPPORT_LINUX_DVFS)

void devfreq_monitor_start(struct devfreq *devfreq);

void devfreq_monitor_stop(struct devfreq *devfreq);

void devfreq_monitor_suspend(struct devfreq *devfreq);

void devfreq_monitor_resume(struct devfreq *devfreq);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 7, 0))
void devfreq_update_interval(struct devfreq *devfreq, unsigned int *delay);
#else
void devfreq_interval_update(struct devfreq *devfreq, unsigned int *delay);
#endif

int devfreq_add_governor(struct devfreq_governor *governor);

int devfreq_remove_governor(struct devfreq_governor *governor);


void xdx_dvfs_enable(struct device *dev);

void xdx_dvfs_disable(struct device *dev);

bool xdx_dvfs_enabled(struct device *dev);
#endif /* SUPPORT_LINUX_DVFS */

int xdx_pm_init(struct xdx_device *xdev);
int xdx_pm_fini(struct xdx_device *xdev);

void xdx_runtime_suspend(struct xdx_device *xdev);
void xdx_runtime_resume(struct xdx_device *xdev);

uint64_t rpm_valid_gddr_offset(struct xdx_device *xdev,
			       uint64_t dev_phy_addr);

#endif /* __XDX_PM_H__ */
