/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include <linux/types.h>
#include <linux/printk.h>
#include "rgxheapconfig.h"
#include "xdx_device.h"
#include "smmu_common.h"
#include "xdx_smmu_drv.h"

#define XDXSMMU_COMGRP_HEAP_BASE        (IMG_UINT64_C(0xF300000000))
#define XDXSMMU_COMGRP_HEAP_SIZE        (IMG_UINT64_C(0x0500000000))

const struct smmu_pxe_config_s xdx_smmu_cfg_pc_4k = {
	.uiBytesPerEntry = 8,
	.uiAddrLog2Align = 12,
	.uiProtMask = XDX_SMMUC_PCE_PROTMASK,
	.uiValidEnMask = XDX_SMMUC_PC_DATA_VALID_EN,
	.uiValidEnShift = XDX_SMMUC_PC_DATA_VALID_SHIFT,
	.uiIndexMask = UINT64_C(0x8000000000),
	.uiIndexShift = XDX_SMMUC_VA_PC_INDEX_SHIFT,
	.uiNumEntries =
		(uint32_t)(UINT64_C(0x8000000000) >> XDX_SMMUC_VA_PC_INDEX_SHIFT) + 1,
};
/*
 * Setup sXDXSMMUPD1Config_4K
 */
const struct smmu_pxe_config_s xdx_smmu_cfg_pd1_4k = {
	.uiBytesPerEntry = 8,
	.uiAddrLog2Align = 12,
	.uiVarCtrlMask = UINT64_C(0x000000000e),
	.uiVarCtrlShift = 1,
	.uiProtMask = XDX_SMMUC_PD_DATA_VALID_EN,
	.uiValidEnMask = XDX_SMMUC_PD_DATA_VALID_EN,
	.uiValidEnShift = XDX_SMMUC_PD_DATA_VALID_SHIFT,
	.uiIndexMask = ~XDX_SMMUC_VA_PD1_INDEX_CLRMSK,
	.uiIndexShift = XDX_SMMUC_VA_PD1_INDEX_SHIFT,
	.uiNumEntries =
	(uint32_t)((~XDX_SMMUC_VA_PD1_INDEX_CLRMSK) >> XDX_SMMUC_VA_PD1_INDEX_SHIFT) + 1,
};

/*
 * Setup sXDXSMMUPD2Config_4K
 */

const struct smmu_pxe_config_s xdx_smmu_cfg_pd2_4k = {
	.uiBytesPerEntry = 8,
	.uiAddrLog2Align = 12,
	.uiProtMask = XDX_SMMUC_PTE_PROTMASK,
	.uiValidEnMask = XDX_SMMUC_PT_DATA_VALID_EN,
	.uiValidEnShift = XDX_SMMUC_PT_DATA_VALID_SHIFT,
	.uiIndexMask = ~XDX_SMMUC_VA_PD2_INDEX_CLRMSK,
	.uiIndexShift = XDX_SMMUC_VA_PD2_INDEX_SHIFT,
	.uiNumEntries =
	(uint32_t)((~XDX_SMMUC_VA_PD2_INDEX_CLRMSK) >> XDX_SMMUC_VA_PD2_INDEX_SHIFT) + 1,
};

/*
 * Setup sXDXSMMUPTConfig_4KB
 */
const struct smmu_pxe_config_s xdx_smmu_cfg_pt_4k = {
	.uiBytesPerEntry = 8,
	.uiAddrLog2Align = 12,
	.uiProtMask = XDX_SMMUC_PTE_PROTMASK,
	.uiValidEnMask = XDX_SMMUC_PT_DATA_VALID_EN,
	.uiValidEnShift = XDX_SMMUC_PT_DATA_VALID_SHIFT,
	.uiIndexMask = ~XDX_SMMUC_VA_PT_INDEX_CLRMSK,
	.uiIndexShift = XDX_SMMUC_VA_PT_INDEX_SHIFT,
	.uiNumEntries =
	(uint32_t)((~XDX_SMMUC_VA_PT_INDEX_CLRMSK) >> XDX_SMMUC_VA_PT_INDEX_SHIFT) + 1,
};

const struct smmu_page_config_s smmu_page_config_4k = {
	.pxe_cfgs = {
		&xdx_smmu_cfg_pt_4k,
		&xdx_smmu_cfg_pd2_4k,
		&xdx_smmu_cfg_pd1_4k,
		&xdx_smmu_cfg_pc_4k
	},
	.top_level = SMMU_LEVEL_4,
	.page_size = SIZE_4KB,
	.page_mask = UINT64_C(0x0000000fff),
	.page_shift = TTL3_OA_SHIFT,
	.offset_in_page = 0,
};

struct smmu_group_cfg_s {
	const struct smmu_page_config_s *page_cfgs;
	const struct smmu_grp_para_s *grp_para;
};

static uint32_t xdx_smmu_com_gsids_b0[] = {
	GSID_ID(TDMA_TCU, 1),
	// GSID_ID(VDMA_TCU, 1),
	// GSID_ID(VDMA_TCU, 2),
	// GSID_ID(XDMA_TCU, 0),
#ifdef SUPPORT_VPU_SMMU
	GSID_ID(VDEC_TCU, 0),
	GSID_ID(VENC_TCU, 0),
#endif
};

const struct smmu_grp_para_s xdx_smmu_b0_comgrp_para = {
	.gsid_tab = xdx_smmu_com_gsids_b0,
	.gsid_cnt = ARRAY_SIZE(xdx_smmu_com_gsids_b0),
	.flags = 0,
	.vaddr_rang_start = XDXSMMU_COMGRP_HEAP_BASE,
	.vaddr_rang_size = XDXSMMU_COMGRP_HEAP_SIZE,
};

struct smmu_group_cfg_s xdx_smmu_b0_comgrp_cfg = {
	.page_cfgs = &smmu_page_config_4k,
	.grp_para = &xdx_smmu_b0_comgrp_para,
};

static uint32_t xdx_smmu_com_gsids_a0[] = {
//	GSID_ID(VDMA_TCU, 1),
//	GSID_ID(VDMA_TCU, 2),
//	GSID_ID(XDMA_TCU, 0),
};

const struct smmu_grp_para_s xdx_smmu_a0_comgrp_para = {
	.gsid_tab = xdx_smmu_com_gsids_a0,
	.gsid_cnt = ARRAY_SIZE(xdx_smmu_com_gsids_a0),
	.flags = 0,
	.vaddr_rang_start = XDXSMMU_COMGRP_HEAP_BASE,
	.vaddr_rang_size = XDXSMMU_COMGRP_HEAP_SIZE,
};

struct smmu_group_cfg_s xdx_smmu_a0_comgrp_cfg = {
	.page_cfgs = &smmu_page_config_4k,
	.grp_para = &xdx_smmu_a0_comgrp_para,
};

static int xdx_smmu_init_com_context(struct xdx_device *xdev)
{
	int ret;
	enum xdx_asic_type asic_type;
	struct smmu_group_cfg_s *grp_cfg = NULL;

	asic_type = xdev_asic_type(xdev);
	switch (asic_type) {
	case CHIP_PANGU_B0:
		grp_cfg = &xdx_smmu_b0_comgrp_cfg;
		break;
	case CHIP_PANGU_A0:
		grp_cfg = &xdx_smmu_a0_comgrp_cfg;
		break;
	default:
		ret = -EOPNOTSUPP;
		return ret;
	}

	ret = xdx_smmu_ctx_creat(xdev, grp_cfg->grp_para, grp_cfg->page_cfgs);
	if (ret < 0) {
		dev_err(xdev->dev, "xdx_smmu_ctx_creat (%d)", ret);
		return ret;
	}
	return ret;
}

int xdx_smmu_plat_init(struct xdx_device *xdev)
{
	int ret;

	ret = xdx_smmu_init_com_context(xdev);
	if (ret < 0) {
		dev_err(xdev->dev, "xdx_smmu_init_com_context failed.(%d) ", ret);
		return ret;
	}
	return 0;
}

int xdx_smmu_plat_deinit(struct xdx_device *xdev)
{
	struct smmu_context_s *_ctx, *ctx;
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);

	list_for_each_entry_safe(ctx, _ctx, &smmu->ctxs_head, list) {
		xdx_smmu_ctx_destroy(ctx);
	}
	return 0;
}

int xdx_smmu_plat_resume(struct xdx_device *xdev)
{
	struct smmu_context_s *ctx;
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);

	list_for_each_entry(ctx, &smmu->ctxs_head, list) {
		xdx_smmu_group_config_pagetab(xdx_smmu_ctx_group(ctx),
				ctx->base_lvl_info->sMemDesc.dev_phy_addr.uiAddr);
		dev_dbg(xdev->dev, "xdx_smmu plat resume ttbr:%llx",
				ctx->base_lvl_info->sMemDesc.dev_phy_addr.uiAddr);
	}

	return 0;
}

