/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include "xdx_device.h"
#include "xdx_gem.h"
#include "hantro_drm.h"
#include "hantro_fpga.h"
#include "vpu_smmu.h"
#include "hantro_priv.h"
#include "kernel_compatibility.h"
#if KERNEL_VERSION(5, 13, 0) <= LINUX_VERSION_CODE
static const struct drm_gem_object_funcs hantro_drm_gem_cma_funcs;
#endif

static int xdx_kref_read(struct kref *kref)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
	return kref_read(kref);
#else
	return atomic_read(&kref->refcount);
#endif
}

void hantro_drm_fb_destroy(struct drm_framebuffer *fb)
{
	struct hantro_drm_fb *vsi_fb = (struct hantro_drm_fb *)fb;
	int i;

	for (i = 0; i < 4; i++)
		hantro_unref_drmobj(vsi_fb->obj[i]);

	drm_framebuffer_cleanup(fb);
	kfree(vsi_fb);
}

int hantro_drm_fb_create_handle(struct drm_framebuffer *fb,
				       struct drm_file *file_priv,
				       unsigned int *handle)
{
	struct hantro_drm_fb *vsi_fb = (struct hantro_drm_fb *)fb;

	return drm_gem_handle_create(file_priv, vsi_fb->obj[0], handle);
}

int hantro_drm_fb_dirty(struct drm_framebuffer *fb,
			       struct drm_file *file, unsigned int flags,
			       unsigned int color, struct drm_clip_rect *clips,
			       unsigned int num_clips)
{
	/*nothing to do now*/
	return 0;
}

const struct drm_framebuffer_funcs hantro_drm_fb_funcs = {
	.destroy = hantro_drm_fb_destroy,
	.create_handle = hantro_drm_fb_create_handle,
	.dirty = hantro_drm_fb_dirty,
};

#if 0
static int hantro_gem_dumb_create_combine(struct drm_file *file_priv,
					   struct drm_device *dev,
					   union drm_xdxgpu_gem_create *args)
{
	int ret = 0;
	struct drm_gem_hantro_object *cma_obj;
	struct drm_gem_object *obj;
	struct xdx_bo *vpu_bo_ptr;
	struct xdx_device *xdev = drm_to_xdev(dev);

//	int min_pitch = DIV_ROUND_UP(args->width * args->bpp, 8);
	unsigned int sliceidx = args->in.alignment;
	unsigned int config;

	struct slice_info *pslice = getslicenode(sliceidx);

	if (!pslice)
		return -EINVAL;

	config = pslice->config;

//	args->handle = 0;
	if (mutex_lock_interruptible(&dev->struct_mutex))
		return -EBUSY;

	cma_obj = kzalloc(sizeof(*cma_obj), GFP_KERNEL);
	if (!cma_obj) {
		ret = -ENOMEM;
		goto out;
	}

	cma_obj->dmapriv.self = cma_obj;

	cma_obj->num_pages = args->in.bo_size >> PAGE_SHIFT;
	cma_obj->flag = 0;
	cma_obj->pageaddr = NULL;
	cma_obj->vaddr = NULL;
	cma_obj->sliceidx = sliceidx;

	ret = xdx_bo_create_reserved(xdev,PAGE_ALIGN(args->in.bo_size),PAGE_SIZE,XDXGPU_PL_VRAM,
		&vpu_bo_ptr, &cma_obj->paddr,
		(void **)&cma_obj->vaddr);
	if (ret) {
		dev_err(xdev->dev, "Failed to alloc memory bo_size:0x%llx\n",PAGE_ALIGN(args->in.bo_size));
		ret = -ENOMEM;
		goto out;
	}

	XDX_DEV_DEBUG("cma_obj->paddr:0x%llx size:0x%llx \n",cma_obj->paddr,PAGE_ALIGN(args->in.bo_size));

	cma_obj->paddr = cma_obj->paddr - PANGU_VRAM_DEV_PHYADDR_BASE + pci_resource_start(xdev->pdev, 2);
	cma_obj->mem_base = pci_resource_start(xdev->pdev, 2);

	vpu_bo_ptr->cma_obj = cma_obj;
	obj = &vpu_bo_ptr->base;

	if (!cma_obj->vaddr) {
		kfree(cma_obj);
		ret = -ENOMEM;
		goto out;
	}

#if KERNEL_VERSION(5, 13, 0) <= LINUX_VERSION_CODE
	if (!obj->funcs)
		obj->funcs = &hantro_drm_gem_cma_funcs;
#endif

	ret = drm_gem_handle_create(file_priv, &vpu_bo_ptr->base, &args->out.handle);
	hantro_unref_drmobj(&vpu_bo_ptr->base);
	if (ret) {
		kfree(cma_obj);
	}
	init_hantro_resv(&cma_obj->kresv, cma_obj);
	cma_obj->handle = args->out.handle;

out:
	mutex_unlock(&dev->struct_mutex);

	return ret;
}
#endif

int hantro_gem_dumb_create_internal(struct drm_file *file_priv,
					   struct drm_device *dev,
					   struct drm_mode_create_dumb *args)
{
	int ret = 0;
	int in_size, out_size;
	struct drm_gem_hantro_object *cma_obj;
	struct drm_gem_object *obj;

	int min_pitch = DIV_ROUND_UP(args->width * args->bpp, 8);
	unsigned int sliceidx = args->handle;
	unsigned int config;
	struct xdx_bo *vpu_bo_ptr;
	struct xdx_device *xdev = drm_to_xdev(dev);


	struct slice_info *pslice = getslicenode(sliceidx);

	if (!pslice)
		return -EINVAL;

	config = pslice->config;

	args->handle = 0;
	if (mutex_lock_interruptible(&dev->struct_mutex))
		return -EBUSY;

	cma_obj = kzalloc(sizeof(*cma_obj), GFP_KERNEL);
	if (!cma_obj) {
		ret = -ENOMEM;
		goto out;
	}
	obj = &cma_obj->base;
	cma_obj->dmapriv.self = cma_obj;
	in_size = sizeof(*args);
	out_size = in_size;
	args->pitch = ALIGN(min_pitch, 64);
	args->size = (__u64)args->pitch * (__u64)args->height;
	args->size = PAGE_ALIGN(args->size);

	cma_obj->num_pages = args->size >> PAGE_SHIFT;
	cma_obj->flag = 0;
	cma_obj->pageaddr = NULL;
	cma_obj->vaddr = NULL;
	cma_obj->sliceidx = sliceidx;

/*CMA is temp disabled here for these reasons:
 *1. to bring up CMA, CONFIG_DMA_CMA should be enabled in kernel building
 *2. dma_alloc/release_from_contiguous should be exported in kernel code
 *3. a boot parameter like "cma=268435456@134217728" should be added
 *4. CMA's memory management is not stable.
 *We could choose to use CMA or page alloc by module parameter later.
 */
#if USE_CMA
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
	cma_obj->pageaddr = dma_alloc_from_contiguous(
		pslice->dev, args->size >> PAGE_SHIFT, 1);
#else
	cma_obj->pageaddr = dma_alloc_from_contiguous(
		pslice->dev, args->size >> PAGE_SHIFT, 1, GFP_KERNEL);
#endif
	if (!cma_obj->pageaddr) {
		kfree(cma_obj);
		ret = -ENOMEM;
		goto out;
	}
	cma_obj->vaddr = page_to_virt(cma_obj->pageaddr);
	cma_obj->paddr = virt_to_phys(cma_obj->vaddr);
#else
#ifndef VSI_FPGA

	if (config & CONFIG_HANTROMMU) {
		cma_obj->vaddr = vmalloc(args->size);
		cma_obj->paddr = page_to_phys(vmalloc_to_page(cma_obj->vaddr));
		cma_obj->flag |= HANTRO_GEM_FLAG_USEVMALLOC;

	} else {
		cma_obj->vaddr = dma_alloc_coherent(pslice->dev, args->size,
						    &cma_obj->paddr,
						    GFP_KERNEL | GFP_DMA);
	}
#else
#if 0
	struct hantro_mem_handle fpga_phandle;

	if (hantro_fpga_memalloc(&fpga_phandle, args->size) != 0)
		pr_debug("%s,%d ret error\n", __func__, __LINE__);
	cma_obj->vaddr = fpga_phandle.vaddr;
	cma_obj->paddr = fpga_phandle.paddr;
	cma_obj->mem_base = fpga_phandle.mem_base;
#endif

	ret = xdx_bo_create_reserved(xdev,args->size,PAGE_SIZE,XDXGPU_PL_VRAM,
			&vpu_bo_ptr, &cma_obj->paddr,
			(void **)&cma_obj->vaddr);
	if (ret) {
		dev_err(xdev->dev, "Failed to alloc memory\n");
		ret = -ENOMEM;
		goto out;
	}

	cma_obj->paddr = cma_obj->paddr - PANGU_VRAM_DEV_PHYADDR_BASE + pci_resource_start(xdev->pdev, 2);
	cma_obj->mem_base = pci_resource_start(xdev->pdev, 2);;

	vpu_bo_ptr->cma_obj = cma_obj;
	obj = &vpu_bo_ptr->base;
#endif
	if (!cma_obj->vaddr) {
		kfree(cma_obj);
		ret = -ENOMEM;
		goto out;
	}
#endif
#if 1
#if KERNEL_VERSION(5, 13, 0) <= LINUX_VERSION_CODE
	if (!obj->funcs)
		obj->funcs = &hantro_drm_gem_cma_funcs;
#endif
//	drm_gem_object_init(dev, obj, args->size);
#endif

	ret = drm_gem_handle_create(file_priv, &vpu_bo_ptr->base, &args->handle);
	hantro_unref_drmobj(&vpu_bo_ptr->base);
	if (ret) {
#if USE_CMA
		dma_release_from_contiguous(pslice->dev, cma_obj->pageaddr,
					    cma_obj->num_pages);
#else
#ifndef VSI_FPGA
		if (cma_obj->flag & HANTRO_GEM_FLAG_USEVMALLOC)
			vfree(cma_obj->vaddr);
		else
			dma_free_coherent(pslice->dev, args->size,
					  cma_obj->vaddr, cma_obj->paddr);
#else
		struct hantro_mem_handle fpga_phandle;

		fpga_phandle.vaddr = cma_obj->vaddr;
		fpga_phandle.paddr = cma_obj->paddr;
		hantro_fpga_memfree(&fpga_phandle);
#endif
#endif
		kfree(cma_obj);
	}
	init_hantro_resv(&cma_obj->kresv, cma_obj);
	cma_obj->handle = args->handle;
out:
	mutex_unlock(&dev->struct_mutex);
	return ret;
}

int hantro_gem_dumb_create(struct drm_device *dev, void *data,
				  struct drm_file *file_priv)
{
	return hantro_gem_dumb_create_internal(
		file_priv, dev, (struct drm_mode_create_dumb *)data);
}

int hantro_gem_dumb_map_offset(struct drm_file *file_priv,
				      struct drm_device *dev, uint32_t handle,
				      uint64_t *offset)
{
	struct drm_gem_object *obj;
	int ret;

	obj = hantro_gem_object_lookup(dev, file_priv, handle);
	if (!obj)
		return -EINVAL;

	ret = drm_gem_create_mmap_offset(obj);
	if (ret == 0) {
		*offset = (drm_vma_node_offset_addr(&obj->vma_node) + VSI_MMAP_ADDRES_CEIL);
		hantro_mmaplog("create normal map offset %llx", *offset);
	}

	hantro_unref_drmobj(obj);
	return ret;
}


int hantro_gem_dumb_destroy(struct drm_file *file,
				struct drm_device *dev, u32 handle)
{
	return drm_gem_handle_delete(file, handle);
}

int hantro_destroy_dumb(struct drm_device *dev, void *data,
			       struct drm_file *file_priv)
{
	struct drm_mode_destroy_dumb *args = data;
	struct drm_gem_object *obj;
	struct drm_gem_hantro_object *cma_obj;

	if (mutex_lock_interruptible(&dev->struct_mutex))
		return -EBUSY;
	obj = hantro_gem_object_lookup(dev, file_priv, args->handle);
	if (!obj) {
		mutex_unlock(&dev->struct_mutex);
		return -EINVAL;
	}
	hantro_unref_drmobj(obj);

	cma_obj = to_drm_gem_hantro_obj(obj);

	drm_gem_handle_delete(file_priv, args->handle);
	mutex_unlock(&dev->struct_mutex);
	return 0;
}

struct sg_table *
hantro_gem_prime_get_sg_table(struct drm_gem_object *obj)
{
	struct drm_gem_hantro_object *cma_obj = to_drm_gem_hantro_obj(obj);
	struct slice_info *pslice = getslicenode(cma_obj->sliceidx);
	struct sg_table *sgt;
	int ret;

	if (!pslice)
		return NULL;
	sgt = kzalloc(sizeof(*sgt), GFP_KERNEL);
	if (!sgt)
		return NULL;

	ret = dma_get_sgtable(pslice->dev, sgt, cma_obj->vaddr, cma_obj->paddr,
			      obj->size);
	if (ret < 0) {
		kfree(sgt);
		sgt = NULL;
	}
	return sgt;
}

struct drm_gem_object *
hantro_gem_prime_import_sg_table(struct drm_device *dev,
				 struct dma_buf_attachment *attach,
				 struct sg_table *sgt)
{
	struct drm_gem_hantro_object *cma_obj;
	struct drm_gem_object *obj;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,18,0))
	struct dma_buf_map map;
#else
	struct iosys_map map;
#endif
	struct xdx_device *xdev = drm_to_xdev(dev);

	cma_obj = kzalloc(sizeof(*cma_obj), GFP_KERNEL);
	if (!cma_obj)
		return ERR_PTR(-ENOMEM);

	obj = &cma_obj->base;

	if (!(xdev->vpu.config & CONFIG_HANTROMMU)
		&& sgt->nents > 1) {
		/* check if the entries in the sg_table are contiguous */
		dma_addr_t next_addr = sg_dma_address(sgt->sgl);
		struct scatterlist *s;
		unsigned int i;

		for_each_sg(sgt->sgl, s, sgt->nents, i) {
			/*
			 * sg_dma_address(s) is only valid for entries
			 * that have sg_dma_len(s) != 0
			 */
			if (!sg_dma_len(s))
				continue;

			if (sg_dma_address(s) != next_addr) {
				kfree(cma_obj);
				return ERR_PTR(-EINVAL);
			}

			next_addr = sg_dma_address(s) + sg_dma_len(s);
		}
	}

#if KERNEL_VERSION(5, 13, 0) <= LINUX_VERSION_CODE
	if (!obj->funcs)
		obj->funcs = &hantro_drm_gem_cma_funcs;
#endif

	if (drm_gem_object_init(dev, obj, attach->dmabuf->size) != 0) {
		kfree(cma_obj);
		return ERR_PTR(-ENOMEM);
	}
	cma_obj->paddr = sg_dma_address(sgt->sgl);
	dma_buf_vmap(attach->dmabuf, &map);
	cma_obj->vaddr = map.vaddr;
	cma_obj->sgt = sgt;
	cma_obj->num_pages = attach->dmabuf->size >> PAGE_SHIFT;
	cma_obj->dmapriv.meta_data =
		*((struct viv_vidmem_metadata *)attach->dmabuf->priv);
	cma_obj->dmapriv.self = cma_obj;
	cma_obj->dmapriv.meta_data.magic = HANTRO_IMAGE_VIV_META_DATA_MAGIC;
	cma_obj->flag |= HANTRO_GEM_FLAG_IMPORT;
	if (xdev->vpu.config & CONFIG_HANTROMMU)
		cma_obj->flag |= HANTRO_GEM_FLAG_USEVMALLOC; //make it use MMU

	return obj;
}

#if LINUX_VERSION_CODE<KERNEL_VERSION(5, 11, 0)
void *hantro_gem_prime_vmap(struct drm_gem_object *obj)
{
	struct drm_gem_hantro_object *cma_obj = to_drm_gem_hantro_obj(obj);

	return cma_obj->vaddr;
}

void hantro_gem_prime_vunmap(struct drm_gem_object *obj, void *vaddr)
{
}

#elif ((LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))&&(LINUX_VERSION_CODE < KERNEL_VERSION(5,18,0)))
int hantro_gem_prime_vmap(struct drm_gem_object *obj,
				  struct dma_buf_map *map)
{
	struct drm_gem_hantro_object *cma_obj = to_drm_gem_hantro_obj(obj);

	dma_buf_map_set_vaddr(map, cma_obj->vaddr);

	return 0;
}
void hantro_gem_prime_vunmap(struct drm_gem_object *obj,
				    struct dma_buf_map *map)
{
}
#else
int hantro_gem_prime_vmap(struct drm_gem_object *obj,
				  struct iosys_map *map)
{
	struct drm_gem_hantro_object *cma_obj = to_drm_gem_hantro_obj(obj);

	iosys_map_set_vaddr(map, cma_obj->vaddr);

	return 0;
}
void hantro_gem_prime_vunmap(struct drm_gem_object *obj,
				    struct iosys_map *map)
{
}
#endif

/* omitted in kernel version > 5.4.0
 *static struct reservation_object *hantro_gem_prime_res_obj(
 *	struct drm_gem_object *obj)
 *{
 *	struct drm_gem_hantro_object *hobj = to_drm_gem_hantro_obj(obj);
 *
 *	return &hobj->kresv;
 *}
 */

int hantro_gem_prime_mmap(struct drm_gem_object *obj,
				 struct vm_area_struct *vma)
{
	struct drm_gem_hantro_object *cma_obj;
	unsigned long page_num = (vma->vm_end - vma->vm_start) >> PAGE_SHIFT;
	int ret = 0;

	cma_obj = to_drm_gem_hantro_obj(obj);
	if (page_num > cma_obj->num_pages)
		return -EINVAL;

	if ((unsigned long)cma_obj->vaddr == 0)
		return -EINVAL;

	ret = drm_gem_mmap_obj(obj, obj->size, vma);
	if (ret < 0)
		return ret;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0))
	vma->vm_flags &= ~VM_PFNMAP;
#else
	vm_flags_clear(vma, VM_PFNMAP);
#endif
	vma->vm_pgoff = 0;
	if (mutex_lock_interruptible(&hantro_dev.drm_dev->struct_mutex))
		return -EBUSY;
	if (dma_mmap_coherent(obj->dev->dev, vma, cma_obj->vaddr,
			      cma_obj->paddr, vma->vm_end - vma->vm_start)) {
		drm_gem_vm_close(vma);
		mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
		return -EAGAIN;
	}
	mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
	vma->vm_private_data = cma_obj;
	return ret;
}

void hantro_gem_free_object(struct drm_gem_object *gem_obj)
{
	struct drm_gem_hantro_object *cma_obj;
	struct slice_info *pslice;
	struct hantro_mem_handle fpga_phandle;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 18, 0))
	struct dma_buf_map map;
#else
	struct iosys_map map;
#endif
	/* dma buf imported from others,
	 * release data structures allocated by ourselves
	 */
	cma_obj = to_drm_gem_hantro_obj(gem_obj);

	if (gem_obj->import_attach) {
		if (cma_obj->vaddr) {
			map.vaddr = cma_obj->vaddr;
			dma_buf_vunmap(gem_obj->import_attach->dmabuf,
				       &map);
		}
		drm_prime_gem_destroy(gem_obj, cma_obj->sgt);
	} else if (cma_obj->vaddr) {
		pslice = getslicenode(cma_obj->sliceidx);
		if (!pslice)
			return;
#if USE_CMA
		dma_release_from_contiguous(pslice->dev, cma_obj->pageaddr,
					    cma_obj->num_pages);
#else
#ifndef VSI_FPGA

		if (cma_obj->flag & HANTRO_GEM_FLAG_USEVMALLOC)
			vfree(cma_obj->vaddr);
		else
			dma_free_coherent(pslice->dev, cma_obj->base.size,
					  cma_obj->vaddr, cma_obj->paddr);
#else

		fpga_phandle.vaddr = cma_obj->vaddr;
		fpga_phandle.paddr = cma_obj->paddr;
		hantro_fpga_memfree(&fpga_phandle);
#endif
#endif
	}
	dma_resv_fini(&cma_obj->kresv);
	drm_gem_object_release(gem_obj);
	kfree(cma_obj);
}

int hantro_gem_close(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct drm_gem_close *args = data;
	int ret = 0;
	struct drm_gem_object *obj =
		hantro_gem_object_lookup(dev, file_priv, args->handle);

	if (!obj)
		return -EINVAL;

	ret = drm_gem_handle_delete(file_priv, args->handle);
	hantro_unref_drmobj(obj);
	return ret;
}

int hantro_gem_open(struct drm_device *dev, void *data,
			   struct drm_file *file_priv)
{
	int ret;
	u32 handle;
	struct drm_gem_open *openarg;
	struct drm_gem_object *obj = NULL;

	openarg = (struct drm_gem_open *)data;

	obj = idr_find(&dev->object_name_idr, (int)openarg->name);
	if (obj)
		hantro_ref_drmobj(obj);
	else
		return -ENOENT;

	ret = drm_gem_handle_create(file_priv, obj, &handle);
	
	hantro_unref_drmobj(obj);
	if (ret)
		return ret;

	openarg->handle = handle;
	openarg->size = obj->size;

	return ret;
}
#if 1
typedef struct DUMMY_PMR_LMALLOCARRAY_DATA {
	uint32_t uiPid;
	uint32_t iNumPagesAllocated;
	uint32_t uiTotalNumPages;
	uint32_t uiPagesToAlloc;

	uint32_t uiLog2AllocSize;
	uint32_t uiContigAllocSize;
	uint64_t *pasDevPAddr;
	uint32_t bZeroOnAlloc;
	uint32_t bPoisonOnAlloc;

	uint32_t bOnDemand;

	uint32_t bPoisonOnFree;
	uint32_t padding_array[100];//fulongdbg padding here to aoid leakage

}DUMMY_PMR_LMALLOCARRAY_DATA;
#endif
int hantro_map_vaddr(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct hantro_addrmap *pamap = data;
	struct drm_gem_object *obj;
	struct xdx_device *xdev = drm_to_xdev(dev);
	struct xdx_bo *bo;
	int ret;
	//struct drm_gem_hantro_object *cma_obj;
	
	obj = hantro_gem_object_lookup(dev, file_priv, pamap->handle);
        if (!obj) {
            dev_err(dev->dev, "Failed to lookup GEM object.\n");
            return  -ENXIO;
        }

        bo = to_xdx_gem(obj);

	ret = xdx_bo_reserve(bo, false);
	if (ret) {
	    dev_err(dev->dev, "(%d) failed to reserve kernel bo\n", ret);

	    return ret;
	}

	if (!xdx_bo_get_pages(bo)) {
	    ret = -ENOMEM;

	    return ret;
	 }

	pamap->phy_addr = xdx_bo_gpu_offset(bo) - PANGU_VRAM_DEV_PHYADDR_BASE + pci_resource_start(xdev->pdev, 2);
	pamap->mem_base = pci_resource_start(xdev->pdev, 2);
	xdx_bo_unreserve(bo);
	hantro_unref_drmobj(obj);

	return 0;
}

int hantro_get_hwcfg(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct xdx_device *xdev = drm_to_xdev(dev);
	return xdev->vpu.config;
}

int hantro_get_slicenum(struct drm_device *dev, void *data,
			       struct drm_file *file_priv)
{
	int vcmd_en;
	hantro_ioctl_id ioctl_id_par;

	ioctl_id_par.ID_PAR.node_idx = 0;
	vcmd_en = hantro_get_vcmdsup(NULL, &ioctl_id_par.data, NULL);

	if (vcmd_en == 0)
		return get_slicenumber();
	else
		return get_vcmd_slice_num();
}

/*reference linux 4.11. ubuntu 16.04 have issues in its drm_gem_flink_ioctl().
 * MODIFICATION:
 * drm_gem_object_lookup(file_priv, args->handle);
 * => drm_gem_object_lookup(dev, file_priv, args->handle);
 */
int hantro_gem_flink(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct drm_gem_flink *args = data;
	struct drm_gem_object *obj;
	int ret;

	if (!drm_core_check_feature(dev, DRIVER_GEM))
		return -ENODEV;

	obj = hantro_gem_object_lookup(dev, file_priv, args->handle);
	if (!obj)
		return -ENOENT;

	mutex_lock(&dev->object_name_lock);
	/* prevent races with concurrent gem_close. */
	if (obj->handle_count == 0) {
		ret = -ENOENT;
		goto err;
	}

	if (!obj->name) {
		ret = idr_alloc(&dev->object_name_idr, obj, 1, 0, GFP_KERNEL);
		if (ret < 0)
			goto err;

		obj->name = ret;
	}

	args->name = (uint64_t)obj->name;
	ret = 0;

err:
	mutex_unlock(&dev->object_name_lock);
	hantro_unref_drmobj(obj);
	return ret;
}

int hantro_map_dumb(struct drm_device *dev, void *data,
			   struct drm_file *file_priv)
{
	int ret;
	struct drm_mode_map_dumb *temparg = (struct drm_mode_map_dumb *)data;

	ret = hantro_gem_dumb_map_offset(file_priv, dev, temparg->handle,
					 &temparg->offset);

	return ret;
}

int hantro_drm_open(struct drm_device *dev, struct drm_file *file)
{
	struct idr *ptr;

	ptr = kzalloc(sizeof(*ptr), GFP_KERNEL);
	if (!ptr)
		return -ENOMEM;
	idr_init(ptr);
	file->driver_priv = ptr;
	return 0;
}

struct drm_gem_object *
hantro_drm_gem_prime_import(struct drm_device *dev, struct dma_buf *dma_buf)
{
	struct device *attach_dev = dev->dev;
	struct dma_buf_attachment *attach;
	struct sg_table *sgt;
	struct drm_gem_object *obj;
	#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0)) && (!defined(REDHAT_83))
	struct xdx_bo *bo;
	#endif
	int ret;

	if (dma_buf->ops == &hantro_dmabuf_ops) {
		obj = hantro_get_gem_from_dmabuf(dma_buf);
		if (obj && obj->dev == dev) {
			drm_gem_object_get(obj);
			return obj;
		}
	}

	if (!dev->driver->gem_prime_import_sg_table)
		return ERR_PTR(-EINVAL);

	attach = dma_buf_attach(dma_buf, attach_dev);
	if (IS_ERR(attach))
		return ERR_CAST(attach);

	get_dma_buf(dma_buf);

	sgt = dma_buf_map_attachment(attach, DMA_BIDIRECTIONAL);
	if (IS_ERR(sgt)) {
		ret = PTR_ERR(sgt);
		goto fail_detach;
	}

	obj = dev->driver->gem_prime_import_sg_table(dev, attach, sgt);
	if (IS_ERR(obj)) {
		ret = PTR_ERR(obj);
		goto fail_unmap;
	}

	obj->import_attach = attach;
	#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0)) || (defined(REDHAT_83))
	obj->resv = dma_buf->resv;	
	#else
	bo = to_xdx_gem(obj);
	bo->resv = dma_buf->resv;	
	#endif

	return obj;

fail_unmap:
	dma_buf_unmap_attachment(attach, sgt, DMA_BIDIRECTIONAL);
fail_detach:
	dma_buf_detach(dma_buf, attach);
	dma_buf_put(dma_buf);

	return ERR_PTR(ret);
}

struct dma_buf *hantro_prime_export(struct drm_gem_object *obj,
					   int flags)
{
	struct dma_buf *dma_buf;
	struct drm_gem_hantro_object *cma_obj;
	struct drm_device *dev = obj->dev;
	struct dma_buf_export_info exp_info = {
		.exp_name = KBUILD_MODNAME,
		.owner = dev->driver->fops->owner,
		.ops = &hantro_dmabuf_ops,
		.flags = flags,
	};

	cma_obj = to_drm_gem_hantro_obj(obj);
	exp_info.resv = &cma_obj->kresv;
	exp_info.size = cma_obj->num_pages << PAGE_SHIFT;
	exp_info.priv = &cma_obj->dmapriv.meta_data;

	dma_buf = dma_buf_export(&exp_info);
	if (IS_ERR(dma_buf))
		return dma_buf;

	drm_dev_get(dev);
	drm_gem_object_get(obj);
	//dma_buf->file->f_mapping = obj->dev->anon_inode->i_mapping;
	//just a remind

	return dma_buf;
}

int hantro_handle_to_fd(struct drm_device *dev, void *data,
			       struct drm_file *file_priv)
{
	int ret;
	struct drm_prime_handle *primeargs = (struct drm_prime_handle *)data;
	struct drm_gem_object *obj;
	struct drm_gem_hantro_object *cma_obj;

	obj = hantro_gem_object_lookup(dev, file_priv, primeargs->handle);
	if (!obj)
		return -ENOENT;

	ret = drm_gem_prime_handle_to_fd(dev, file_priv, primeargs->handle,
					 primeargs->flags, &primeargs->fd);

	if (ret == 0) {
		cma_obj = to_drm_gem_hantro_obj(obj);
		if (cma_obj)
			cma_obj->flag |= HANTRO_GEM_FLAG_EXPORT;
	}
	hantro_unref_drmobj(obj);
	return ret;
}

int hantro_fd_to_handle(struct drm_device *dev, void *data,
			       struct drm_file *file_priv)
{
	struct drm_prime_handle *primeargs = (struct drm_prime_handle *)data;
	s32 ret = 0;
	struct dma_buf *dma_buf;
	struct drm_gem_object *obj;
	struct drm_gem_hantro_object *cma_obj;

	primeargs->flags = 0;
	ret = drm_gem_prime_fd_to_handle(dev, file_priv, primeargs->fd,
					 &primeargs->handle);

	/*Just for debugging to get object*/
	if (ret == 0) {
		dma_buf = dma_buf_get(primeargs->fd);
		if (IS_ERR(dma_buf))
			return PTR_ERR(dma_buf);
		obj = dma_buf->priv;
		cma_obj = to_drm_gem_hantro_obj(obj);
		dma_buf_put(dma_buf);
		pr_debug("client[%p]import  gem name[%u] gem size[%zu] handle_cnt[%d] refcnt[%d] flag[0x%x]\n",
			file_priv, obj->name, obj->size, obj->handle_count,
			xdx_kref_read(&obj->refcount), cma_obj->flag);
	}

	return ret;
}

int hantro_fb_create2(struct drm_device *dev, void *data,
			     struct drm_file *file_priv)
{
	struct drm_mode_fb_cmd2 *mode_cmd = (struct drm_mode_fb_cmd2 *)data;
	struct hantro_drm_fb *vsifb;
	struct drm_gem_object *objs[4];
	struct drm_gem_object *obj;
#if KERNEL_VERSION(4, 16, 0) <= LINUX_VERSION_CODE
	const struct drm_format_info *info = drm_get_format_info(dev, mode_cmd);
#endif
	unsigned int hsub;
	unsigned int vsub;
	int num_planes;
	int ret;
	int i;

#if KERNEL_VERSION(4, 16, 0) <= LINUX_VERSION_CODE
	hsub = info->hsub;
	vsub = info->vsub;
	num_planes = min_t(int, info->num_planes, 4);
#else
	hsub = drm_format_horz_chroma_subsampling(mode_cmd->pixel_format);
	vsub = drm_format_vert_chroma_subsampling(mode_cmd->pixel_format);
	num_planes = min(drm_format_num_planes(mode_cmd->pixel_format), 4);
#endif
	for (i = 0; i < num_planes; i++) {
		unsigned int width = mode_cmd->width / (i ? hsub : 1);
		unsigned int height = mode_cmd->height / (i ? vsub : 1);
		unsigned int min_size;

		obj = hantro_gem_object_lookup(dev, file_priv,
					       mode_cmd->handles[i]);
		if (!obj) {
			ret = -ENXIO;
			goto err_gem_object_unreference;
		}
		hantro_unref_drmobj(obj);
		min_size = (height - 1) * mode_cmd->pitches[i] +
			   mode_cmd->offsets[i] +
#if KERNEL_VERSION(4, 16, 0) <= LINUX_VERSION_CODE
			   width * info->cpp[i];
#else
			   width * drm_format_plane_cpp(mode_cmd->pixel_format,
							i);
#endif
		if (obj->size < min_size) {
			//hantro_unref_drmobj(obj);
			ret = -EINVAL;
			goto err_gem_object_unreference;
		}
		objs[i] = obj;
	}
	vsifb = kzalloc(sizeof(*vsifb), GFP_KERNEL);
	if (!vsifb)
		return -ENOMEM;
	xdx_drm_helper_mode_fill_fb_struct(dev, &vsifb->fb, mode_cmd);
	for (i = 0; i < num_planes; i++)
		vsifb->obj[i] = objs[i];
	ret = drm_framebuffer_init(dev, &vsifb->fb, &hantro_drm_fb_funcs);
	if (ret)
		kfree(vsifb);
	return ret;

err_gem_object_unreference:
	for (i--; i >= 0; i--)
		; //hantro_unref_drmobj(objs[i]);

	return ret;
}

int hantro_fb_create(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct drm_mode_fb_cmd *cmd_or = data;
	struct drm_mode_fb_cmd2 r = {};
	int ret;

	/* convert to new format and call new ioctl */
	r.fb_id = cmd_or->fb_id;
	r.width = cmd_or->width;
	r.height = cmd_or->height;
	r.pitches[0] = cmd_or->pitch;
	r.pixel_format = drm_mode_legacy_fb_format(cmd_or->bpp, cmd_or->depth);
	r.handles[0] = cmd_or->handle;

	ret = hantro_fb_create2(dev, &r, file_priv);
	if (ret)
		return ret;

	cmd_or->fb_id = r.fb_id;

	return 0;
}

int hantro_get_version(struct drm_device *dev, void *data,
			      struct drm_file *file_priv)
{
	struct drm_version *pversion;
	char *sname = DRIVER_NAME;
	char *sdesc = DRIVER_DESC;
	char *sdate = DRIVER_DATE;

	pversion = (struct drm_version *)data;
	pversion->version_major = dev->driver->major;
	pversion->version_minor = dev->driver->minor;
	pversion->version_patchlevel = 0;
	pversion->name_len = strlen(DRIVER_NAME);
	pversion->desc_len = strlen(DRIVER_DESC);
	pversion->date_len = strlen(DRIVER_DATE);
	if (pversion->name)
		if (copy_to_user(pversion->name, sname, pversion->name_len))
			return -EFAULT;
	if (pversion->date)
		if (copy_to_user(pversion->date, sdate, pversion->date_len))
			return -EFAULT;
	if (pversion->desc)
		if (copy_to_user(pversion->desc, sdesc, pversion->desc_len))
			return -EFAULT;
	return 0;
}

int hantro_get_cap(struct drm_device *dev, void *data,
			  struct drm_file *file_priv)
{
	struct drm_get_cap *req = (struct drm_get_cap *)data;

	req->value = 0;
	/*some values should be reset*/
	switch (req->capability) {
	case DRM_CAP_PRIME:
		req->value |= dev->driver->prime_fd_to_handle ?
				      DRM_PRIME_CAP_IMPORT :
				      0;
		req->value |= dev->driver->prime_handle_to_fd ?
				      DRM_PRIME_CAP_EXPORT :
				      0;
		return 0;
	case DRM_CAP_DUMB_BUFFER:
		req->value = 1;
		break;
	case DRM_CAP_VBLANK_HIGH_CRTC:
		req->value = 1;
		break;
	case DRM_CAP_DUMB_PREFERRED_DEPTH:
		req->value = dev->mode_config.preferred_depth;
		break;
	case DRM_CAP_DUMB_PREFER_SHADOW:
		req->value = dev->mode_config.prefer_shadow;
		break;
	case DRM_CAP_ASYNC_PAGE_FLIP:
		req->value = dev->mode_config.async_page_flip;
		break;
	case DRM_CAP_CURSOR_WIDTH:
		if (dev->mode_config.cursor_width)
			req->value = dev->mode_config.cursor_width;
		else
			req->value = 64;
		break;
	case DRM_CAP_CURSOR_HEIGHT:
		if (dev->mode_config.cursor_height)
			req->value = dev->mode_config.cursor_height;
		else
			req->value = 64;
		break;
	case DRM_CAP_ADDFB2_MODIFIERS:
	#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,18,0)) 
		req->value = dev->mode_config.allow_fb_modifiers;
	#else
        req->value = !dev->mode_config.fb_modifiers_not_supported;
	#endif
		break;
	default:
		return -EINVAL;
	}
	return 0;
}

/*just a test API for any purpose*/
int hantro_test(struct drm_device *dev, void *data,
		       struct drm_file *file_priv)
{
	unsigned int *input = data;
	int handle = *input;
	struct drm_gem_object *obj;
	hantro_fence_t *pfence;
	int ret = 10 * HZ; /*timeout*/

	obj = hantro_gem_object_lookup(dev, file_priv, handle);
	if (!obj)
		return -EINVAL;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0))
	pfence = dma_resv_get_excl(obj->dma_buf->resv);
#elif (LINUX_VERSION_CODE < KERNEL_VERSION(5, 19, 0))
	pfence = dma_resv_excl_fence(obj->dma_buf->resv);
#else
    /* TBD for this test API (only for test purpose) */
#endif

	while (ret > 0)
		ret = schedule_timeout(ret);

	hantro_fence_signal(pfence);
	hantro_unref_drmobj(obj);
	return 0;
}

int hantro_getprimeaddr(struct drm_device *dev, void *data,
			       struct drm_file *file_priv)
{
	unsigned long *input = data;
	int fd = *input;
	struct drm_gem_hantro_object *cma_obj;
	struct dma_buf *dma_buf;

	dma_buf = dma_buf_get(fd);
	if (IS_ERR(dma_buf))
		return PTR_ERR(dma_buf);

	cma_obj = (struct drm_gem_hantro_object *)dma_buf->priv;
	*input = cma_obj->paddr;
	dma_buf_put(dma_buf);
	return 0;
}

int hantro_ptr_to_phys(struct drm_device *dev, void *data,
			      struct drm_file *file_priv)
{
	unsigned long *arg = data;
	struct vm_area_struct *vma;
	struct drm_gem_hantro_object *cma_obj;
	unsigned long vaddr = *arg;
	struct drm_gem_object *bo;

	vma = find_vma(current->mm, vaddr);
	if (!vma)
		return -EFAULT;

	bo = (struct drm_gem_object *)vma->vm_private_data;
	cma_obj = to_drm_gem_hantro_obj(bo);;
	if (!cma_obj)
		return -EFAULT;
#if 0
	if (cma_obj->base.dev != dev)
		return -EFAULT;
#endif

	if (vaddr < vma->vm_start ||
	    vaddr >= vma->vm_start + (cma_obj->num_pages << PAGE_SHIFT))
		return -EFAULT;

	*arg = (phys_addr_t)(vaddr - vma->vm_start) + cma_obj->paddr;
	return 0;
}

int hantro_query_metadata(struct drm_device *dev, void *data,
				 struct drm_file *file_priv)
{
	struct hantro_metadata_params *metadata_p =
		(struct hantro_metadata_params *)data;
	struct drm_gem_object *obj = NULL;
	struct drm_gem_hantro_object *cma_obj = NULL;

	obj = hantro_gem_object_lookup(dev, file_priv, metadata_p->handle);
	if (!obj)
		return -ENOENT;
	cma_obj = to_drm_gem_hantro_obj(obj);

	memcpy(&metadata_p->meta_data, &cma_obj->dmapriv.meta_data,
	       sizeof(struct viv_vidmem_metadata));

	hantro_unref_drmobj(obj);

	return 0;
}

int hantro_update_metadata(struct drm_device *dev, void *data,
				  struct drm_file *file_priv)
{
	struct hantro_metadata_params *metadata_p =
		(struct hantro_metadata_params *)data;
	struct drm_gem_object *obj = NULL;
	struct xdx_bo *bo;
	int ret;

	obj = hantro_gem_object_lookup(dev, file_priv, metadata_p->handle);
	if (!obj)
	    return -ENOENT;

	bo = to_xdx_gem(obj);
	ret = xdx_bo_reserve(bo, false);
	if (ret) {
	    dev_err(dev->dev, "(%d) failed to reserve kernel bo\n", ret);

            return ret;
	}

	memcpy(&bo->meta_data, &metadata_p->meta_data, sizeof(struct viv_vidmem_metadata));

	xdx_bo_unreserve(bo);
	hantro_unref_drmobj(obj);

	return 0;
}

int hantro_getmagic(struct drm_device *dev, void *data,
			   struct drm_file *file_priv)
{
	struct drm_auth *auth = data;
	int ret = 0;

	mutex_lock(&dev->struct_mutex);
	if (!file_priv->magic) {
		ret = idr_alloc(&file_priv->master->magic_map, file_priv, 1, 0,
				GFP_KERNEL);
		if (ret >= 0)
			file_priv->magic = ret;
	}
	auth->magic = file_priv->magic;
	mutex_unlock(&dev->struct_mutex);

	return ret < 0 ? ret : 0;
}

int hantro_authmagic(struct drm_device *dev, void *data,
			    struct drm_file *file_priv)
{
	struct drm_auth *auth = data;
	struct drm_file *file;

	mutex_lock(&dev->struct_mutex);
	file = idr_find(&file_priv->master->magic_map, auth->magic);
	DBG("get kmagic %d\n", auth->magic);
	if (file) {
		file->authenticated = 1;
		idr_replace(&file_priv->master->magic_map, NULL, auth->magic);
	}
	mutex_unlock(&dev->struct_mutex);

	return file ? 0 : -EINVAL;
}

#define DRM_IOCTL_DEF(ioctl, _func, _flags)                                    \
	[DRM_IOCTL_NR(ioctl)] = {                                              \
		.cmd = ioctl, .func = _func, .flags = _flags, .name = #ioctl   \
	}

/*after kernel 4.16 this definition is removed*/
#ifndef DRM_CONTROL_ALLOW
#define DRM_CONTROL_ALLOW 0
#endif

#if DRM_CONTROL_ALLOW == 0
#undef DRM_CONTROL_ALLOW
#endif

long hantro_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	int retcode = 0;
	unsigned int nr = _IOC_NR(cmd);
	unsigned int in_size;

	struct xdx_device *xdev = NULL;
	xdev = get_file_xdev(filp);

	in_size = _IOC_SIZE(cmd);

	if (in_size > 0) {
		if (_IOC_DIR(cmd) & _IOC_READ)
			retcode = !hantro_access_ok(VERIFY_WRITE, (void *)arg,
						    in_size);
		else if (_IOC_DIR(cmd) & _IOC_WRITE)
			retcode = !hantro_access_ok(VERIFY_READ, (void *)arg,
						    in_size);
		if (retcode)
			return -EFAULT;
	}

	if (nr >= _IOC_NR(HANTROENC_IOC_START) &&
	    nr <= _IOC_NR(HANTROENC_IOC_END)) {
#ifdef HAS_VC8000E
		return hantroenc_ioctl(filp, cmd, arg);
#else
		if (cmd == HANTROENC_IOCG_CORE_NUM) {
			int corenum = 0;

			__put_user(corenum, (unsigned int *)arg);
			return 0;
		} else {
			return -EFAULT;
		}
#endif
	}

	if (nr >= _IOC_NR(HANTRODEC_IOC_START) &&
	    nr <= _IOC_NR(HANTRODEC_IOC_END)) {
#ifdef HAS_VC8000D
		return hantrodec_ioctl(filp, cmd, arg);
#else
		return -EFAULT;
#endif
	}

	if (nr >= _IOC_NR(HANTROCACHE_IOC_START) &&
	    nr <= _IOC_NR(HANTROCACHE_IOC_END)) {
#ifdef HAS_CACHECORE
		return hantrocache_ioctl(filp, cmd, arg);
#else
		return -EFAULT;
#endif
	}

	if (nr >= _IOC_NR(HANTRODEC400_IOC_START) &&
	    nr <= _IOC_NR(HANTRODEC400_IOC_END)) {
#ifdef HAS_DEC400
		return hantrodec400_ioctl(filp, cmd, arg);
#else
		return -EFAULT;
#endif
	}

	if (nr >= _IOC_NR(HANTROMMU_IOC_START) &&
	    nr <= _IOC_NR(HANTROMMU_IOC_END)) {
#ifdef HAS_MMU
	int tmp = 0;
	if(xdev->asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0){
		if (xdev->vpu.config & CONFIG_HANTROMMU)
			return hantroMMUIoctl(cmd, filp, arg);
		else
			return -EFAULT;
	}
#endif

#ifdef SUPPORT_VPU_SMMU
	if(xdev->asic_type == CHIP_PANGU_B0){
		if (xdev->vpu.config & CONFIG_VPU_SMMU){
			return vpu_smmu_ioctl(cmd, filp, arg);
		}else{
			return -EFAULT;
		}
	}
#endif
	}

	if (nr >= _IOC_NR(VCMD_IOC_START) &&
	    nr <= _IOC_NR(VCMD_IOC_END)) {
		return vcmd_ioctl(filp, cmd, arg);
	}

	printk("invalid hantro ioctl cmd, type:%d nr:%d", _IOC_TYPE(cmd), nr);
	return -EINVAL;
}

int hantro_device_open(struct inode *inode, struct file *filp)
{
	int ret = 0;
	int vcmd_en;
	hantro_ioctl_id ioctl_id_par;

	ret = drm_open(inode, filp);


	ioctl_id_par.ID_PAR.node_idx = 0;
	vcmd_en = hantro_get_vcmdsup(NULL, &ioctl_id_par.data, NULL);
	//vcmd_en = 0;
	if (vcmd_en == 0) {
#if USE_HW == 1 /*hw init*/
#ifdef HAS_VC8000D
	hantrodec_open(inode, filp);
#endif
#ifdef HAS_CACHECORE
	cache_open(inode, filp);
#endif
#endif
	} else {
		hantrovcmd_open(inode, filp);
	}
	return ret;
}

int hantro_device_release(struct inode *inode, struct file *filp)
{
	int vcmd_en;
	int ret = 0;
	hantro_ioctl_id ioctl_id_par;

#ifdef HAS_MMU
	int tmp;
	tmp = 0;
#endif

	ret = drm_release(inode, filp);

	ioctl_id_par.ID_PAR.node_idx = 0;
	vcmd_en = hantro_get_vcmdsup(NULL, &ioctl_id_par.data, NULL);
    //vcmd_en = 0;
    if (vcmd_en == 0) {
#if USE_HW == 1
#ifdef HAS_CACHECORE
	cache_release(filp);
#endif
#ifdef HAS_VC8000D
	hantrodec_release(filp);
#endif
#ifdef HAS_VC8000E
	hantroenc_release();
#endif
#ifdef HAS_MMU
	if(hantro_dev.asic_type == CHIP_PANGU_B0){
#ifdef SUPPORT_VPU_SMMU
		tmp = 1;
#endif
	}
	if(tmp == 0)
		hantroMMURelease(filp);
#endif
#endif
	} else {
		hantrovcmd_release(inode, filp);
	}
	return ret;
}

int hantro_mmap(struct file *filp, struct vm_area_struct *vma)
{
	int ret = 0;
	struct drm_gem_object *obj = NULL;
	struct drm_gem_hantro_object *cma_obj;
	struct drm_vma_offset_node *node;
	unsigned long page_num = (vma->vm_end - vma->vm_start) >> PAGE_SHIFT;
	int sgtidx = 0;
	struct scatterlist *pscatter = NULL;
	struct slice_info *pslice;
	struct device *dev;

	if (mutex_lock_interruptible(&hantro_dev.drm_dev->struct_mutex))
		return -EBUSY;

	drm_vma_offset_lock_lookup(hantro_dev.drm_dev->vma_offset_manager);
	node = drm_vma_offset_exact_lookup_locked(
		hantro_dev.drm_dev->vma_offset_manager, vma->vm_pgoff,
		vma_pages(vma));

	if (likely(node)) {
		obj = container_of(node, struct drm_gem_object, vma_node);
		if (!kref_get_unless_zero(&obj->refcount))
			obj = NULL;
	}
	drm_vma_offset_unlock_lookup(hantro_dev.drm_dev->vma_offset_manager);

	if (!obj) {
		mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
		return -EINVAL;
	}
	hantro_unref_drmobj(obj);
	cma_obj = to_drm_gem_hantro_obj(obj);

	if (page_num > cma_obj->num_pages) {
		mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
		return -EINVAL;
	}
	if (!(cma_obj->flag & HANTRO_GEM_FLAG_IMPORT)) {
		pslice = getslicenode(cma_obj->sliceidx);
		if (!pslice) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return -EINVAL;
		}
		dev = pslice->dev;
	} else {
		dev = obj->dev->dev;
	}
	if ((cma_obj->flag & HANTRO_GEM_FLAG_IMPORT) == 0) {
		if (cma_obj->vaddr == 0) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return -EINVAL;
		}
		ret = drm_gem_mmap_obj(
			obj, drm_vma_node_size(node) << PAGE_SHIFT, vma);

		if (ret) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return ret;
		}
	} else {
		pscatter = &cma_obj->sgt->sgl[sgtidx];
#ifdef __amd64__
		set_memory_uc((unsigned long)cma_obj->vaddr, (int)page_num);
#endif
		vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
		/*else mmap report uncached error for some importer, e.g. i915*/
	}

	if (cma_obj->flag & HANTRO_GEM_FLAG_USEVMALLOC) {
		unsigned int i, j;
		unsigned long uaddr = vma->vm_start;
		void *address = cma_obj->vaddr;
		struct page *pages = NULL;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0))
		vma->vm_flags |= VM_LOCKED;
		vma->vm_flags &= ~VM_PFNMAP;
#else
		vm_flags_set(vma, VM_LOCKED);
		vm_flags_clear(vma, VM_PFNMAP);
#endif

		for (i = 0; i < page_num; i++) {
			pages = (vmalloc_to_page(address));
			if (IS_ERR(pages) || !page_count(pages) ||
			    vm_insert_page(vma, uaddr, pages)) {
				pr_err("alloc page %d fail = %lx:%lx\n", i,
				       (unsigned long)address,
				       (unsigned long)pages);
				address = cma_obj->vaddr;
				for (j = 0; j < i; j++) {
					pages = (vmalloc_to_page(address));
					unref_page(pages);
					address += PAGE_SIZE;
				}
				mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
				return -ENOMEM;
			}
			ref_page(pages);
			uaddr += PAGE_SIZE;
			address += PAGE_SIZE;
		}
	} else {
#ifndef VSI_FPGA
		vma->vm_pgoff = 0;
		if (dma_mmap_coherent(dev, vma, cma_obj->vaddr, cma_obj->paddr,
				      page_num << PAGE_SHIFT)) {
			mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
			return -EAGAIN;
		}
#else
		pgprot_t sPageProt;
		sPageProt = vm_get_page_prot(vma->vm_flags);
		sPageProt = pgprot_writecombine(sPageProt);
		vma->vm_page_prot = sPageProt;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0))
		vma->vm_flags |= VM_IO;
		/* Don't include the mapping in core dumps */
		vma->vm_flags |= VM_DONTDUMP;
		/*
		* Disable mremap because our nopage handler assumes all
		* page requests have already been validated.
		*/
		vma->vm_flags |= VM_DONTEXPAND;
		/* Don't allow mapping to be inherited across a process fork */
		vma->vm_flags |= VM_DONTCOPY;
#else
		vm_flags_set(vma, VM_IO);
		/* Don't include the mapping in core dumps */
		vm_flags_set(vma, VM_DONTDUMP);
		/*
		* Disable mremap because our nopage handler assumes all
		* page requests have already been validated.
		*/
		vm_flags_set(vma, VM_DONTEXPAND);
		/* Don't allow mapping to be inherited across a process fork */
		vm_flags_set(vma, VM_DONTCOPY);
#endif
		remap_pfn_range(vma, vma->vm_start,
				cma_obj->paddr >> PAGE_SHIFT,
				(vma->vm_end - vma->vm_start), vma->vm_page_prot);
#endif
	}
	vma->vm_private_data = obj;
	mutex_unlock(&hantro_dev.drm_dev->struct_mutex);
	return ret;
}

void hantro_gem_vm_close(struct vm_area_struct *vma)
{
	drm_gem_vm_close(vma);
}

#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
int hantro_unload(struct drm_device *dev)
{
	return 0;
}
#else
void hantro_release(struct drm_device *dev)
{
#if (KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE) && (!defined(REDHAT_83))
	drm_dev_unregister(dev);
#elif (KERNEL_VERSION(5, 8, 0) > LINUX_VERSION_CODE) && (!defined(REDHAT_83))
	drm_dev_fini(dev);
#else
	drm_dev_put(hantro_dev.drm_dev);
#endif
}
#endif

int hantro_gem_prime_handle_to_fd(struct drm_device *dev,
					 struct drm_file *filp, uint32_t handle,
					 u32 flags, int *prime_fd)
{
	return drm_gem_prime_handle_to_fd(dev, filp, handle, flags, prime_fd);
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 13, 0))
/*we shall not support page fault. */
int hantro_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
{
	return -EPERM;
}
#elif (LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)) && (!defined(REDHAT_83))
int hantro_vm_fault(struct vm_fault *vmf)
{
	return -EPERM;
}
#else
vm_fault_t hantro_vm_fault(struct vm_fault *vmf)
{
	return (vm_fault_t)(-EPERM);
}
#endif

static const struct vm_operations_struct hantro_drm_gem_cma_vm_ops = {
	.open = drm_gem_vm_open,
	.close = hantro_gem_vm_close,
	.fault = hantro_vm_fault,
};

/*temp no usage now*/
u32 hantro_vblank_no_hw_counter(struct drm_device *dev,
				       unsigned int pipe)
{
	return 0;
}

#if KERNEL_VERSION(5, 13, 0) <= LINUX_VERSION_CODE
static const struct drm_gem_object_funcs hantro_drm_gem_cma_funcs = {
	.export = hantro_prime_export,
	.get_sg_table = hantro_gem_prime_get_sg_table,
	.vmap = hantro_gem_prime_vmap,
	.vunmap = hantro_gem_prime_vunmap,
	.free = hantro_gem_free_object,
	.vm_ops = &hantro_drm_gem_cma_vm_ops,
};
#endif


#if 0
struct drm_driver hantro_drm_driver = {
	//these two are related with controlD and renderD
	.driver_features = DRIVER_GEM | DRIVER_RENDER,
	.get_vblank_counter = hantro_vblank_no_hw_counter,
	.open = hantro_drm_open,
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
	.unload = hantro_unload,
#else
	.release = hantro_release,
#endif
	.dumb_create = hantro_gem_dumb_create_internal,
	.dumb_map_offset = hantro_gem_dumb_map_offset,
#if KERNEL_VERSION(5, 13, 0) > LINUX_VERSION_CODE
	.dumb_destroy = drm_gem_dumb_destroy,
	.gem_prime_export = hantro_prime_export,
	.gem_prime_get_sg_table = hantro_gem_prime_get_sg_table,
	.gem_prime_vmap = hantro_gem_prime_vmap,
	.gem_prime_vunmap = hantro_gem_prime_vunmap,
	.gem_free_object_unlocked = hantro_gem_free_object,
	.gem_vm_ops = &hantro_drm_gem_cma_vm_ops,
#else
  .dumb_destroy = hantro_gem_dumb_destroy,
#endif
	.gem_prime_import = hantro_drm_gem_prime_import,
	.prime_handle_to_fd = hantro_gem_prime_handle_to_fd,
	.prime_fd_to_handle = drm_gem_prime_fd_to_handle,
	.gem_prime_import_sg_table = hantro_gem_prime_import_sg_table,
	//.gem_prime_res_obj = hantro_gem_prime_res_obj,
	.gem_prime_mmap = hantro_gem_prime_mmap,
	.fops = &hantro_fops,
	.name = DRIVER_NAME,
	.desc = DRIVER_DESC,
	.date = DRIVER_DATE,
	.major = DRIVER_MAJOR,
	.minor = DRIVER_MINOR,
};

struct drm_device *create_hantro_drm(struct device *dev)
{
	struct drm_device *ddev;
	int result;

	ddev = drm_dev_alloc(&hantro_drm_driver, dev);
	if (IS_ERR(ddev))
		return ddev;

	ddev->dev = dev;
	drm_mode_config_init(ddev);
	result = drm_dev_register(ddev, 0);
	if (result < 0) {
		drm_dev_unregister(ddev);
		drm_dev_put(ddev);
		return NULL;
	}

	return ddev;
}
#endif

