/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef _HANTROMMU_H_
#define _HANTROMMU_H_

#include "hantro_priv.h"
#include "xdx_device.h"
#define REGION_IN_START 0x0
#define REGION_IN_END 0x40000000u
#define REGION_OUT_START 0x40000000u
#define REGION_OUT_END 0x80000000u
#define REGION_PRIVATE_START 0x80000000u
#define REGION_PRIVATE_END 0xc0000000u

#define REGION_IN_MMU_START 0x1000u
#define REGION_IN_MMU_END 0x40002000u
#define REGION_OUT_MMU_START 0x40002000u
#define REGION_OUT_MMU_END 0x80001000u
#define REGION_PRIVATE_MMU_START 0x80001000u
#define REGION_PRIVATE_MMU_END 0xc0000000u

enum MMUStatus {
	MMU_STATUS_OK = 0,

	MMU_STATUS_FALSE = -1,
	MMU_STATUS_INVALID_ARGUMENT = -2,
	MMU_STATUS_INVALID_OBJECT = -3,
	MMU_STATUS_OUT_OF_MEMORY = -4,
	MMU_STATUS_NOT_FOUND = -19,
};

struct addr_desc {
	void *virtual_address; /* buffer virtual address */
	unsigned long bus_address; /* buffer physical address */
	unsigned long mmu_bus_address;
	unsigned long size; /* physical size */
	unsigned int id; /*SLICE, TYPE*/
	unsigned int handle;
	int codec_type;
};

struct list_binder {
	struct list_head list;
	void *data;
};

/* Init MMU, should be called in driver init function. */
int hantroMMUInit(struct xdx_device *xdev, int type);
int hantroMMUHwInit(struct xdx_device *xdev, int type);
int hantroMMUMap(struct xdx_device *xdev, struct xdx_bo *bo, unsigned long* vaddr, int type);
int hantroMMUprobe(dtbnode *pnode, int loop, struct mmu_t *pmmu,
		   struct xdx_device *xdev, unsigned long ddr_base, unsigned long reg_base);
int vpu_mmu_probe(dtbnode *pnode, int type, struct mmu_t *pmmu,
		   struct xdx_device *xdev, unsigned long ddr_base, unsigned long reg_base);
/* Clean up all data in MMU */
void hantroMMUCleanup(void);
/* The function should be called in driver realease function */
void hantroMMURelease(void *filp);
void vpu_mmu_release(struct file *filp);
/* Memmap the address to hantro mmu */
void hantro_mmu_map(struct addr_desc *addr, int valid_bus_addr);
long hantroMMUIoctl(unsigned int cmd, struct file *filp, unsigned long arg);
/* hw init of MMU */
void MMUEnableStep1(unsigned char *hwregs, struct MMU *mmu);

#endif //#ifndef _HANTROMMU_H_
