/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef _HX280ENC_H_
#define _HX280ENC_H_

#include "hantro_priv.h"
#include "hantro.h"
/*
 * Macros to help debugging
 */

#undef PDEBUG /* undef it, just in case */
#ifdef HX280ENC_DEBUG
#ifdef __KERNEL__
/* This one if debugging is on, and kernel space */
#define PDEBUG(fmt, args...) pr_info("hmp4e: " fmt, ##args)
#else
/* This one for user space */
#define PDEBUG(fmt, args...) printf(__FILE__ ":%d: " fmt, __LINE__, ##args)
#endif
#else
#define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#define IRQF_DISABLED 0x00000020

#define ENC_HW_ID1 0x48320100
#define ENC_HW_ID2 0x80006000
#define ENC_HW_ID3 0x90009100
#define CORE_INFO_MODE_OFFSET 31
#define CORE_INFO_AMOUNT_OFFSET 28

#define ASIC_STATUS_SEGMENT_READY 0x1000
#define ASIC_STATUS_FUSE_ERROR 0x200
#define ASIC_STATUS_SLICE_READY 0x100
#define ASIC_STATUS_LINE_BUFFER_DONE 0x080 /* low latency */
#define ASIC_STATUS_HW_TIMEOUT 0x040
#define ASIC_STATUS_BUFF_FULL 0x020
#define ASIC_STATUS_HW_RESET 0x010
#define ASIC_STATUS_ERROR 0x008
#define ASIC_STATUS_FRAME_READY 0x004
#define ASIC_IRQ_LINE 0x001

#define ASIC_STATUS_ALL                                                        \
	(ASIC_STATUS_SEGMENT_READY | ASIC_STATUS_FUSE_ERROR |                  \
	 ASIC_STATUS_SLICE_READY | ASIC_STATUS_LINE_BUFFER_DONE |              \
	 ASIC_STATUS_HW_TIMEOUT | ASIC_STATUS_BUFF_FULL |                      \
	 ASIC_STATUS_HW_RESET | ASIC_STATUS_ERROR | ASIC_STATUS_FRAME_READY)

long hantroenc_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
int hantroenc_init(void);
int hantroenc_probe(dtbnode *pnode, int useirq, int loop,
		    struct hantroenc_t *penccore);
void hantroenc_cleanup(void);
u32 *hantroenc_getRegAddr(u32 coreid, u32 regid);
u32 hantroenc_readbandwidth(int sliceidx, int isreadBW);
int hantroenc_release(void);

/* IRQ handler */
#if KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE
irqreturn_t hantroenc_isr(int irq, void *dev_id, struct pt_regs *regs);
#else
irqreturn_t hantroenc_isr(int irq, void *dev_id);
#endif

#endif /* !_HX280ENC_H_ */
