/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_DEVICE_H__
#define __XDX_DEVICE_H__

#ifdef pr_fmt
#undef pr_fmt
#endif

#define pr_fmt(fmt) "xdxgpu: " fmt

#ifdef dev_fmt
#undef dev_fmt
#endif

#define dev_fmt(fmt) "xdxgpu: " fmt

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0)) || (defined(REDHAT_83))
#include <drm/drm_device.h>
#include <drm/drm_file.h>
#else
#include <drm/drmP.h>
#endif

#include <drm/drm_vma_manager.h>
#include <linux/atomic.h>
#include <linux/wait.h>
#include <linux/list.h>
#include <linux/kref.h>
#include <linux/rbtree.h>
#include <linux/hashtable.h>
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0)) && !defined(CHROMIUMOS_KERNEL_HAS_DMA_FENCE)
#include <linux/fence.h>
#else
#include <linux/dma-fence.h>
#endif
#include <linux/pci.h>
#include <linux/aer.h>

#include <drm/drm_gem.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 12, 0))
#include <drm/drm_ioctl.h>
#endif
#include <drm/drm.h>
#include <drm/drm_atomic.h>

#include <drm/xdx_asic_type.h>

#include "xdx_irq.h"

#include "xdx_vkms.h"

#include "dm.h"
#include "xdx_vmm.h"
#include "xdx_gfx.h"

#include "xdx_vpu.h"
#include "xdx_vdma.h"
#include "pm/xdx_thm.h"
#include "pm/xdx_cmcu.h"
#include "xdx_acpu.h"
#include "pm/xdx_fw.h"
#include "xdx_shared.h"
#include "xdx_tdma.h"
#include "xdx_smmu.h"

struct xdx_pm {
        /* base */
        struct xdx_device *dev;

        /* notifier */
        struct notifier_block notifier;

        /* clk */
	u32 default_gfxclk;
	u32 current_gfxclk;
	u32 current_gfxvol;
	u32 default_mclk;
	u32 current_mclk;
	u32 default_dispclk;
	u32 current_dispclk;
	u32 current_acpuclk;
	u32 current_hostclk;
};

union xdx_device_flags {
	struct {
		uint32_t run_in_emu:1;
		uint32_t no_hardware:1;
		uint32_t dec_support:1; /* decompression support */
		uint32_t virtual_vblank:1; /* only simulate vblank interrupt */
		uint32_t virtual_display:1; /* only simulate whole display drivers, include dc and hdmi */
		uint32_t virtual_output:1;
		uint32_t fake_mode:1;
		uint32_t reserved:25;
	};
	uint32_t ordinal;
};

struct xdx_reg_resource {
	uint32_t base;
	uint32_t size;
};

struct xdx_device_info {
	uint64_t gddr_dev_phyaddr_base;
	uint64_t host_dev_phyaddr_base;
	uint64_t shareram_dev_phyaddr_base;
	struct xdx_reg_resource gfx_regs;
	struct xdx_reg_resource dc_regs[MAX_OUTPUT_NUM];
	struct xdx_reg_resource dc_mmu_regs[MAX_OUTPUT_NUM];
	struct xdx_reg_resource hdmi_core_regs[MAX_OUTPUT_NUM];
	struct xdx_reg_resource hdmi_tile_regs[MAX_OUTPUT_NUM];
	struct xdx_reg_resource hdmi_pcs_regs[MAX_OUTPUT_NUM];
	struct xdx_reg_resource dp_regs[MAX_OUTPUT_NUM];
	struct xdx_reg_resource dp_pcs_regs[MAX_OUTPUT_NUM];
	uint64_t reserved_vram_size;
	uint32_t alignment;
	uint32_t core_num;
};

struct xdx_soc_funcs {
	void (*hw_init)(struct xdx_device *xdev);
	void (*set_interrupt_funcs)(struct xdx_device *xdev);
	void (*get_device_info)(struct xdx_device *xdev);
	uint32_t (*get_msix_idx)(enum msix_vector id);
};

struct xdx_fwinfo {
	void __iomem *fwinfo;
	resource_size_t fwinfo_base;
	resource_size_t fwinfo_size;
};

struct xdx_device {
	struct device *dev;
#ifndef NO_HARDWARE
	struct pci_dev *pdev;
	struct pci_dev *audio_pdev;
#else
	struct platform_device *pdev;
#endif
	union xdx_device_flags flags;

	struct xdx_soc_funcs soc_funcs;

	struct xdx_device_info hw_info;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0))
	struct drm_device ddev;
#else
	struct drm_device *ddev;
#endif

	struct device *hwmon_dev;
	enum xdx_power_state power_state;
	enum xdx_asic_type asic_type;

	/* register mmio */
	resource_size_t rmmio_base;
	resource_size_t rmmio_size;
	void __iomem *rmmio;

	struct xdx_pm pm;

	struct xdx_irq irq;

	int	vram_mtrr;

	/* FB's physical address in MMIO space (for CPU to
	* map FB). This is different compared to the agp/
	* gart/vram_start/end field as the later is from
	* GPU's view and aper_base is from CPU's view.
	*/
	resource_size_t	aper_base;
	resource_size_t	aper_size;

	/*FW team copy KCCB, FWCCB and some other important 
	 * data structs to share ram to speed up
	 * Refer to internal email @20220817 for details
	 */
	resource_size_t	shareram_base;
	resource_size_t	shareram_size;
	resource_size_t	shareram_useful_size;
	void __iomem *share_ram;
	struct xdx_fw_static_info fw_st_info;
	
	/*
	* Frame buffer aperture of this GPU device.
	 */
	uint64_t vram_start;
	uint64_t vram_end;

	/* gart memory start address of this GPU device */
	uint64_t gtt_start;

	/* share ram start address of this GPU device */
	uint64_t shareram_start;

	/* vram size include visable and invisable */
	uint64_t vram_size;
	uint64_t real_vram_size; /* real used vram*/

	/* used visable vram size, which may be less than aper_size */
	uint64_t visible_vram_size;

	/* gfx */
	struct xdx_gfx gfx;

	/* display */
	struct xdx_dm dm;

	/* vkms */
	struct vkms_device vkmsdev;

	/* acpu */
	struct xdx_acpu acpu;

	/* vpu */
	struct xdx_vpu vpu;

	/* dma dev */
	struct xdx_vdma vdma;
	struct xdx_tdma tdma;

	/* audio dev */
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_hotplug_ptr;
	
	/* cmcu */
	struct xdx_cmcu cmcu;

	/* thermal */
	struct xdx_thermal thm;
	
	/* vdma */
	long vdma_timeout;

	/* status */
	bool enable_runpm;
	bool shutdown;
	atomic_t in_runpm;

	atomic_t in_s3;
	atomic_t in_s4;

	/* fb dev */
	void* fbdev;
	struct xdx_vmm vmm;

	/* PVR Sync debug notify handle */
	void *sync_debug_notify_handle;
#if defined(SUPPORT_NATIVE_FENCE_SYNC)
	/* Only used in fence sync as sync_debug_notify_handle is used
	 * to print a header only. Content is registered separately.
	 * Used to print foreign sync debug
	 */
	void *sync_foreign_debug_notify_handle;
#endif
	/* Flag stating if the device was suspended/resumed. If this is 0 then
	 * the device was either resumed or no suspend was called but if 1 then
	 * the OS called suspend on this device.
	 */
	atomic_t suspended;

	uint32_t num_ip_blocks;
	struct xdx_ip_block *ip_blocks[XDX_IP_BLOCK_TYPE_NUM];
	struct drm_atomic_state *modeset_restore_state;

	struct xdx_fwinfo fwinfo;
	struct xdx_smmu smmu;
	uint32_t dc_heap_size;
};

static inline struct xdx_device *vmm_to_xdev(struct xdx_vmm *vmm)
{
	return container_of(vmm, struct xdx_device, vmm);
}

static inline struct xdx_dm *xdev_to_dm(struct xdx_device *xdev)
{
	return &xdev->dm;
}

static inline struct xdx_device *dm_to_xdev(struct xdx_dm *dm)
{
	return container_of(dm, struct xdx_device, dm);
}

static inline struct xdx_vpu *xdev_to_vpu(struct xdx_device *xdev)
{
	return &xdev->vpu;
}

static inline struct xdx_device *vpu_to_xdev(struct xdx_vpu *vpu)
{
	return container_of(vpu, struct xdx_device, vpu);
}

static inline struct xdx_device *drm_to_xdev(struct drm_device *ddev)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0))
	return container_of(ddev, struct xdx_device, ddev);
#else
	return ddev->dev_private;
#endif
}

static inline struct drm_device *xdev_to_drm(struct xdx_device *xdev)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0))
	return &xdev->ddev;
#else
	return xdev->ddev;
#endif
}

static inline struct xdx_device *vkmsdev_to_xdev(struct vkms_device *vkmsdev)
{
	return container_of(vkmsdev, struct xdx_device, vkmsdev);
}

static inline struct vkms_device *xdev_to_vkmsdev(struct xdx_device *xdev)
{
	return &xdev->vkmsdev;
}

static inline struct xdx_device *vdma_to_xdev(struct xdx_vdma *vdma)
{
	return container_of(vdma, struct xdx_device, vdma);
}

static inline struct xdx_vdma *xdev_to_vdma(struct xdx_device *xdev)
{
	return &xdev->vdma;
}
#ifdef SUPPORT_DMA_TRANSFER_B0 
static inline struct xdx_tdma *xdev_to_tdma(struct xdx_device *xdev)
{
	return &xdev->tdma;
}
#endif
static inline struct pci_dev *xdev_to_audio_pdev(struct xdx_device *xdev)
{
	return pci_get_domain_bus_and_slot(pci_domain_nr(xdev->pdev->bus),
					   xdev->pdev->bus->number,
					   PCI_DEVFN(0, 1));
}
static inline struct xdx_acpu *xdev_to_acpu(struct xdx_device *xdev)
{
	return &xdev->acpu;
}

static inline struct xdx_smmu *xdev_to_smmu(struct xdx_device *xdev)
{
	return &xdev->smmu;
}

static inline enum xdx_asic_type xdev_asic_type(struct xdx_device *xdev)
{
	return xdev->asic_type;
}

/* mmio registers */
uint64_t xdx_mmio_readq(struct xdx_device *xdev, uint32_t offset);
void xdx_mmio_writeq(struct xdx_device *xdev, uint32_t offset,
		     uint64_t v);
uint32_t xdx_mmio_readl(struct xdx_device *xdev, uint32_t offset);
void xdx_mmio_writel(struct xdx_device *xdev, uint32_t offset,
		     uint32_t v);
uint16_t xdx_mmio_readw(struct xdx_device *xdev, uint32_t offset);
void xdx_mmio_writew(struct xdx_device *xdev, uint32_t offset,
                     uint16_t v);
uint8_t xdx_mmio_readb(struct xdx_device *xdev, uint32_t offset);
void xdx_mmio_writeb(struct xdx_device *xdev, uint32_t offset, uint8_t value);
uint32_t xdx_device_rreg(struct xdx_device *xdev, uint32_t reg,
			 uint32_t acc_flags);
void xdx_device_wreg(struct xdx_device *xdev, uint32_t reg, uint32_t v);

bool xdx_device_is_dc_supported(enum xdx_asic_type asic_type);

int xdx_device_init(struct xdx_device *xdev);
int xdx_device_fini(struct xdx_device *xdev);
int xdx_device_ip_suspend(struct xdx_device *xdev);
int xdx_device_suspend(struct xdx_device *xdev, bool fbcon);
int xdx_device_resume(struct xdx_device *xdev, bool fbcon);
int xdx_device_reset(struct xdx_device *xdev);

int xdx_device_ip_block_add(struct xdx_device *xdev,
			    const struct xdx_ip_block *ip_block);

int xdx_device_info_ioctl(struct drm_device *dev, void *data,
			  struct drm_file *filp);

u32 xdx_pci_rebar_get_possible_sizes(struct pci_dev *pdev, int bar);
int xdx_device_resize_fb_bar(struct pci_dev *dev, int resno, u64 rbar_size);

/* SOC related functions */
int pangu_a0_soc_setup(struct xdx_device *xdev);
int pangu_b0_soc_setup(struct xdx_device *xdev);
#endif /* __XDX_DEVICE_H__ */
